@@ -12,7 +12,7 @@
 
 from gi.repository import Gio, GLib, Gtk, Gst
 
-from lollypop.define import App, ScanType, Type
+from lollypop.define import App, ScanType, Type, MARGIN
 
 
 class ApplicationActions:
@@ -24,7 +24,6 @@ class ApplicationActions:
         """
             Init actions
         """
-        self.__user_scan = False
         settings_action = Gio.SimpleAction.new("settings",
                                                GLib.VariantType("i"))
         settings_action.connect("activate", self.__on_settings_activate)
@@ -47,14 +46,6 @@ class ApplicationActions:
         App().set_accels_for_action("app.equalizer", ["<Shift><Alt>e"])
         App().add_action(equalizer_action)
 
-        show_sidebar = App().settings.get_value("show-sidebar")
-        sidebar_action = Gio.SimpleAction.new_stateful(
-            "sidebar",
-            None,
-            GLib.Variant.new_boolean(show_sidebar))
-        sidebar_action.connect("change-state", self.__on_sidebar_change_state)
-        App().add_action(sidebar_action)
-
         about_action = Gio.SimpleAction.new("about", None)
         about_action.connect("activate", self.__on_about_activate)
         App().add_action(about_action)
@@ -80,8 +71,8 @@ class ApplicationActions:
                                              GLib.VariantType.new("s"))
         App().add_action(search_action)
         App().set_accels_for_action("app.search('')", ["<Control>f"])
+        search_action.connect("activate", self.__on_search_activate)
 
-        App().scanner.connect("scan-finished", self.__on_scan_finished)
         self.__setup_global_shortcuts()
         self.enable_special_shortcuts(True)
 
@@ -133,7 +124,6 @@ class ApplicationActions:
         App().set_accels_for_action("app.shortcut::next_album", ["<Control>n"])
         App().set_accels_for_action("app.shortcut::current_artist",
                                     ["<Control><Alt>a"])
-        App().set_accels_for_action("app.shortcut::show_sidebar", ["F9"])
         App().set_accels_for_action("app.update_db", ["<Control>u"])
         App().set_accels_for_action("app.settings(-14)", ["<Control>comma"])
         App().set_accels_for_action("app.fullscreen", ["F11", "F7"])
@@ -148,10 +138,6 @@ class ApplicationActions:
                                     ["<Shift><Alt>Up"])
         App().set_accels_for_action("app.shortcut::volume_down",
                                     ["<Shift><Alt>Down"])
-        App().set_accels_for_action("app.shortcut::list_one_type_ahead",
-                                    ["<Control>o"])
-        App().set_accels_for_action("app.shortcut::list_two_type_ahead",
-                                    ["<Control>t"])
 
     def __on_update_db_activate(self, action=None, param=None):
         """
@@ -160,20 +146,8 @@ class ApplicationActions:
             @param param as GLib.Variant
         """
         if App().window:
-            self.__user_scan = True
             App().scanner.update(ScanType.FULL)
 
-    def __on_scan_finished(self, scanner, modifications):
-        """
-            Run artwork update if needed
-            @param scanner as CollectionScanner
-            @param modifications as bool
-        """
-        if self.__user_scan:
-            if App().settings.get_value("artist-artwork"):
-                App().art.cache_artists_artwork()
-        self.__user_scan = False
-
     def __on_about_activate_response(self, dialog, response_id):
         """
             Destroy about dialog when closed
@@ -182,16 +156,13 @@ class ApplicationActions:
         """
         dialog.destroy()
 
-    def __on_sidebar_change_state(self, action, value):
+    def __on_search_activate(self, action, variant):
         """
-            Show/hide sidebar
             @param action as Gio.SimpleAction
-            @param value as bool
+            @param variant as GLib.Variant
         """
-        action.set_state(value)
-        App().settings.set_value("show-sidebar",
-                                 GLib.Variant("b", value))
-        App().window.container.show_sidebar(value)
+        search = variant.get_string()
+        App().window.container.show_view([Type.SEARCH], search)
 
     def __on_fullscreen_activate(self, action, param):
         """
@@ -228,12 +199,32 @@ class ApplicationActions:
             @param action as Gio.SimpleAction
             @param param as GLib.Variant
         """
+        def get_instance(children, instance):
+            for child in children:
+                if isinstance(child, instance):
+                    return child
+            return None
         builder = Gtk.Builder()
         builder.add_from_resource("/org/gnome/Lollypop/AboutDialog.ui")
         about = builder.get_object("about_dialog")
-        about.set_transient_for(App().window)
-        about.connect("response", self.__on_about_activate_response)
-        about.show()
+        if App().window.is_adaptive:
+            box = Gtk.Box.new(Gtk.Orientation.VERTICAL, MARGIN)
+            box.show()
+            about_children = about.get_children()
+            about_box = get_instance(about_children, Gtk.Box)
+            about_headerbar = get_instance(about_children, Gtk.HeaderBar)
+            about_headerbar.set_show_close_button(False)
+            about_headerbar.get_style_context().add_class("no-border")
+            about.remove(about_headerbar)
+            about.remove(about_box)
+            box.add(about_headerbar)
+            box.add(about_box)
+            box.set_hexpand(True)
+            App().window.container.show_widget(box)
+        else:
+            about.set_transient_for(App().window)
+            about.connect("response", self.__on_about_activate_response)
+            about.show()
 
     def __on_shortcuts_activate(self, action, param):
         """
@@ -289,31 +280,16 @@ class ApplicationActions:
         elif string == "prev":
             App().player.prev()
         elif string == "go_back":
-            App().window.go_back()
-        elif string == "lyrics":
-            App().window.container.show_lyrics()
+            if App().window.toolbar.playback.back_button.get_sensitive():
+                App().window.container.go_back()
         elif string == "reload":
             App().window.container.reload_view()
         elif string == "volume_up":
             App().player.set_volume(App().player.volume + 0.1)
         elif string == "volume_down":
             App().player.set_volume(App().player.volume - 0.1)
-        elif string == "list_one_type_ahead":
-            if App().window.container.list_one.is_visible():
-                App().window.container.list_one.type_ahead_popover.popup()
-        elif string == "list_two_type_ahead":
-            if App().window.container.list_two.is_visible():
-                App().window.container.list_two.type_ahead_popover.popup()
-        elif string == "show_sidebar":
-            value = App().settings.get_value("show-sidebar")
-            App().settings.set_value("show-sidebar",
-                                     GLib.Variant("b", not value))
-            App().main_window.container.show_sidebar(not value)
         elif string == "filter":
-            if App().main_window.container.view is not None:
-                App().main_window.container.view.enable_filter()
-        elif string == "volume":
-            App().window.container.show_view([Type.EQUALIZER])
+            App().window.container.show_filter()
         elif string == "current_artist":
             if App().player.current_track.id is not None and\
                     App().player.current_track.id > 0:
@@ -327,14 +303,13 @@ class ApplicationActions:
                 else:
                     loved = Type.NONE
                 track.set_loved(loved)
-                if App().notify is not None:
-                    if track.loved == 1:
-                        heart = "❤"
-                    elif track.loved == -1:
-                        heart = "⏭"
-                    else:
-                        heart = "♡"
-                    App().notify.send("%s - %s: %s" %
-                                      (", ".join(track.artists),
-                                       track.name,
-                                       heart))
+                if track.loved == 1:
+                    heart = "❤"
+                elif track.loved == -1:
+                    heart = "⏭"
+                else:
+                    heart = "♡"
+                App().notify.send("Lollypop", "%s - %s: %s" %
+                                  (", ".join(track.artists),
+                                   track.name,
+                                   heart))
