@@ -16,25 +16,31 @@ from gi.repository import GLib, GObject, Gio
 from gi.repository.Gio import FILE_ATTRIBUTE_STANDARD_NAME, \
                               FILE_ATTRIBUTE_STANDARD_TYPE, \
                               FILE_ATTRIBUTE_STANDARD_IS_HIDDEN,\
+                              FILE_ATTRIBUTE_STANDARD_IS_SYMLINK,\
+                              FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET,\
                               FILE_ATTRIBUTE_TIME_MODIFIED
 
 from gettext import gettext as _
-from threading import Thread
 from time import time
 import json
+from multiprocessing import cpu_count
 
 from lollypop.inotify import Inotify
-from lollypop.define import App, ScanType, Type
+from lollypop.define import App, ScanType, Type, StorageType
 from lollypop.sqlcursor import SqlCursor
-from lollypop.tagreader import TagReader
+from lollypop.tagreader import TagReader, Discoverer
 from lollypop.logger import Logger
 from lollypop.database_history import History
 from lollypop.utils import is_audio, is_pls, get_mtime, profile, create_dir
+from lollypop.utils import split_list, emit_signal
 
 
-SCAN_QUERY_INFO = "{},{},{},{}".format(FILE_ATTRIBUTE_STANDARD_NAME,
+SCAN_QUERY_INFO = "{},{},{},{},{},{}".format(
+                                       FILE_ATTRIBUTE_STANDARD_NAME,
                                        FILE_ATTRIBUTE_STANDARD_TYPE,
                                        FILE_ATTRIBUTE_STANDARD_IS_HIDDEN,
+                                       FILE_ATTRIBUTE_STANDARD_IS_SYMLINK,
+                                       FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET,
                                        FILE_ATTRIBUTE_TIME_MODIFIED)
 
 
@@ -56,10 +62,12 @@ class CollectionScanner(GObject.GObject, TagReader):
             Init collection scanner
         """
         GObject.GObject.__init__(self)
-        TagReader.__init__(self)
-
         self.__thread = None
         self.__history = History()
+        self.__progress_total = 1
+        self.__progress_count = 0
+        self.__new_tracks = []
+        self.__new_non_album_artists = []
         self.__disable_compilations = True
         if App().settings.get_value("auto-update"):
             self.__inotify = Inotify()
@@ -76,9 +84,12 @@ class CollectionScanner(GObject.GObject, TagReader):
         """
         App().lookup_action("update_db").set_enabled(False)
         # Stop previous scan
-        if self.is_locked() and scan_type != ScanType.EPHEMERAL:
+        if self.is_locked() and scan_type != ScanType.EXTERNAL:
             self.stop()
             GLib.timeout_add(250, self.update, scan_type, uris)
+        elif App().spotify.is_running:
+            App().stop_spotify()
+            GLib.timeout_add(250, self.update, scan_type, uris)
         else:
             self.__disable_compilations = not App().settings.get_value(
                 "show-compilations")
@@ -88,13 +99,12 @@ class CollectionScanner(GObject.GObject, TagReader):
             if not uris:
                 return
             # Register to progressbar
-            if scan_type != ScanType.EPHEMERAL:
+            if scan_type != ScanType.EXTERNAL:
                 App().window.container.progress.add(self)
                 App().window.container.progress.set_fraction(0, self)
+            Logger.info("Scan started")
             # Launch scan in a separate thread
-            self.__thread = Thread(target=self.__scan, args=(scan_type, uris))
-            self.__thread.daemon = True
-            self.__thread.start()
+            self.__thread = App().task_helper.run(self.__scan, scan_type, uris)
 
     def update_album(self, album_id, album_artist_ids,
                      genre_ids, year, timestamp):
@@ -110,11 +120,6 @@ class CollectionScanner(GObject.GObject, TagReader):
             @commit needed
         """
         if album_artist_ids:
-            # Update UI based on previous artist calculation
-            mtime = App().albums.get_mtime(album_id)
-            if mtime != 0:
-                for artist_id in album_artist_ids:
-                    GLib.idle_add(self.emit, "artist-updated", artist_id, True)
             App().albums.set_artist_ids(album_id, album_artist_ids)
         # Set artist ids based on content
         else:
@@ -135,7 +140,7 @@ class CollectionScanner(GObject.GObject, TagReader):
                    album_rate, album_synced, album_mtime, title, duration,
                    tracknumber, discnumber, discname, year, timestamp,
                    track_mtime, track_pop, track_rate, track_loved,
-                   track_ltime, mb_track_id, bpm):
+                   track_ltime, mb_track_id, bpm, storage_type):
         """
             Add track to DB
             @param genres as str/None
@@ -167,23 +172,26 @@ class CollectionScanner(GObject.GObject, TagReader):
             @param track_ltime as int
             @param mb_track_id as str
             @param bpm as int
+            @param storage_type as int
         """
-        Logger.debug(
-            "CollectionScanner::save_track(): Add artists %s" % artists)
-        artist_ids = self.add_artists(artists, a_sortnames, mb_artist_id)
-
         Logger.debug("CollectionScanner::save_track(): "
                      "Add album artists %s" % album_artists)
-        album_artist_ids = self.add_artists(album_artists, aa_sortnames,
-                                            mb_album_artist_id)
+        (added_album_artist_ids,
+         album_artist_ids) = self.add_artists(album_artists, aa_sortnames,
+                                              mb_album_artist_id)
+
+        Logger.debug(
+            "CollectionScanner::save_track(): Add artists %s" % artists)
+        (added_artist_ids,
+         artist_ids) = self.add_artists(artists, a_sortnames, mb_artist_id)
 
         # User does not want compilations
         if self.__disable_compilations and not album_artist_ids:
             album_artist_ids = artist_ids
 
         missing_artist_ids = list(set(album_artist_ids) - set(artist_ids))
-        # https://github.com/gnumdk/lollypop/issues/507#issuecomment-200526942
         # Special case for broken tags
+        # If all artist album tags are missing
         # Can't do more because don't want to break split album behaviour
         if len(missing_artist_ids) == len(album_artist_ids):
             artist_ids += missing_artist_ids
@@ -194,7 +202,7 @@ class CollectionScanner(GObject.GObject, TagReader):
                                                  album_artist_ids,
                                                  uri, album_loved, album_pop,
                                                  album_rate, album_synced,
-                                                 album_mtime)
+                                                 album_mtime, storage_type)
         if genres is None:
             genre_ids = [Type.WEB]
         else:
@@ -206,20 +214,27 @@ class CollectionScanner(GObject.GObject, TagReader):
                                     tracknumber, discnumber, discname,
                                     album_id, year, timestamp, track_pop,
                                     track_rate, track_loved, track_ltime,
-                                    track_mtime, mb_track_id, bpm)
+                                    track_mtime, mb_track_id, bpm,
+                                    storage_type)
         Logger.debug("CollectionScanner::save_track(): Update track")
         self.update_track(track_id, artist_ids, genre_ids)
         Logger.debug("CollectionScanner::save_track(): Update album")
-        SqlCursor.commit(App().db)
         self.update_album(album_id, album_artist_ids,
                           genre_ids, year, timestamp)
         SqlCursor.commit(App().db)
-        for genre_id in genre_ids:
-            # Be sure to not send Type.WEB
-            if genre_id >= 0:
-                GLib.idle_add(self.emit, "genre-updated", genre_id, True)
-        if album_added:
-            GLib.idle_add(self.emit, "album-updated", album_id, True)
+        # We need to keep new artist because we don't know yet if we
+        # are playing with a compilation or an album
+        self.__new_non_album_artists += list(
+            set(added_artist_ids) - set(added_album_artist_ids))
+        if storage_type & StorageType.COLLECTION:
+            for artist_id in added_album_artist_ids:
+                if artist_id in self.__new_non_album_artists:
+                    self.__new_non_album_artists.remove(artist_id)
+                emit_signal(self, "artist-updated", artist_id, True)
+            for genre_id in genre_ids:
+                emit_signal(self, "genre-updated", genre_id, True)
+            if album_added:
+                emit_signal(self, "album-updated", album_id, True)
         return (track_id, album_id)
 
     def update_track(self, track_id, artist_ids, genre_ids):
@@ -274,16 +289,13 @@ class CollectionScanner(GObject.GObject, TagReader):
             App().albums.clean()
             App().genres.clean()
             App().artists.clean()
-            if notify:
-                if App().albums.get_name(album_id) is None:
-                    GLib.idle_add(self.emit, "album-updated",
-                                  album_id, False)
+            SqlCursor.commit(App().db)
+            if notify and App().albums.get_name(album_id) is None:
+                emit_signal(self, "album-updated", album_id, False)
                 for artist_id in album_artist_ids + artist_ids:
-                    GLib.idle_add(self.emit, "artist-updated",
-                                  artist_id, False)
+                    emit_signal(self, "artist-updated", artist_id, False)
                 for genre_id in genre_ids:
-                    GLib.idle_add(self.emit, "genre-updated",
-                                  genre_id, False)
+                    emit_signal(self, "genre-updated", genre_id, False)
             return (track_pop, track_rate, track_ltime, album_mtime,
                     track_loved, album_loved, album_pop, album_rate)
         except Exception as e:
@@ -291,7 +303,8 @@ class CollectionScanner(GObject.GObject, TagReader):
 
     def is_locked(self):
         """
-            Return True if db locked
+            True if db locked
+            @return bool
         """
         return self.__thread is not None and self.__thread.isAlive()
 
@@ -326,12 +339,15 @@ class CollectionScanner(GObject.GObject, TagReader):
             Notify from main thread when scan finished
             @param modifications as bool
         """
+        self.__thread = None
+        Logger.info("Scan finished")
         App().lookup_action("update_db").set_enabled(True)
         App().window.container.progress.set_fraction(1.0, self)
         self.stop()
-        self.emit("scan-finished", modifications)
+        emit_signal(self, "scan-finished", modifications)
         # Update max count value
         App().albums.update_max_count()
+        App().start_spotify()
 
     def __add_monitor(self, dirs):
         """
@@ -376,7 +392,8 @@ class CollectionScanner(GObject.GObject, TagReader):
                        data["duration"], data["tracknumber"],
                        data["discnumber"], data["discname"], data["year"],
                        data["timestamp"], -1, data["track_popularity"],
-                       data["track_rate"], data["track_loved"], 0, "", 0)
+                       data["track_rate"], data["track_loved"], 0, "", 0,
+                       StorageType.SAVED)
             infos.close(None)
         except Exception as e:
             Logger.error("CollectionScanner::__import_web_tracks(): %s", e)
@@ -418,6 +435,12 @@ class CollectionScanner(GObject.GObject, TagReader):
                         child_uri = f.get_uri()
                         if info.get_is_hidden():
                             continue
+                        # We only support symlinks on collections root
+                        elif info.get_is_symlink():
+                            Logger.info(
+                                "Symlinks are not supported: %s",
+                                child_uri)
+                            continue
                         elif info.get_file_type() == Gio.FileType.DIRECTORY:
                             dirs.append(child_uri)
                             walk_uris.append(child_uri)
@@ -443,28 +466,52 @@ class CollectionScanner(GObject.GObject, TagReader):
             @param uris as [str]
             @thread safe
         """
+        self.__new_non_album_artists = []
+        SqlCursor.add(App().db)
+        App().art.clean_rounded()
+
         if not App().tracks.get_mtimes():
             self.__import_web_tracks()
+            SqlCursor.commit(App().db)
 
         (files, dirs) = self.__get_objects_for_uris(scan_type, uris)
 
         if files is None:
-            if App().notify is not None:
-                App().notify.send(_("Scan disabled, missing collection"))
+            App().notify.send("Lollypop",
+                              _("Scan disabled, missing collection"))
             return
 
         if scan_type == ScanType.NEW_FILES:
             db_uris = App().tracks.get_uris(uris)
         else:
             db_uris = App().tracks.get_uris()
-        new_tracks = self.__scan_files(files, db_uris, scan_type)
 
-        if scan_type != ScanType.EPHEMERAL:
+        # Get mtime of all tracks to detect which has to be updated
+        db_mtimes = App().tracks.get_mtimes()
+        self.__progress_total = len(files)
+        self.__progress_count = 0
+        split_files = split_list(files, max(1, cpu_count() // 2))
+        self.__new_tracks = []
+        threads = []
+        for files in split_files:
+            thread = App().task_helper.run(self.__scan_files, files, db_mtimes,
+                                           scan_type)
+            threads.append(thread)
+
+        # Wait for scan to finish
+        for thread in threads:
+            thread.join()
+
+        self.__remove_old_tracks(db_uris, scan_type)
+
+        SqlCursor.remove(App().db)
+
+        if scan_type != ScanType.EXTERNAL:
             self.__add_monitor(dirs)
-            GLib.idle_add(self.__finish, new_tracks)
+            GLib.idle_add(self.__finish, self.__new_tracks)
 
-        if scan_type == ScanType.EPHEMERAL:
-            App().player.play_uris(new_tracks)
+        if scan_type == ScanType.EXTERNAL:
+            App().player.play_uris(self.__new_tracks)
 
     def __scan_to_handle(self, uri):
         """
@@ -475,9 +522,10 @@ class CollectionScanner(GObject.GObject, TagReader):
         try:
             f = Gio.File.new_for_uri(uri)
             # Scan file
-            if App().settings.get_value("import-playlists") and is_pls(f):
-                # Handle playlist
-                App().playlists.import_tracks(f)
+            if is_pls(f):
+                # Import playlist
+                if App().settings.get_value("import-playlists"):
+                    App().playlists.import_tracks(f)
             elif is_audio(f):
                 return True
             else:
@@ -486,86 +534,98 @@ class CollectionScanner(GObject.GObject, TagReader):
             Logger.error("CollectionScanner::__scan_to_handle(): %s" % e)
         return False
 
-    @profile
-    def __scan_files(self, files, db_uris, scan_type):
+    def __scan_files(self, files, db_mtimes, scan_type):
         """
             Scan music collection for new audio files
             @param files as [str]
-            @param db_uris as [str]
+            @param db_mtimes as {}
             @param scan_type as ScanType
             @return new track uris as [str]
             @thread safe
         """
         SqlCursor.add(App().db)
-        i = 0
-        # New tracks present in collection
-        new_tracks = []
-        # Get mtime of all tracks to detect which has to be updated
-        db_mtimes = App().tracks.get_mtimes()
-        count = len(files) + 1
+        discoverer = Discoverer()
         try:
             # Scan new files
             for (mtime, uri) in files:
                 # Handle a stop request
-                if self.__thread is None and scan_type != ScanType.EPHEMERAL:
-                    raise Exception("Scan add cancelled")
+                if self.__thread is None and scan_type != ScanType.EXTERNAL:
+                    raise Exception("cancelled")
                 try:
                     if not self.__scan_to_handle(uri):
                         continue
-                    if mtime > db_mtimes.get(uri, 0):
+                    db_mtime = db_mtimes.get(uri, 0)
+                    if mtime > db_mtime:
                         # If not saved, use 0 as mtime, easy delete on quit
-                        if scan_type == ScanType.EPHEMERAL:
-                            mtime = 0
+                        if scan_type == ScanType.EXTERNAL:
+                            storage_type = StorageType.EXTERNAL
+                        else:
+                            storage_type = StorageType.COLLECTION
                         # Do not use mtime if not intial scan
-                        elif db_mtimes:
+                        if db_mtimes:
                             mtime = int(time())
                         Logger.debug("Adding file: %s" % uri)
-                        self.__add2db(uri, mtime)
-                        SqlCursor.allow_thread_execution(App().db)
-                        new_tracks.append(uri)
+                        self.__add2db(discoverer, uri, mtime, storage_type)
+                        self.__new_tracks.append(uri)
+                    if db_mtime != 0:
+                        del db_mtimes[uri]
                 except Exception as e:
-                    Logger.error(
-                               "CollectionScanner:: __scan_add_files: % s" % e)
-                i += 1
-                self.__update_progress(i, count)
-            if scan_type != ScanType.EPHEMERAL and self.__thread is not None:
-                # We need to check files are always in collections
-                if scan_type == ScanType.FULL:
-                    collections = App().settings.get_music_uris()
-                else:
-                    collections = None
-                for uri in db_uris:
-                    # Handle a stop request
-                    if self.__thread is None:
-                        raise Exception("Scan del cancelled")
-                    in_collection = True
-                    if collections is not None:
-                        in_collection = False
-                        for collection in collections:
-                            if collection in uri:
-                                in_collection = True
-                                break
-                    f = Gio.File.new_for_uri(uri)
-                    if not in_collection or not f.query_exists():
-                        self.del_from_db(uri, True)
-                        SqlCursor.allow_thread_execution(App().db)
+                    Logger.error("Adding file: %s, %s" % (uri, e))
+                self.__progress_count += 1
+                self.__update_progress(self.__progress_count,
+                                       self.__progress_total)
         except Exception as e:
-            Logger.warning("CollectionScanner:: __scan_files: % s" % e)
-        SqlCursor.commit(App().db)
+            Logger.warning("CollectionScanner::__scan_files(): % s" % e)
+        for artist_id in self.__new_non_album_artists:
+            album_ids = App().albums.get_ids([artist_id], [])
+            if album_ids:
+                emit_signal(self, "artist-updated", artist_id, True)
         SqlCursor.remove(App().db)
-        return new_tracks
 
-    def __add2db(self, uri, track_mtime):
+    def __remove_old_tracks(self, uris, scan_type):
+        """
+            Remove non existent tracks from DB
+            @param scan_type as ScanType
+        """
+        if scan_type != ScanType.EXTERNAL and self.__thread is not None:
+            # We need to check files are always in collections
+            if scan_type == ScanType.FULL:
+                collections = App().settings.get_music_uris()
+            else:
+                collections = None
+            for uri in uris:
+                # Handle a stop request
+                if self.__thread is None:
+                    raise Exception("cancelled")
+                in_collection = True
+                if collections is not None:
+                    in_collection = False
+                    for collection in collections:
+                        if collection in uri:
+                            in_collection = True
+                            break
+                f = Gio.File.new_for_uri(uri)
+                if not in_collection:
+                    Logger.warning(
+                        "Removed, not in collection anymore: %s -> %s",
+                        uri, collections)
+                    self.del_from_db(uri, True)
+                elif not f.query_exists():
+                    Logger.warning("Removed, file has been deleted: %s", uri)
+                    self.del_from_db(uri, True)
+
+    def __add2db(self, discoverer, uri,
+                 track_mtime, storage_type=StorageType.COLLECTION):
         """
             Add new file(or update one) to db with information
+            @param discoverer as Discoverer
             @param uri as string
             @param track_mtime as int
-            @return track id as int
-            @warning, be sure SqlCursor is available for App().db
+            @param storage_type as StorageType
         """
         f = Gio.File.new_for_uri(uri)
         Logger.debug("CollectionScanner::add2db(): Read tags")
-        info = self.get_info(uri)
+        info = discoverer.get_info(uri)
         tags = info.get_tags()
         name = f.get_basename()
         title = self.get_title(tags, name)
@@ -636,12 +696,10 @@ class CollectionScanner(GObject.GObject, TagReader):
         if album_mtime == 0:
             album_mtime = track_mtime
 
-        (track_id, album_id) = self.save_track(
-                   genres, artists, a_sortnames, mb_artist_id,
-                   album_artists, aa_sortnames, mb_album_artist_id,
-                   album_name, mb_album_id, uri, album_loved, album_pop,
-                   album_rate, album_synced, album_mtime, title, duration,
-                   tracknumber, discnumber, discname, year, timestamp,
-                   track_mtime, track_pop, track_rate, track_loved,
-                   track_ltime, mb_track_id, bpm)
-        return track_id
+        self.save_track(genres, artists, a_sortnames, mb_artist_id,
+                        album_artists, aa_sortnames, mb_album_artist_id,
+                        album_name, mb_album_id, uri, album_loved, album_pop,
+                        album_rate, album_synced, album_mtime, title, duration,
+                        tracknumber, discnumber, discname, year, timestamp,
+                        track_mtime, track_pop, track_rate, track_loved,
+                        track_ltime, mb_track_id, bpm, storage_type)
