@@ -14,7 +14,7 @@ from gi.repository import GLib
 
 from gettext import gettext as _
 
-from lollypop.define import App, SidebarContent
+from lollypop.define import App, SelectionListMask, Type
 
 
 class ScannerContainer:
@@ -40,8 +40,6 @@ class ScannerContainer:
             @param modifications as bool
         """
         if modifications:
-            self.update_list_one(True)
-            self.update_list_two(True)
             from lollypop.app_notification import AppNotification
             notification = AppNotification(_("New tracks available"),
                                            [_("Refresh")],
@@ -59,13 +57,12 @@ class ScannerContainer:
             @param genre_id as int
             @param add as bool
         """
-        sidebar_content = App().settings.get_enum("sidebar-content")
-        if sidebar_content == SidebarContent.GENRES:
+        if Type.GENRES_LIST in self.sidebar.selected_ids:
             if add:
                 genre_name = App().genres.get_name(genre_id)
-                self._list_one.add_value((genre_id, genre_name, genre_name))
+                self.left_list.add_value((genre_id, genre_name, genre_name))
             elif not App().artists.get_ids([genre_id]):
-                self._list_one.remove_value(genre_id)
+                self.left_list.remove_value(genre_id)
 
     def __on_artist_updated(self, scanner, artist_id, add):
         """
@@ -74,30 +71,23 @@ class ScannerContainer:
             @param artist_id as int
             @param add as bool
         """
+        if Type.GENRES_LIST in self.sidebar.selected_ids:
+            selection_list = self.right_list
+            genre_ids = self.left_list.selected_ids
+        elif Type.ARTISTS_LIST in self.sidebar.selected_ids and\
+                self.left_list.mask & SelectionListMask.ARTISTS:
+            selection_list = self.left_list
+            genre_ids = []
+        else:
+            return
+        artist_ids = App().artists.get_ids(genre_ids)
+        # We only test add, remove and absent is safe
+        if artist_id not in artist_ids and add:
+            return
         artist_name = App().artists.get_name(artist_id)
         sortname = App().artists.get_sortname(artist_id)
         genre_ids = []
-        if App().settings.get_value("show-sidebar"):
-            sidebar_content = App().settings.get_enum("sidebar-content")
-            if sidebar_content == SidebarContent.DEFAULT:
-                return
-            elif sidebar_content == SidebarContent.GENRES:
-                genre_ids = self._list_one.selected_ids
-                l = self._list_two
-                artist_ids = App().artists.get_ids(genre_ids)
-                if artist_id not in artist_ids:
-                    l.remove_value(artist_id)
-                    return
-            else:
-                l = self._list_one
-            if add:
-                l.add_value((artist_id, artist_name, sortname))
-            elif not App().albums.get_ids([artist_id], genre_ids):
-                l.remove_value(artist_id)
-        elif self._rounded_artists_view is not None:
-            if add:
-                self._rounded_artists_view.add_value((artist_id,
-                                                     artist_name,
-                                                     sortname))
-            else:
-                self._rounded_artists_view.remove_value(artist_id)
+        if add:
+            selection_list.add_value((artist_id, artist_name, sortname))
+        elif not App().albums.get_ids([artist_id], genre_ids):
+            selection_list.remove_value(artist_id)
