@@ -11,11 +11,12 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 import itertools
+from time import time
 
 from lollypop.sqlcursor import SqlCursor
-from lollypop.define import App, Type, OrderBy
+from lollypop.define import App, Type, OrderBy, StorageType
 from lollypop.logger import Logger
-from lollypop.utils import noaccents, get_network_available, remove_static
+from lollypop.utils import noaccents, get_default_storage_type, remove_static
 
 
 class AlbumsDatabase:
@@ -28,10 +29,9 @@ class AlbumsDatabase:
             Init albums database object
         """
         self.__max_count = 1
-        self.__cached_randoms = []
 
     def add(self, album_name, mb_album_id, artist_ids,
-            uri, loved, popularity, rate, synced, mtime):
+            uri, loved, popularity, rate, synced, mtime, storage_type):
         """
             Add a new album to database
             @param album_name as str
@@ -43,17 +43,19 @@ class AlbumsDatabase:
             @param rate as int
             @param synced as int
             @param mtime as int
+            @param storage_type as int
             @return inserted rowid as int
             @warning: commit needed
         """
         with SqlCursor(App().db, True) as sql:
             result = sql.execute("INSERT INTO albums\
                                   (name, mb_album_id, no_album_artist,\
-                                  uri, loved, popularity, rate, mtime, synced)\
-                                  VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)",
+                                  uri, loved, popularity, rate, mtime, synced,\
+                                  storage_type)\
+                                  VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
                                  (album_name, mb_album_id or None,
                                   artist_ids == [], uri, loved, popularity,
-                                  rate, mtime, synced))
+                                  rate, mtime, synced, storage_type))
             for artist_id in artist_ids:
                 sql.execute("INSERT INTO album_artists\
                              (album_id, artist_id)\
@@ -179,6 +181,17 @@ class AlbumsDatabase:
             sql.execute("UPDATE albums SET uri=? WHERE rowid=?",
                         (uri, album_id))
 
+    def set_storage_type(self, album_id, storage_type):
+        """
+            Set storage type
+            @param album_id as int
+            @param storage_type as int
+        """
+        with SqlCursor(App().db, True) as sql:
+            sql.execute("UPDATE albums SET storage_type=?\
+                         WHERE rowid=?",
+                        (storage_type, album_id))
+
     def set_popularity(self, album_id, popularity):
         """
             Set popularity
@@ -203,13 +216,13 @@ class AlbumsDatabase:
                        WHERE album_artists.album_id = albums.rowid\
                        AND (album_artists.artist_id = artists.rowid\
                             OR album_artists.artist_id=?)\
-                       AND synced & (1 << ?) AND albums.mtime != 0"
+                       AND synced & (1 << ?) AND albums.storage_type & ?"
             order = " ORDER BY artists.sortname\
                      COLLATE NOCASE COLLATE LOCALIZED,\
                      albums.timestamp,\
                      albums.name\
                      COLLATE NOCASE COLLATE LOCALIZED"
-            filters = (Type.COMPILATIONS, index)
+            filters = (Type.COMPILATIONS, index, StorageType.COLLECTION)
             result = sql.execute(request + order, filters)
             return list(itertools.chain(*result))
 
@@ -217,7 +230,7 @@ class AlbumsDatabase:
         """
             Get album synced status
             @param album_id as int
-            @return synced as int
+            @return int
         """
         with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT synced FROM albums WHERE\
@@ -231,7 +244,7 @@ class AlbumsDatabase:
         """
             Get album loved
             @param album_id as int
-            @return loved as int
+            @return int
         """
         with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT loved FROM albums WHERE\
@@ -242,19 +255,79 @@ class AlbumsDatabase:
                 return v[0]
             return 0
 
-    def get_by_year(self, year):
+    def get_storage_type(self, album_id):
         """
-            Get albums with year
-            @param year as str
+            Get storage type
+            @param album_id as int
+            @return int
         """
         with SqlCursor(App().db) as sql:
-            filters = (year, )
-            request = "SELECT DISTINCT albums.rowid\
+            result = sql.execute("SELECT storage_type FROM albums WHERE\
+                                 rowid=?", (album_id,))
+
+            v = result.fetchone()
+            if v is not None:
+                return v[0]
+            return StorageType.NONE
+
+    def get_for_storage_type(self, storage_type, limit=-1):
+        """
+            Get albums by storage type
+            @param storage_type as StorageType
+            @param limit as int
+            @return [int]
+        """
+        with SqlCursor(App().db) as sql:
+            filters = (storage_type, limit)
+            request = "SELECT rowid\
                        FROM albums\
-                       WHERE year=?"
+                       WHERE storage_type=? ORDER BY RANDOM() LIMIT ?"
             result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
+    def get_newer_for_storage_type(self, storage_type, timestamp):
+        """
+            Get albums newer than timestamp for storage type
+            @param storage_type as StorageType
+            @param timestamp as int
+            @return [int]
+        """
+        with SqlCursor(App().db) as sql:
+            filters = (storage_type, timestamp)
+            request = "SELECT rowid FROM albums\
+                       WHERE storage_type=? and mtime>?"
+            result = sql.execute(request, filters)
+            return list(itertools.chain(*result))
+
+    def get_oldest_for_storage_type(self, storage_type, limit):
+        """
+            Get albums by storage type
+            @param storage_type as StorageType
+            @param limit as int
+            @return [int]
+        """
+        with SqlCursor(App().db) as sql:
+            filters = (storage_type, limit)
+            request = "SELECT rowid FROM albums\
+                       WHERE storage_type=? ORDER BY mtime ASC LIMIT ?"
+            result = sql.execute(request, filters)
+            return list(itertools.chain(*result))
+
+    def get_count_for_storage_type(self, storage_type):
+        """
+            Get albums count for storage type
+            @param storage_type as StorageType
+            @return int
+        """
+        with SqlCursor(App().db) as sql:
+            filters = (storage_type,)
+            request = "SELECT COUNT(*) FROM albums WHERE storage_type=?"
+            result = sql.execute(request, filters)
+            v = result.fetchone()
+            if v is not None:
+                return v[0]
+            return 0
+
     def get_rate(self, album_id):
         """
             Get album rate
@@ -293,7 +366,8 @@ class AlbumsDatabase:
             @raise sqlite3.OperationalError on db update
         """
         with SqlCursor(App().db, True) as sql:
-            result = sql.execute("SELECT popularity from albums WHERE rowid=?",
+            # First increment popularity
+            result = sql.execute("SELECT popularity FROM albums WHERE rowid=?",
                                  (album_id,))
             pop = result.fetchone()
             if pop:
@@ -301,8 +375,26 @@ class AlbumsDatabase:
             else:
                 current = 0
             current += pop_to_add
-            sql.execute("UPDATE albums set popularity=? WHERE rowid=?",
+            sql.execute("UPDATE albums SET popularity=? WHERE rowid=?",
                         (current, album_id))
+            # Then increment timed popularity
+            result = sql.execute("SELECT popularity\
+                                  FROM albums_timed_popularity\
+                                  WHERE album_id=?",
+                                 (album_id,))
+            pop = result.fetchone()
+            mtime = int(time())
+            if pop is not None:
+                popularity = pop[0] + pop_to_add
+                sql.execute("UPDATE albums_timed_popularity\
+                             SET popularity=?, mtime=?\
+                             WHERE album_id=?",
+                            (popularity, mtime, album_id))
+            else:
+                sql.execute("INSERT INTO albums_timed_popularity\
+                             (album_id, popularity, mtime)\
+                             VALUES (?, 1, ?)",
+                            (album_id, mtime))
 
     def get_higher_popularity(self):
         """
@@ -367,17 +459,17 @@ class AlbumsDatabase:
                 return v[0]
             return None
 
-    def get_id_by_name_artists(self, album_name, artist_ids):
+    def get_id_by(self, album_name, artist_ids):
         """
-            Get non compilation album id
-            @param album_name as str
+            Get album for name and artists
+            @param album_name as escaped str
             @param artist_ids as [int]
             @return int
         """
         with SqlCursor(App().db) as sql:
             filters = (album_name,)
             request = "SELECT albums.rowid FROM albums, album_artists\
-                       WHERE name=? COLLATE NOCASE AND\
+                       WHERE sql_escape(name)=? COLLATE NOCASE AND\
                        album_artists.album_id=albums.rowid AND (1=0 "
             filters += tuple(artist_ids)
             for artist_id in artist_ids:
@@ -427,7 +519,7 @@ class AlbumsDatabase:
             v = result.fetchone()
             if v is not None:
                 return v[0]
-            return None
+            return ""
 
     def get_artists(self, album_id):
         """
@@ -536,21 +628,6 @@ class AlbumsDatabase:
             result = sql.execute("SELECT uri FROM albums")
             return list(itertools.chain(*result))
 
-    def get_tracks_count(self, album_id):
-        """
-            Return tracks count
-            @param album_id as int
-            @return count as int
-        """
-        with SqlCursor(App().db) as sql:
-            result = sql.execute("SELECT COUNT(tracks.rowid)\
-                                  FROM tracks WHERE album_id=?",
-                                 (album_id,))
-            v = result.fetchone()
-            if v is not None:
-                return v[0]
-            return 1
-
     def get_rated(self, limit=100):
         """
             Get albums with user rating >= 4
@@ -558,11 +635,12 @@ class AlbumsDatabase:
             @return [int]
         """
         with SqlCursor(App().db) as sql:
+            storage_type = get_default_storage_type()
             request = "SELECT DISTINCT albums.rowid\
                        FROM albums\
-                       WHERE rate>=4 AND loved != -1 AND mtime != 0\
+                       WHERE rate>=4 AND loved != -1 AND storage_type & ?\
                        ORDER BY popularity DESC LIMIT ?"
-            result = sql.execute(request, (limit,))
+            result = sql.execute(request, (storage_type, limit))
             return list(itertools.chain(*result))
 
     def get_populars(self, limit=100):
@@ -572,23 +650,47 @@ class AlbumsDatabase:
             @return [int]
         """
         with SqlCursor(App().db) as sql:
+            storage_type = get_default_storage_type()
             request = "SELECT DISTINCT albums.rowid FROM albums\
-                       WHERE popularity!=0 AND loved != -1 AND mtime != 0\
+                       WHERE popularity!=0 AND loved != -1 AND\
+                       storage_type & ?\
                        ORDER BY popularity DESC LIMIT ?"
-            result = sql.execute(request, (limit,))
+            result = sql.execute(request,
+                                 (storage_type, limit))
             return list(itertools.chain(*result))
 
+    def get_populars_at_the_moment(self, limit=100):
+        """
+            Get popular albums at the moment
+            @param limit as int
+            @return [int]
+        """
+        with SqlCursor(App().db) as sql:
+            storage_type = get_default_storage_type()
+            request = "SELECT DISTINCT albums.rowid\
+                       FROM albums, albums_timed_popularity\
+                       WHERE albums.storage_type & ? AND\
+                             albums.rowid = albums_timed_popularity.album_id\
+                       ORDER BY albums_timed_popularity.popularity DESC\
+                       LIMIT ?"
+            result = sql.execute(request, (storage_type, limit))
+            album_ids = list(itertools.chain(*result))
+            if album_ids:
+                return album_ids
+        return []
+
     def get_loved_albums(self):
         """
             Get loved albums
             @return [int]
         """
         with SqlCursor(App().db) as sql:
+            storage_type = get_default_storage_type()
             request = "SELECT DISTINCT albums.rowid\
                        FROM albums\
                        WHERE loved=1 AND\
-                       mtime != 0 ORDER BY popularity DESC"
-            result = sql.execute(request)
+                       storage_type & ? ORDER BY popularity DESC"
+            result = sql.execute(request, (storage_type,))
             return list(itertools.chain(*result))
 
     def get_recents(self):
@@ -597,36 +699,42 @@ class AlbumsDatabase:
             @return [int]
         """
         with SqlCursor(App().db) as sql:
+            storage_type = get_default_storage_type()
             request = "SELECT DISTINCT albums.rowid FROM albums\
                        WHERE albums.loved != -1 AND\
-                       albums.mtime != 0\
+                       albums.storage_type & ?\
                        ORDER BY mtime DESC LIMIT 100"
-            result = sql.execute(request)
+            result = sql.execute(request, (storage_type,))
             return list(itertools.chain(*result))
 
-    def get_randoms(self):
+    def get_randoms(self, genre_id=None, limit=100):
         """
             Return random albums
+            @param genre_id as int
+            @param limit as int
             @return [int]
         """
-        if self.__cached_randoms:
-            return self.__cached_randoms
         with SqlCursor(App().db) as sql:
             albums = []
-            request = "SELECT DISTINCT albums.rowid FROM albums\
-                       WHERE albums.loved != -1 AND\
-                       albums.mtime != 0 ORDER BY random() LIMIT 100"
-            result = sql.execute(request)
+            storage_type = get_default_storage_type()
+            if genre_id is not None:
+                filter = (storage_type, genre_id, limit)
+                request = "SELECT DISTINCT albums.rowid\
+                           FROM albums, album_genres\
+                           WHERE albums.loved != -1 AND\
+                                 albums.storage_type & ? AND\
+                                 album_genres.album_id = albums.rowid AND\
+                                 album_genres.genre_id = ?\
+                                 ORDER BY random() LIMIT ?"
+            else:
+                filter = (storage_type, limit)
+                request = "SELECT DISTINCT rowid FROM albums\
+                           WHERE loved != -1 AND\
+                           storage_type & ? ORDER BY random() LIMIT ?"
+            result = sql.execute(request, filter)
             albums = list(itertools.chain(*result))
-            self.__cached_randoms = list(albums)
             return albums
 
-    def clear_cached_randoms(self):
-        """
-            Clear cached random albums
-        """
-        self.__cached_randoms = []
-
     def get_disc_names(self, album_id, disc):
         """
             Get disc names
@@ -722,6 +830,42 @@ class AlbumsDatabase:
             result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
+    def get_tracks_count(self, album_id, genre_ids, artist_ids):
+        """
+            Get tracks ids for album
+            @param album_id as int
+            @param genre_ids as [int]
+            @param artist_ids as [int]
+            @return [int]
+        """
+        genre_ids = remove_static(genre_ids)
+        artist_ids = remove_static(artist_ids)
+        with SqlCursor(App().db) as sql:
+            filters = (album_id,)
+            request = "SELECT COUNT(*) FROM tracks"
+            if genre_ids:
+                request += ", track_genres"
+                filters += tuple(genre_ids)
+            if artist_ids:
+                request += ", track_artists"
+                filters += tuple(artist_ids)
+            request += " WHERE album_id=?"
+            if genre_ids:
+                request += " AND track_genres.track_id = tracks.rowid AND ("
+                for genre_id in genre_ids:
+                    request += "track_genres.genre_id=? OR "
+                request += "1=0)"
+            if artist_ids:
+                request += " AND track_artists.track_id=tracks.rowid AND ("
+                for artist_id in artist_ids:
+                    request += "track_artists.artist_id=? OR "
+                request += "1=0)"
+            result = sql.execute(request, filters)
+            v = result.fetchone()
+            if v is not None and v[0] > 0:
+                return v[0]
+            return 1
+
     def get_id_by_uri(self, uri):
         """
             Get album id for uri
@@ -738,17 +882,19 @@ class AlbumsDatabase:
                 return v[0]
             return 0
 
-    def get_ids(self, artist_ids, genre_ids, ignore=False):
+    def get_ids(self, artist_ids, genre_ids, ignore=False, orderby=None):
         """
             Get albums ids
             @param artist_ids as [int]
             @param genre_ids as [int]
             @param ignore as bool => ignore albums with loved == 1
+            @param orderby as OrderBy
             @return albums ids as [int]
         """
         genre_ids = remove_static(genre_ids)
         artist_ids = remove_static(artist_ids)
-        orderby = App().settings.get_enum("orderby")
+        if orderby is None:
+            orderby = App().settings.get_enum("orderby")
         if artist_ids or orderby == OrderBy.ARTIST:
             order = " ORDER BY artists.sortname\
                      COLLATE NOCASE COLLATE LOCALIZED,\
@@ -758,10 +904,14 @@ class AlbumsDatabase:
         elif orderby == OrderBy.NAME:
             order = " ORDER BY albums.name\
                      COLLATE NOCASE COLLATE LOCALIZED"
-        elif orderby == OrderBy.YEAR:
+        elif orderby == OrderBy.YEAR_DESC:
             order = " ORDER BY albums.timestamp DESC,\
                      albums.name\
                      COLLATE NOCASE COLLATE LOCALIZED"
+        elif orderby == OrderBy.YEAR_ASC:
+            order = " ORDER BY albums.timestamp ASC,\
+                     albums.name\
+                     COLLATE NOCASE COLLATE LOCALIZED"
         else:
             order = " ORDER BY albums.popularity DESC,\
                      albums.name\
@@ -769,63 +919,61 @@ class AlbumsDatabase:
 
         with SqlCursor(App().db) as sql:
             result = []
+            storage_type = get_default_storage_type()
             # Get albums for all artists
             if not artist_ids and not genre_ids:
                 request = "SELECT DISTINCT albums.rowid\
                            FROM albums, album_artists, artists\
                            WHERE albums.rowid = album_artists.album_id AND\
-                           albums.mtime!=0 AND\
+                           albums.storage_type & ? AND\
                            artists.rowid = album_artists.artist_id"
                 if ignore:
                     request += " AND albums.loved != -1"
-                if not get_network_available("YOUTUBE"):
-                    request += " AND albums.mtime != -1"
                 request += order
-                result = sql.execute(request)
+                result = sql.execute(request, (storage_type,))
             # Get albums for genres
             elif not artist_ids:
-                filters = tuple(genre_ids)
+                filters = (storage_type,)
+                filters += tuple(genre_ids)
                 request = "SELECT DISTINCT albums.rowid FROM albums,\
                            album_genres, album_artists, artists\
                            WHERE albums.rowid = album_artists.album_id AND\
                            artists.rowid = album_artists.artist_id AND\
-                           albums.mtime!=0 AND\
+                           albums.storage_type & ? AND\
                            album_genres.album_id=albums.rowid AND ( "
                 for genre_id in genre_ids:
                     request += "album_genres.genre_id=? OR "
                 request += "1=0)"
                 if ignore:
                     request += " AND albums.loved != -1"
-                if not get_network_available("YOUTUBE"):
-                    request += " AND albums.mtime != -1"
                 request += order
                 result = sql.execute(request, filters)
             # Get albums for artist
             elif not genre_ids:
-                filters = tuple(artist_ids)
+                filters = (storage_type,)
+                filters += tuple(artist_ids)
                 request = "SELECT DISTINCT albums.rowid\
                            FROM albums, album_artists, artists\
                            WHERE album_artists.album_id=albums.rowid AND\
-                           albums.mtime!=0 AND\
+                           albums.storage_type & ? AND\
                            artists.rowid = album_artists.artist_id AND ("
                 for artist_id in artist_ids:
                     request += "artists.rowid=? OR "
                 request += "1=0)"
                 if ignore:
                     request += " AND albums.loved != -1"
-                if not get_network_available("YOUTUBE"):
-                    request += " AND albums.mtime != -1"
                 request += order
                 result = sql.execute(request, filters)
             # Get albums for artist id and genre id
             else:
-                filters = tuple(artist_ids)
+                filters = (storage_type,)
+                filters += tuple(artist_ids)
                 filters += tuple(genre_ids)
                 request = "SELECT DISTINCT albums.rowid\
                            FROM albums, album_genres, album_artists, artists\
                            WHERE album_genres.album_id=albums.rowid AND\
                            artists.rowid = album_artists.artist_id AND\
-                           albums.mtime!=0 AND\
+                           albums.storage_type & ? AND\
                            album_artists.album_id=albums.rowid AND ("
                 for artist_id in artist_ids:
                     request += "artists.rowid=? OR "
@@ -835,8 +983,6 @@ class AlbumsDatabase:
                 request += "1=0)"
                 if ignore:
                     request += " AND albums.loved != -1"
-                if not get_network_available("YOUTUBE"):
-                    request += " AND albums.mtime != -1"
                 request += order
                 result = sql.execute(request, filters)
             return list(itertools.chain(*result))
@@ -852,28 +998,27 @@ class AlbumsDatabase:
         with SqlCursor(App().db) as sql:
             order = " ORDER BY albums.name, albums.timestamp"
             result = []
+            storage_type = get_default_storage_type()
             # Get all compilations
             if not genre_ids or genre_ids[0] == Type.ALL:
-                filters = (Type.COMPILATIONS,)
+                filters = (storage_type, Type.COMPILATIONS)
                 request = "SELECT DISTINCT albums.rowid\
                            FROM albums, album_artists\
-                           WHERE album_artists.artist_id=?\
-                           AND albums.mtime != 0\
+                           WHERE albums.storage_type & ?\
+                           AND album_artists.artist_id=?\
                            AND album_artists.album_id=albums.rowid"
                 if ignore:
                     request += " AND albums.loved != -1"
-                if not get_network_available("YOUTUBE"):
-                    request += " AND albums.mtime != -1"
                 request += order
                 result = sql.execute(request, filters)
             # Get compilation for genre id
             else:
-                filters = (Type.COMPILATIONS,)
+                filters = (storage_type, Type.COMPILATIONS)
                 filters += tuple(genre_ids)
                 request = "SELECT DISTINCT albums.rowid\
                            FROM albums, album_genres, album_artists\
                            WHERE album_genres.album_id=albums.rowid\
-                           AND albums.mtime != 0\
+                           AND albums.storage_type & ?\
                            AND albums.loved != -1\
                            AND album_artists.album_id=albums.rowid\
                            AND album_artists.artist_id=? AND ( "
@@ -882,8 +1027,6 @@ class AlbumsDatabase:
                 request += "1=0)"
                 if ignore:
                     request += " AND albums.loved != -1"
-                if not get_network_available("YOUTUBE"):
-                    request += " AND albums.mtime != -1"
                 request += order
                 result = sql.execute(request, filters)
             return list(itertools.chain(*result))
@@ -935,14 +1078,15 @@ class AlbumsDatabase:
             @return album ids as [int]
         """
         with SqlCursor(App().db) as sql:
+            storage_type = get_default_storage_type()
             result = sql.execute("SELECT DISTINCT albums.rowid\
                                   FROM albums, tracks\
-                                  WHERE tracks.ltime=0 AND\
-                                  albums.loved != -1 AND\
-                                  albums.mtime != 0 AND\
+                                  WHERE albums.loved != -1 AND\
+                                  albums.storage_type & ? AND\
                                   albums.rowid=tracks.album_id\
-                                  AND albums.popularity < 10\
-                                  ORDER BY random() LIMIT 100")
+                                  AND albums.popularity = 0\
+                                  ORDER BY random() LIMIT 100",
+                                 (storage_type,))
             return list(itertools.chain(*result))
 
     def get_years(self):
@@ -969,12 +1113,13 @@ class AlbumsDatabase:
             @return album ids as [int]
         """
         with SqlCursor(App().db) as sql:
+            storage_type = get_default_storage_type()
             if limit != -1:
                 result = sql.execute("SELECT albums.rowid\
                                       FROM albums\
-                                      WHERE year=? AND mtime != 0\
+                                      WHERE year=? AND storage_type & ?\
                                       ORDER BY random() LIMIT ?",
-                                     (year, limit))
+                                     (year, storage_type, limit))
             else:
                 order = " ORDER BY artists.sortname\
                          COLLATE NOCASE COLLATE LOCALIZED,\
@@ -986,15 +1131,15 @@ class AlbumsDatabase:
                                FROM albums, album_artists, artists\
                                WHERE albums.rowid=album_artists.album_id AND\
                                artists.rowid=album_artists.artist_id AND\
-                               albums.year is null AND albums.mtime != 0"
-                    filter = ()
+                               albums.year is null AND albums.storage_type & ?"
+                    filter = (storage_type,)
                 else:
                     request = "SELECT DISTINCT albums.rowid\
                                FROM albums, album_artists, artists\
                                WHERE albums.rowid=album_artists.album_id AND\
                                artists.rowid=album_artists.artist_id AND\
-                               albums.year=? AND albums.mtime != 0"
-                    filter = (year,)
+                               albums.year=? AND albums.storage_type & ?"
+                    filter = (year, storage_type)
                 request += order
                 result = sql.execute(request, filter)
             return list(itertools.chain(*result))
@@ -1007,14 +1152,16 @@ class AlbumsDatabase:
             @return album ids as [int]
         """
         with SqlCursor(App().db) as sql:
+            storage_type = get_default_storage_type()
             if limit != -1:
                 result = sql.execute("SELECT albums.rowid\
                                       FROM albums, album_artists\
                                       WHERE album_artists.artist_id=?\
                                       AND album_artists.album_id=albums.rowid\
-                                      AND albums.mtime != 0\
+                                      AND albums.storage_type & ?\
                                       AND albums.year=? LIMIT ?",
-                                     (Type.COMPILATIONS, year, limit))
+                                     (Type.COMPILATIONS, year,
+                                      storage_type, limit))
             else:
                 order = " ORDER BY albums.timestamp, albums.name\
                          COLLATE NOCASE COLLATE LOCALIZED"
@@ -1023,39 +1170,52 @@ class AlbumsDatabase:
                                FROM albums, album_artists\
                                WHERE album_artists.artist_id=?\
                                AND album_artists.album_id=albums.rowid\
-                               AND albums.mtime != 0\
+                               AND albums.storage_type & ?\
                                AND albums.year is null"
-                    filter = (Type.COMPILATIONS,)
+                    filter = (Type.COMPILATIONS, storage_type)
                 else:
                     request = "SELECT DISTINCT albums.rowid\
                                FROM albums, album_artists\
                                WHERE album_artists.artist_id=?\
                                AND album_artists.album_id=albums.rowid\
-                               AND albums.mtime != 0\
+                               AND albums.storage_type & ?\
                                AND albums.year=?"
-                    filter = (Type.COMPILATIONS, year)
+                    filter = (Type.COMPILATIONS, storage_type, year)
                 request += order
                 result = sql.execute(request, filter)
             return list(itertools.chain(*result))
 
-    def search(self, searched):
+    def search(self, searched, storage_type):
         """
             Search for albums looking like string
             @param searched as str
+            @param storage_type as StorageType
             @return album ids as [int]
         """
-        no_accents = noaccents(searched)
         with SqlCursor(App().db) as sql:
+            no_accents = noaccents(searched)
             items = []
-            for filter in [(no_accents + "%",),
-                           ("%" + no_accents,),
-                           ("%" + no_accents + "%",)]:
+            # From best filter to worst filter
+            for filter in [(no_accents + "%", storage_type),
+                           ("%" + no_accents, storage_type),
+                           ("%" + no_accents + "%", storage_type)]:
                 request = "SELECT albums.rowid FROM albums\
                            WHERE noaccents(name) LIKE ?\
-                           AND albums.mtime!=0 LIMIT 25"
+                           AND albums.storage_type & ? LIMIT 25"
                 result = sql.execute(request, filter)
                 items += list(itertools.chain(*result))
-            return items
+            for filter in [(no_accents + "%", storage_type),
+                           ("%" + no_accents, storage_type),
+                           ("%" + no_accents + "%", storage_type)]:
+                request = "SELECT DISTINCT albums.rowid\
+                       FROM albums, album_artists, artists\
+                       WHERE album_artists.artist_id=artists.rowid AND\
+                       album_artists.album_id=albums.rowid AND\
+                       noaccents(artists.name) LIKE ? AND\
+                       albums.storage_type & ? LIMIT 25"
+                result = sql.execute(request, filter)
+                items += list(itertools.chain(*result))
+            return list(set(items))
 
     def calculate_artist_ids(self, album_id):
         """
@@ -1098,17 +1258,20 @@ class AlbumsDatabase:
         """
         with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT COUNT(1) FROM albums\
-                                  WHERE mtime !=0")
+                                  WHERE storage_type & ?",
+                                 ((StorageType.COLLECTION |
+                                   StorageType.SAVED,)))
             v = result.fetchone()
             if v is not None:
                 return v[0]
             return 0
 
-    def clean(self):
+    def clean(self, commit=True):
         """
             Clean albums
+            @param commit as bool
         """
-        with SqlCursor(App().db, True) as sql:
+        with SqlCursor(App().db, commit) as sql:
             sql.execute("DELETE FROM albums WHERE albums.rowid NOT IN (\
                             SELECT tracks.album_id FROM tracks)")
             sql.execute("DELETE FROM album_genres\
@@ -1117,6 +1280,14 @@ class AlbumsDatabase:
             sql.execute("DELETE FROM album_artists\
                          WHERE album_artists.album_id NOT IN (\
                             SELECT albums.rowid FROM albums)")
+            sql.execute("DELETE FROM albums_timed_popularity\
+                         WHERE albums_timed_popularity.album_id NOT IN (\
+                            SELECT albums.rowid FROM albums)")
+            # We clear timed popularity based on mtime
+            # For now, we don't need to keep more data than a month
+            month = int(time()) - 2678400
+            sql.execute("DELETE FROM albums_timed_popularity\
+                         WHERE albums_timed_popularity.mtime < ?", (month,))
 
     @property
     def max_count(self):
