@@ -13,10 +13,11 @@
 
 from gettext import gettext as _
 import itertools
+from time import time
 
 from lollypop.sqlcursor import SqlCursor
-from lollypop.define import App, OrderBy
-from lollypop.utils import noaccents, get_network_available, remove_static
+from lollypop.define import App, OrderBy, StorageType, Type
+from lollypop.utils import noaccents, remove_static, get_default_storage_type
 
 
 class TracksDatabase:
@@ -29,11 +30,11 @@ class TracksDatabase:
         """
             Init tracks database object
         """
-        self.__cached_randoms = []
+        pass
 
     def add(self, name, uri, duration, tracknumber, discnumber, discname,
             album_id, year, timestamp, popularity, rate, loved, ltime, mtime,
-            mb_track_id, bpm):
+            mb_track_id, bpm, storage_type):
         """
             Add a new track to database
             @param name as string
@@ -60,8 +61,8 @@ class TracksDatabase:
                 "INSERT INTO tracks (name, uri, duration, tracknumber,\
                 discnumber, discname, album_id,\
                 year, timestamp, popularity, rate, loved,\
-                ltime, mtime, mb_track_id, bpm) VALUES\
-                (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", (
+                ltime, mtime, mb_track_id, bpm, storage_type) VALUES\
+                (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", (
                     name,
                     uri,
                     duration,
@@ -77,7 +78,8 @@ class TracksDatabase:
                     ltime,
                     mtime,
                     mb_track_id,
-                    bpm))
+                    bpm,
+                    storage_type))
             return result.lastrowid
 
     def add_artist(self, track_id, artist_id):
@@ -115,7 +117,7 @@ class TracksDatabase:
             @return track ids as [int]
         """
         with SqlCursor(App().db) as sql:
-            result = sql.execute("SELECT rowid FROM tracks")
+            result = sql.execute("SELECT rowid FROM tracks ORDER BY album_id")
             return list(itertools.chain(*result))
 
     def get_ids_for_name(self, name):
@@ -163,8 +165,8 @@ class TracksDatabase:
 
     def get_id_by(self, name, album_id, artist_ids=[]):
         """
-            Return track id for name/album/artists
-            @param name as str
+            Get track for name, album and artists
+            @param name as escaped str
             @param album_id as int
             @param artist_ids as [int]
             @return track id as int
@@ -173,7 +175,7 @@ class TracksDatabase:
             if artist_ids:
                 filters = (name, album_id) + tuple(artist_ids)
                 request = "SELECT tracks.rowid FROM tracks\
-                           WHERE name = ? COLLATE NOCASE\
+                           WHERE sql_escape(name) = ? COLLATE NOCASE\
                            AND album_id = ?\
                            AND EXISTS (\
                                 SELECT rowid\
@@ -186,7 +188,7 @@ class TracksDatabase:
             else:
                 filters = (name, album_id)
                 request = "SELECT tracks.rowid FROM tracks\
-                           WHERE name = ? COLLATE NOCASE\
+                           WHERE sql_escape(name) = ? COLLATE NOCASE\
                            AND album_id = ?"
             result = sql.execute(request, filters)
             v = result.fetchone()
@@ -208,25 +210,6 @@ class TracksDatabase:
             result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
-    def get_ids_by_performer(self, artist_id):
-        """
-            Return track id for performer
-            @param artist_id as int
-            @return [int]
-        """
-        with SqlCursor(App().db) as sql:
-            filters = (artist_id, artist_id)
-            request = "SELECT tracks.rowid\
-                       FROM tracks, track_artists\
-                       WHERE track_artists.artist_id=? AND\
-                       tracks.rowid = track_artists.track_id AND NOT EXISTS (\
-                        SELECT albums.rowid FROM albums, album_artists\
-                        WHERE albums.rowid=tracks.album_id AND\
-                        album_artists.artist_id = ? AND\
-                        album_artists.album_id = albums.rowid)"
-            result = sql.execute(request, filters)
-            return list(itertools.chain(*result))
-
     def get_name(self, track_id):
         """
             Get track name for track id
@@ -255,6 +238,21 @@ class TracksDatabase:
                 return v[0]
             return None
 
+    def get_storage_type(self, track_id):
+        """
+            Get storage type
+            @param track_id as int
+            @return int
+        """
+        with SqlCursor(App().db) as sql:
+            result = sql.execute("SELECT storage_type FROM tracks WHERE\
+                                 rowid=?", (track_id,))
+
+            v = result.fetchone()
+            if v is not None:
+                return v[0]
+            return 0
+
     def get_timestamp(self, track_id):
         """
             Get track timestamp
@@ -347,6 +345,17 @@ class TracksDatabase:
                          WHERE rowid=?",
                         (uri, track_id))
 
+    def set_storage_type(self, track_id, storage_type):
+        """
+            Set storage type
+            @param track_id as int
+            @param storage_type as int
+        """
+        with SqlCursor(App().db, True) as sql:
+            sql.execute("UPDATE tracks SET storage_type=?\
+                         WHERE rowid=?",
+                        (storage_type, track_id))
+
     def set_rate(self, track_id, rate):
         """
             Set track rate
@@ -483,7 +492,7 @@ class TracksDatabase:
         elif orderby == OrderBy.NAME:
             order = " ORDER BY albums.name\
                      COLLATE NOCASE COLLATE LOCALIZED"
-        elif orderby == OrderBy.YEAR:
+        elif orderby == OrderBy.YEAR_DESC:
             order = " ORDER BY albums.timestamp DESC,\
                      albums.name\
                      COLLATE NOCASE COLLATE LOCALIZED"
@@ -494,57 +503,58 @@ class TracksDatabase:
 
         with SqlCursor(App().db) as sql:
             result = []
+            storage_type = get_default_storage_type()
             # Get albums for all artists
             if not artist_ids and not genre_ids:
                 request = "SELECT DISTINCT tracks.album_id\
                            FROM tracks, track_artists, artists, albums\
                            WHERE tracks.rowid = track_artists.track_id AND\
                            artists.rowid = track_artists.artist_id AND\
+                           track.storage_type & ? AND\
                            album_id=tracks.album_id"
                 if ignore:
                     request += " AND tracks.loved != -1"
-                if not get_network_available("YOUTUBE"):
-                    request += " AND tracks.mtime != -1"
                 request += order
-                result = sql.execute(request)
+                result = sql.execute(request, (storage_type,))
             # Get albums for genres
             elif not artist_ids:
-                filters = tuple(genre_ids)
+                filters = (storage_type,)
+                filters += tuple(genre_ids)
                 request = "SELECT DISTINCT tracks.album_id FROM tracks,\
                            album, track_genres, track_artists, artists\
                            WHERE tracks.rowid = track_artists.track_id AND\
                            artists.rowid = track_artists.artist_id  AND\
                            album_id=tracks.album_id AND\
+                           track.storage_type & ? AND\
                            track_genres.track_id=tracks.rowid AND ( "
                 for genre_id in genre_ids:
                     request += "track_genres.genre_id=? OR "
                 request += "1=0)"
                 if ignore:
                     request += " AND tracks.loved != -1"
-                if not get_network_available("YOUTUBE"):
-                    request += " AND tracks.mtime != -1"
                 request += order
                 result = sql.execute(request, filters)
             # Get albums for artist
             elif not genre_ids:
-                filters = tuple(artist_ids)
+                filters = (storage_type,)
+                filters += tuple(artist_ids)
                 request = "SELECT DISTINCT tracks.album_id\
                            FROM tracks, track_artists, artists, albums\
                            WHERE track_artists.track_id=tracks.rowid AND\
                            album_id=tracks.album_id AND\
+                           track.storage_type & ? AND\
                            artists.rowid = track_artists.artist_id AND ("
                 for artist_id in artist_ids:
                     request += "artists.rowid=? OR "
                 request += "1=0)"
                 if ignore:
                     request += " AND tracks.loved != -1"
-                if not get_network_available("YOUTUBE"):
-                    request += " AND tracks.mtime != -1"
                 request += order
                 result = sql.execute(request, filters)
             # Get albums for artist id and genre id
             else:
-                filters = tuple(artist_ids)
+                filters = (storage_type,)
+                filters += tuple(artist_ids)
                 filters += tuple(genre_ids)
                 request = "SELECT DISTINCT tracks.album_id\
                            FROM tracks, track_genres, albums,\
@@ -552,6 +562,7 @@ class TracksDatabase:
                            WHERE track_genres.track_id=tracks.rowid AND\
                            artists.rowid = track_artists.artist_id AND\
                            album_id=tracks.album_id AND\
+                           track.storage_type & ? AND\
                            track_artists.track_id=tracks.rowid AND ("
                 for artist_id in artist_ids:
                     request += "artists.rowid=? OR "
@@ -561,8 +572,6 @@ class TracksDatabase:
                 request += "1=0)"
                 if ignore:
                     request += " AND tracks.loved != -1"
-                if not get_network_available("YOUTUBE"):
-                    request += " AND tracks.mtime != -1"
                 request += order
                 result = sql.execute(request, filters)
             return list(itertools.chain(*result))
@@ -570,23 +579,48 @@ class TracksDatabase:
     def get_mtimes(self):
         """
             Get mtime for tracks
-            WARNING: Should be called before anything is shown on screen
             @return dict of {uri as string: mtime as int}
         """
         with SqlCursor(App().db) as sql:
             mtimes = {}
             result = sql.execute("SELECT DISTINCT uri, mtime\
-                                  FROM tracks")
+                                  FROM tracks WHERE storage_type & ?",
+                                 (StorageType.COLLECTION,))
             for row in result:
                 mtimes.update((row,))
             return mtimes
 
-    def del_non_persistent(self):
+    def remove_album(self, album_id, commit=True):
+        """
+            Remove album
+            @param album_id as int
+            @param commit as bool
+        """
+        with SqlCursor(App().db, commit) as sql:
+            sql.execute("DELETE FROM tracks WHERE album_id=?", (album_id,))
+
+    def del_non_persistent(self, commit=True):
         """
             Delete non persistent tracks
+            @param commit as bool
         """
+        with SqlCursor(App().db, commit) as sql:
+            sql.execute("DELETE FROM tracks WHERE storage_type & ?",
+                        (StorageType.EPHEMERAL | StorageType.EXTERNAL,))
+
+    def del_old_for_storage_type(self, storage_type, offset=604800):
+        """
+            Delete album id tracks
+            @param storage_type as StorageType
+            @param offset as int
+        """
+        # Last week
+        timestamp = time() - offset
         with SqlCursor(App().db, True) as sql:
-            sql.execute("DELETE FROM tracks WHERE mtime=0")
+            sql.execute("DELETE FROM tracks\
+                         WHERE storage_type=? AND\
+                         mtime < ?",
+                        (storage_type, timestamp))
 
     def get_uris(self, uris_concerned=None):
         """
@@ -601,10 +635,13 @@ class TracksDatabase:
                     result = sql.execute("SELECT uri\
                                           FROM tracks\
                                           WHERE uri LIKE ? AND\
-                                          mtime!=0", (uri + "%",))
+                                          storage_type & ?",
+                                         (uri + "%", StorageType.COLLECTION))
                     uris += list(itertools.chain(*result))
             else:
-                result = sql.execute("SELECT uri FROM tracks WHERE mtime>0")
+                result = sql.execute("SELECT uri FROM tracks\
+                                      WHERE  storage_type & ?",
+                                     (StorageType.COLLECTION,))
                 uris = list(itertools.chain(*result))
             return uris
 
@@ -748,9 +785,11 @@ class TracksDatabase:
             @return int
         """
         with SqlCursor(App().db) as sql:
+            storage_type = get_default_storage_type()
             result = sql.execute("SELECT popularity\
-                                  FROM tracks WHERE mtime != 0\
-                                  ORDER BY POPULARITY DESC LIMIT 1")
+                                  FROM tracks WHERE storage_type & ?\
+                                  ORDER BY POPULARITY DESC LIMIT 1",
+                                 (storage_type,))
             v = result.fetchone()
             if v is not None:
                 return v[0]
@@ -762,11 +801,13 @@ class TracksDatabase:
             @return avarage popularity as int
         """
         with SqlCursor(App().db) as sql:
+            storage_type = StorageType.COLLECTION | StorageType.SAVED
             result = sql.execute("SELECT AVG(popularity)\
                                   FROM (SELECT popularity\
                                         FROM tracks\
-                                        WHERE mtime != 0\
-                                        ORDER BY POPULARITY DESC LIMIT 100)")
+                                        WHERE storage_type & ?\
+                                        ORDER BY POPULARITY DESC LIMIT 100)",
+                                 (storage_type,))
             v = result.fetchone()
             if v and v[0] > 5:
                 return v[0]
@@ -806,10 +847,12 @@ class TracksDatabase:
             @return tracks as [int]
         """
         with SqlCursor(App().db) as sql:
+            storage_type = get_default_storage_type()
             result = sql.execute("SELECT tracks.rowid\
                                   FROM tracks\
-                                  WHERE ltime=0 AND mtime!=0\
-                                  ORDER BY random() LIMIT 100")
+                                  WHERE ltime=0 AND storage_type & ?\
+                                  ORDER BY random() LIMIT 100",
+                                 (storage_type,))
             return list(itertools.chain(*result))
 
     def get_recently_listened_to(self):
@@ -818,10 +861,12 @@ class TracksDatabase:
             @return tracks as [int]
         """
         with SqlCursor(App().db) as sql:
+            storage_type = get_default_storage_type()
             result = sql.execute("SELECT tracks.rowid\
                                   FROM tracks\
-                                  WHERE ltime!=0 AND mtime!=0\
-                                  ORDER BY ltime DESC LIMIT 100")
+                                  WHERE ltime!=0 AND storage_type & ?\
+                                  ORDER BY ltime DESC LIMIT 100",
+                                 (storage_type,))
             return list(itertools.chain(*result))
 
     def get_randoms(self):
@@ -829,22 +874,15 @@ class TracksDatabase:
             Return random tracks
             @return array of track ids as int
         """
-        if self.__cached_randoms:
-            return self.__cached_randoms
         with SqlCursor(App().db) as sql:
+            storage_type = get_default_storage_type()
             result = sql.execute("SELECT tracks.rowid\
-                                  FROM tracks WHERE mtime != 0\
-                                  ORDER BY random() LIMIT 100")
+                                  FROM tracks WHERE storage_type & ?\
+                                  ORDER BY random() LIMIT 100",
+                                 (storage_type,))
             tracks = list(itertools.chain(*result))
-            self.__cached_randoms = list(tracks)
             return tracks
 
-    def clear_cached_randoms(self):
-        """
-            Clear cached random albums
-        """
-        self.__cached_randoms = []
-
     def set_popularity(self, track_id, popularity):
         """
             Set popularity
@@ -940,18 +978,19 @@ class TracksDatabase:
         """
         with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT COUNT(1) FROM tracks\
-                                  WHERE mtime != 0")
+                                  WHERE storage_type & ?",
+                                 (StorageType.COLLECTION | StorageType.SAVED,))
             v = result.fetchone()
             if v is not None:
                 return v[0]
             return 0
 
-    def clean(self):
+    def clean(self, commit=True):
         """
             Clean database for track id
-            @warning commit needed
+            @param commit as bool
         """
-        with SqlCursor(App().db, True) as sql:
+        with SqlCursor(App().db, commit) as sql:
             sql.execute("DELETE FROM track_artists\
                          WHERE track_artists.track_id NOT IN (\
                             SELECT tracks.rowid FROM tracks)")
@@ -959,24 +998,40 @@ class TracksDatabase:
                          WHERE track_genres.track_id NOT IN (\
                             SELECT tracks.rowid FROM tracks)")
 
-    def search(self, searched):
+    def search(self, searched, storage_type):
         """
             Search for tracks looking like searched
             @param searched as str
+            @param storage_type as StorageType
             @return [int]
         """
-        no_accents = noaccents(searched)
         with SqlCursor(App().db) as sql:
+            no_accents = noaccents(searched)
             items = []
-            for filter in [(no_accents + "%",),
-                           ("%" + no_accents,),
-                           ("%" + no_accents + "%",)]:
+            # From best filter to worst filter
+            for filter in [(no_accents + "%", storage_type),
+                           ("%" + no_accents, storage_type),
+                           ("%" + no_accents + "%", storage_type)]:
                 request = "SELECT tracks.rowid FROM tracks\
                            WHERE noaccents(name) LIKE ?\
-                           AND tracks.mtime!=0 LIMIT 25"
+                           AND tracks.storage_type & ? LIMIT 25"
                 result = sql.execute(request, filter)
                 items += list(itertools.chain(*result))
-            return items
+            for filter in [(Type.COMPILATIONS, no_accents + "%", storage_type),
+                           (Type.COMPILATIONS, "%" + no_accents, storage_type),
+                           (Type.COMPILATIONS, "%" + no_accents + "%",
+                            storage_type)]:
+                request = "SELECT DISTINCT tracks.rowid\
+                       FROM track_artists, album_artists, tracks, artists\
+                       WHERE tracks.album_id=album_artists.album_id AND\
+                       album_artists.artist_id != ? AND\
+                       track_artists.artist_id=artists.rowid AND\
+                       track_artists.track_id=tracks.rowid AND\
+                       noaccents(artists.name) LIKE ? AND\
+                       tracks.storage_type & ? LIMIT 25"
+                result = sql.execute(request, filter)
+                items += list(itertools.chain(*result))
+            return list(set(items))
 
     def search_track(self, artist, title):
         """
