@@ -27,6 +27,13 @@ TAG_EDITORS = ["kid3-qt", "exfalso", "easytag", "picard", "puddletag"]
 MARGIN = 15
 MARGIN_SMALL = 5
 
+# All cache goes here
+CACHE_PATH = GLib.get_user_cache_dir() + "/lollypop"
+# Fallback when album dir is readonly
+STORE_PATH = GLib.get_user_data_dir() + "/lollypop/store"
+# Store for Web
+TMP_PATH = GLib.get_user_data_dir() + "/lollypop/tmp"
+
 
 class Repeat:
     NONE = 0
@@ -41,19 +48,30 @@ class GstPlayFlags:
     GST_PLAY_FLAG_TEXT = 1 << 3   # We want subtitle output
 
 
+class StorageType:
+    NONE = 1 << 0
+    COLLECTION = 1 << 1
+    EPHEMERAL = 1 << 2
+    SAVED = 1 << 3
+    SPOTIFY_NEW_RELEASES = 1 << 4
+    SPOTIFY_SIMILARS = 1 << 5
+    EXTERNAL = 1 << 6
+
+
 class ArtBehaviour:
     NONE = 1 << 0
     ROUNDED = 1 << 1
-    BLUR = 1 << 2
-    BLUR_HARD = 1 << 3
-    BLUR_MAX = 1 << 4
-    FALLBACK = 1 << 5
-    DARKER = 1 << 6
-    LIGHTER = 1 << 7
-    CROP = 1 << 8
-    CROP_SQUARE = 1 << 9
-    CACHE = 1 << 10
-    NO_CACHE = 1 << 11
+    ROUNDED_BORDER = 1 << 2
+    BLUR = 1 << 3
+    BLUR_HARD = 1 << 4
+    BLUR_MAX = 1 << 5
+    FALLBACK = 1 << 6
+    DARKER = 1 << 7
+    LIGHTER = 1 << 8
+    CROP = 1 << 9
+    CROP_SQUARE = 1 << 10
+    CACHE = 1 << 11
+    NO_CACHE = 1 << 12
 
 
 class ViewType:
@@ -62,16 +80,14 @@ class ViewType:
     DND = 1 << 2
     SEARCH = 1 << 3
     PLAYLISTS = 1 << 4
-    POPOVER = 1 << 5
-    ALBUM = 1 << 6
-    NAVIGATION = 1 << 7
-    SMALL = 1 << 8
-    MEDIUM = 1 << 9
-    SCROLLED = 1 << 10
-    FILTERED = 1 << 11
-    FULLSCREEN = 1 << 12
-    DEVICES = 1 << 13
-    NOT_ADAPTIVE = 1 << 14
+    ALBUM = 1 << 5
+    SMALL = 1 << 6
+    ADAPTIVE = 1 << 7
+    SCROLLED = 1 << 8
+    OVERLAY = 1 << 9
+    FULLSCREEN = 1 << 10
+    PLAYBACK = 1 << 11
+    BANNER = 1 << 12
 
 
 NetworkAccessACL = {
@@ -88,10 +104,17 @@ NetworkAccessACL = {
     "DEEZER": 1 << 11,
     "WIKIA": 1 << 12,
     "GENIUS": 1 << 13,
-    "AUDIODB": 1 << 14
+    "AUDIODB": 1 << 14,
+    "FANARTTV": 1 << 15
 }
 
 
+class LoadingState:
+    NONE = 0
+    ABORTED = 1
+    FINISHED = 2
+
+
 class IndicatorType:
     NONE = 1 << 0
     PLAY = 1 << 1
@@ -101,47 +124,37 @@ class IndicatorType:
 
 
 class ArtSize:
-    NONE = 0
-    SMALL = 33
-    MEDIUM = 48  # If changed, adapt width request in AlbumRow
-    ARTIST_SMALL = 60
-    LARGE = 100
+    SMALL = 50
+    MEDIUM = 100
     BANNER = 150
     BIG = 200
     MINIPLAYER = 300
     FULLSCREEN = 400
-    ARTIST = 800
-    MONSTER = 900
-    MAX = 4000
+    MPRIS = 900
 
 
 class ScanType:
-    EPHEMERAL = 0
+    EXTERNAL = 0
     NEW_FILES = 1
     FULL = 2
 
 
-class SidebarContent:
-    DEFAULT = 0
-    GENRES = 1
-    ARTISTS = 2
-
-
 class SelectionListMask:
-    LIST_ONE = 1 << 1
-    LIST_TWO = 1 << 2
-    LIST_DEVICE = 1 << 3
-    ARTISTS = 1 << 4
-    GENRES = 1 << 5
-    PLAYLISTS = 1 << 6
-    COMPILATIONS = 1 << 7
-    ARTISTS_VIEW = 1 << 8
+    NONE = 1 << 0
+    SIDEBAR = 1 << 1
+    VIEW = 1 << 2
+    ARTISTS = 1 << 3
+    GENRES = 1 << 4
+    PLAYLISTS = 1 << 5
+    COMPILATIONS = 1 << 6
+    LABEL = 1 << 7
+    ELLIPSIZE = 1 << 8
 
 
-class Shuffle:
-    NONE = 0             # No shuffle
-    TRACKS = 1           # Shuffle by tracks on genre
-    ALBUMS = 2           # Shuffle by albums on genre
+class Notifications:
+    NONE = 0
+    ALL = 1
+    MPRIS = 2
 
 
 class PowerManagement:
@@ -151,54 +164,69 @@ class PowerManagement:
     BOTH = 3             # Inhibit screensaver and suspend
 
 
-class Sizing:
-    MINI = 200
-    SMALL = 350
-    MEDIUM = 500
-    BIG = 700
-    MONSTER = 800
-    LARGE = 1000
+class AdaptiveSize:
+    NONE = 1 << 0
+    SMALL = 1 << 1
+    MEDIUM = 1 << 2
+    NORMAL = 1 << 3
+    BIG = 1 << 4
+    LARGE = 1 << 5
+
+
+class Size:
+    MINI = 250
+    SMALL = 400
+    MEDIUM = 720  # Librem Phone
+    NORMAL = 1000
+    BIG = 1300
 
 
 class OrderBy:
     ARTIST = 0
     NAME = 1
-    YEAR = 2
+    YEAR_DESC = 2
     POPULARITY = 3
+    YEAR_ASC = 4  # Not available for user
+    GENRE = 5
 
 
 # Order is important
 class Type:
     NONE = -1
-    POPULARS = -2
-    RANDOMS = -3
-    RECENTS = -4
-    LOVED = -5
-    NEVER = -6
-    PLAYLISTS = -7
-    RADIOS = -8
-    SMART = -9
-    SEARCH = -10
-    INFO = -11
-    YEARS = -12
-    ARTISTS = -13
-    SETTINGS = -14
-    SETTINGS_APPEARANCE = -15
-    SETTINGS_BEHAVIOUR = -16
-    SETTINGS_COLLECTIONS = -17
-    SETTINGS_WEB = -18
-    SETTINGS_DEVICES = -19
-    EQUALIZER = -20
-    GENRES = -21
+    SUGGESTIONS = -2
+    POPULARS = -3
+    RANDOMS = -4
+    RECENTS = -5
+    LOVED = -6
+    NEVER = -7
+    ALL = -8
     # WEB is stored in DB, can't be changed
-    WEB = -22
-    ALBUM = -23
-    ALL = -100
+    WEB = -9
+    # Stored in DB, can't be changed
+    COMPILATIONS = -10
+    ARTISTS = -11
+    ARTISTS_LIST = -12
+    GENRES = -13
+    GENRES_LIST = -14
+    YEARS = -15
+    PLAYLISTS = -16
+    RADIOS = -17
+    INFO = -18
+    SMART = -19
+    EQUALIZER = -20
+    SETTINGS = -21
+    SETTINGS_APPEARANCE = -22
+    SETTINGS_BEHAVIOUR = -23
+    SETTINGS_COLLECTIONS = -24
+    SETTINGS_WEB = -25
+    SETTINGS_DEVICES = -26
+    DEVICE_ALBUMS = -27
+    DEVICE_PLAYLISTS = -28
+    ALBUM = -29
+    SEPARATOR = -100
     CURRENT = -101
-    DEVICE_ALBUMS = -1000
-    DEVICE_PLAYLISTS = -1001
-    SEPARATOR = -2000
-    COMPILATIONS = -2001
+    SEARCH = -102
+    LYRICS = -103
 
 
 LATIN1_ENCODING = b"\x00"
@@ -214,6 +242,8 @@ UTF_8_ENCODING = b"\x03"
 SPOTIFY_CLIENT_ID = "0b144843878a46b2b12e0958c342c3ac"
 SPOTIFY_SECRET = "265ab8e057684f1b9e69e0c58f4881c1"
 AUDIODB_CLIENT_ID = "195003"
+FANARTTV_ID = "1cb10bc910c8d4fc34e0c78ac4e8ef46"
+
 
 STATIC_ALBUM_NAME = {
     Type.POPULARS: "Popular albums",
