@@ -14,9 +14,7 @@ from gi.repository import Gio, Gtk
 
 from gettext import gettext as _
 
-from lollypop.define import App
-from lollypop.logger import Logger
-from lollypop.menu_sync import SyncPlaylistsMenu
+from lollypop.define import App, MARGIN_SMALL
 
 
 class PlaylistMenu(Gio.Menu):
@@ -24,36 +22,40 @@ class PlaylistMenu(Gio.Menu):
         A playlist menu
     """
 
-    def __init__(self, playlist_id):
+    def __init__(self, playlist_id, header=False):
         """
             Init variable
             @param playlist_id as int
+            @param header as bool
         """
         Gio.Menu.__init__(self)
         self.__playlist_id = playlist_id
+        if header:
+            from lollypop.menu_header import RoundedMenuHeader
+            name = App().playlists.get_name(playlist_id)
+            artwork_name = "playlist_%s" % name
+            self.append_item(RoundedMenuHeader(name, artwork_name))
+        menu = Gio.Menu()
         save_action = Gio.SimpleAction(name="save_pl_action")
         App().add_action(save_action)
         save_action.connect("activate", self.__on_save_action_activate)
-        self.append(_("Save playlist"), "app.save_pl_action")
+        menu.append(_("Save playlist"), "app.save_pl_action")
         smart_action = Gio.SimpleAction(name="smart_action")
         App().add_action(smart_action)
         smart_action.connect("activate", self.__on_smart_action_activate)
-        self.append(_("Manage smart playlist"), "app.smart_action")
-        if not App().window.is_adaptive:
-            split_action = Gio.SimpleAction.new_stateful(
-                "split_action",
-                None,
-                App().settings.get_value("split-view"))
-            split_action.connect("change-state",
-                                 self.__on_split_action_change_state)
-            App().add_action(split_action)
-            self.append(_("Split view"), "app.split_action")
+        menu.append(_("Manage smart playlist"), "app.smart_action")
         remove_action = Gio.SimpleAction(name="remove_pl_action")
         App().add_action(remove_action)
         remove_action.connect("activate", self.__on_remove_action_activate)
-        self.append(_("Remove playlist"), "app.remove_pl_action")
-        self.append_section(_("Synchronization"),
-                            SyncPlaylistsMenu(playlist_id))
+        menu.append(_("Remove playlist"), "app.remove_pl_action")
+        from lollypop.menu_playback import PlaylistPlaybackMenu
+        from lollypop.menu_sync import SyncPlaylistsMenu
+        self.append_section(_("Playback"), PlaylistPlaybackMenu(playlist_id))
+        section = Gio.Menu()
+        self.append_section(_("Add to"), section)
+        section.append_submenu(_("Devices"),
+                               SyncPlaylistsMenu(playlist_id))
+        self.append_section(_("Edit"), menu)
 
 #######################
 # PRIVATE             #
@@ -109,27 +111,46 @@ class PlaylistMenu(Gio.Menu):
             @param dialog as Gtk.NativeDialog
             @param response_id as int
         """
-        try:
-            if response_id == Gtk.ResponseType.ACCEPT:
-                uris = App().playlists.get_track_uris(self.__playlist_id)
-                stream = dialog.get_file().replace(
-                    None,
-                    False,
-                    Gio.FileCreateFlags.REPLACE_DESTINATION,
-                    None)
-                stream.write("#EXTM3U\n".encode("utf-8"))
-                for uri in uris:
-                    string = "%s\n" % uri
-                    stream.write(string.encode("utf-8"))
-                stream.close()
-        except Exception as e:
-            Logger.error("PlaylistMenu::__on_save_response(): %s", e)
+        if response_id == Gtk.ResponseType.ACCEPT:
+            uri = dialog.get_file().get_uri()
+            App().playlists.set_sync_uri(self.__playlist_id, uri)
+            App().playlists.sync_to_disk(self.__playlist_id, True)
 
-    def __on_split_action_change_state(self, action, value):
+
+class PlaylistMenuExt(Gtk.Grid):
+    """
+        Additional widgets for playlist menu
+    """
+
+    def __init__(self, playlist_id):
         """
-            Activate party mode
-            @param action as Gio.SimpleAction
-            @param value as bool
+            Init widget
+            @param playlist_id as int
         """
-        App().settings.set_value("split-view", value)
-        action.set_state(value)
+        Gtk.Grid.__init__(self)
+        self.set_margin_top(MARGIN_SMALL)
+        self.set_row_spacing(MARGIN_SMALL)
+        self.set_orientation(Gtk.Orientation.VERTICAL)
+
+        entry = Gtk.Entry()
+        entry.set_margin_top(MARGIN_SMALL)
+        entry.set_margin_start(MARGIN_SMALL)
+        entry.set_margin_end(MARGIN_SMALL)
+        entry.set_margin_bottom(MARGIN_SMALL)
+        entry.set_property("hexpand", True)
+        entry.set_text(App().playlists.get_name(playlist_id))
+        entry.connect("changed", self.__on_entry_changed, playlist_id)
+        entry.show()
+        self.add(entry)
+
+#######################
+# PRIVATE             #
+#######################
+    def __on_entry_changed(self, entry, playlist_id):
+        """
+            Update playlist name
+            @param entry as Gtk.Entry
+            @param playlist_id as int
+        """
+        new_name = entry.get_text()
+        App().playlists.rename(playlist_id, new_name)
