@@ -19,15 +19,17 @@ from lollypop.player_queue import QueuePlayer
 from lollypop.player_linear import LinearPlayer
 from lollypop.player_shuffle import ShufflePlayer
 from lollypop.player_radio import RadioPlayer
-from lollypop.player_playlist import PlaylistPlayer
 from lollypop.player_similars import SimilarsPlayer
 from lollypop.radios import Radios
 from lollypop.logger import Logger
-from lollypop.objects import Track, Album
-from lollypop.define import App, Type, LOLLYPOP_DATA_PATH, Shuffle
+from lollypop.objects_track import Track
+from lollypop.objects_album import Album
+from lollypop.objects_radio import Radio
+from lollypop.define import App, Type, LOLLYPOP_DATA_PATH
+from lollypop.utils import emit_signal
 
 
-class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
+class Player(BinPlayer, QueuePlayer, RadioPlayer,
              LinearPlayer, ShufflePlayer, SimilarsPlayer):
     """
         Player object used to manage playback and playlists
@@ -41,21 +43,19 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         QueuePlayer.__init__(self)
         LinearPlayer.__init__(self)
         ShufflePlayer.__init__(self)
-        PlaylistPlayer.__init__(self)
         RadioPlayer.__init__(self)
         SimilarsPlayer.__init__(self)
         self.__stop_after_track_id = None
         self.update_crossfading()
-        App().settings.connect("changed::repeat", self.__on_repeat_changed)
-        self._albums_backup = []
+        App().settings.connect("changed::repeat", self.update_next_prev)
 
     def prev(self):
         """
             Play previous track
         """
-        if App().window.toolbar.playback.seek_wanted:
+        if self.position / Gst.SECOND > 2:
             self.seek(0)
-            self.emit("current-changed")
+            emit_signal(self, "current-changed")
             if not self.is_playing:
                 self.play()
         elif self._prev_track.id is not None:
@@ -79,40 +79,27 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
             @param track as Track
             @param play as bool, ignored for radios
         """
-        if track.id == Type.RADIOS:
+        if isinstance(track, Radio):
             RadioPlayer.load(self, track, play)
         else:
             if play:
                 BinPlayer.load(self, track)
             else:
                 BinPlayer._load_track(self, track)
-                self.emit("current-changed")
+                emit_signal(self, "current-changed")
 
-    def add_album(self, album, index=-1):
+    def add_album(self, album):
         """
-            Add album to player. We may merge album!
+            Add album to player
             @param album as Album
-            @param index as int
         """
-        # We are not playing a user playlist anymore
-        self._playlist_tracks = []
-        self._playlist_ids = []
-        # We do not shuffle when user add an album
-        self._albums_backup = []
-        if index == -1:
-            if self._albums and self._albums[-1].id == album.id:
-                self._albums[-1].set_tracks(self._albums[-1].tracks +
-                                            album.tracks)
-            else:
-                self._albums.append(album)
+        # Merge album if previous is same
+        if self._albums and self._albums[-1].id == album.id:
+            tracks = list(set(self._albums[-1].tracks) | set(album.tracks))
+            self._albums[-1].set_tracks(tracks)
         else:
-            self._albums.insert(index, album)
-        if self._current_track.id is not None and self._current_track.id > 0:
-            if not self.is_party:
-                self.set_next()
-            self.set_prev()
-        self.emit("album-added", album.id)
-        self.emit("playlist-changed")
+            self._albums.append(album)
+        emit_signal(self, "playback-changed")
 
     def remove_album(self, album):
         """
@@ -120,14 +107,14 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
             @param album as Album
         """
         try:
+            if album not in self._albums:
+                return
+            if self._current_track.album == album:
+                self.skip_album()
+            else:
+                self.update_next_prev()
             self._albums.remove(album)
-            if album in self._albums_backup:
-                self._albums_backup.remove(album)
-            if not self.is_party or self._next_track.album_id == album.id:
-                self.set_next()
-            self.set_prev()
-            self.emit("album-removed", album.id)
-            self.emit("playlist-changed")
+            emit_signal(self, "playback-changed")
         except Exception as e:
             Logger.error("Player::remove_album(): %s" % e)
 
@@ -140,155 +127,55 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
             for album in self._albums:
                 if album.id == album_id:
                     self.remove_album(album)
-            self.emit("playlist-changed")
+            emit_signal(self, "playback-changed")
         except Exception as e:
             Logger.error("Player::remove_album_by_id(): %s" % e)
 
-    def remove_disc(self, disc, album_id):
-        """
-            Remove disc for album_id
-            @param disc as Disc
-            @param album_id as int
-        """
-        try:
-            removed = []
-            for album in self._albums:
-                if album.id == album_id:
-                    for track in list(album.tracks):
-                        if track.id in disc.track_ids:
-                            empty = album.remove_track(track)
-                            if empty:
-                                removed.append(album)
-            for album in removed:
-                self._albums.remove(album)
-            self.emit("playlist-changed")
-        except Exception as e:
-            Logger.error("Player::remove_disc(): %s" % e)
-
     def play_album(self, album):
         """
             Play album
             @param album as Album
         """
+        self.play_album_for_albums(album, [album])
+
+    def play_track_for_albums(self, track, albums):
+        """
+            Play track and set albums as current playlist
+            @param albums as [Album]
+            @param track as Track
+        """
         if self.is_party:
             App().lookup_action("party").change_state(GLib.Variant("b", False))
-        self.reset_history()
-        # We are not playing a user playlist anymore
-        self._playlist_tracks = []
-        self._playlist_ids = []
-        if App().settings.get_enum("shuffle") == Shuffle.TRACKS:
-            track = choice(album.tracks)
-        else:
-            track = album.tracks[0]
+        self._albums = albums
         self.load(track)
-        self._albums = [album]
-        self.emit("playlist-changed")
+        emit_signal(self, "playback-changed")
 
-    def play_albums(self, album_id, filter1_ids, filter2_ids):
+    def play_album_for_albums(self, album, albums):
         """
-            Play albums related to track/genre_ids/artist_ids
-            @param album_id as int/None
-            @param filter1_ids as [int]
-            @param filter2_ids as [int]
+            Play album and set albums as current playlist
+            @param album as Album
+            @param albums as [Album]
         """
-        self._albums = []
-        album_ids = []
-        self.reset_history()
-        # We are not playing a user playlist anymore
-        self._playlist_tracks = []
-        self._playlist_ids = []
-        # We are in all artists
-        if (filter1_ids and filter1_ids[0] == Type.ALL) or\
-           (filter2_ids and filter2_ids[0] == Type.ALL):
-            # Genres: all, Artists: compilations
-            if filter2_ids and filter2_ids[0] == Type.COMPILATIONS:
-                album_ids += App().albums.get_compilation_ids([], True)
-            # Genres: all, Artists: ids
-            elif filter2_ids and filter2_ids[0] != Type.ALL:
-                album_ids += App().albums.get_ids(filter2_ids, [], True)
-            # Genres: all, Artists: all
-            else:
-                if App().settings.get_value("show-compilations-in-album-view"):
-                    album_ids += App().albums.get_compilation_ids([], True)
-                album_ids += App().albums.get_ids([], [], True)
-        # We are in populars view, add popular albums
-        elif filter1_ids and filter1_ids[0] == Type.POPULARS:
-            album_ids += App().albums.get_populars()
-        # We are in loved view, add loved albums
-        elif filter1_ids and filter1_ids[0] == Type.LOVED:
-            album_ids += App().albums.get_loved_albums()
-        # We are in recents view, add recent albums
-        elif filter1_ids and filter1_ids[0] == Type.RECENTS:
-            album_ids += App().albums.get_recents()
-        # We are in randoms view, add random albums
-        elif filter1_ids and filter1_ids[0] == Type.RANDOMS:
-            album_ids += App().albums.get_randoms()
-        # We are in compilation view without genre
-        elif filter1_ids and filter1_ids[0] == Type.COMPILATIONS:
-            album_ids += App().albums.get_compilation_ids([])
-        # We are in years view
-        elif filter1_ids and filter1_ids[0] == Type.YEARS:
-            album_ids += []
-            for year in filter2_ids:
-                album_ids += App().albums.get_albums_for_year(year)
-                album_ids += App().albums.get_compilations_for_year(year)
-            # Reset filter2_ids as contains unwanted filter for later
-            # Album constructor
-            filter2_ids = []
-        # Add albums for artists/genres
-        else:
-            # If we are not in compilation view and show compilation is on,
-            # add compilations
-            if filter2_ids and filter2_ids[0] == Type.COMPILATIONS:
-                album_ids += App().albums.get_compilation_ids(
-                    filter1_ids, True)
-            elif filter2_ids:
-                # In artist view, play all albums if ignoring return []
-                if App().settings.get_value("show-performers"):
-                    album_ids += App().tracks.get_album_ids(filter2_ids,
-                                                            filter1_ids,
-                                                            True)
-                else:
-                    album_ids += App().albums.get_ids(filter2_ids,
-                                                      filter1_ids,
-                                                      True)
-                if not album_ids:
-                    if App().settings.get_value("show-performers"):
-                        album_ids += App().tracks.get_album_ids(filter2_ids,
-                                                                filter1_ids,
-                                                                False)
-                    else:
-                        album_ids += App().albums.get_ids(filter2_ids,
-                                                          filter1_ids,
-                                                          False)
-            elif App().settings.get_value(
-                            "show-compilations-in-album-view"):
-                album_ids += App().albums.get_compilation_ids(
-                    filter1_ids, True)
-                album_ids += App().albums.get_ids([], filter1_ids, True)
-            else:
-                album_ids += App().albums.get_ids([], filter1_ids, True)
-
-        if not album_ids:
-            return
-
-        # Create album objects
-        albums = []
-        album = None
-        for _album_id in album_ids:
-            _album = Album(_album_id, filter1_ids, filter2_ids, True)
-            if album_id == _album_id:
-                album = _album
-            albums.append(_album)
-
-        shuffle_setting = App().settings.get_enum("shuffle")
-        if shuffle_setting == Shuffle.ALBUMS:
-            self.__play_shuffle_albums(album, albums)
-        elif shuffle_setting == Shuffle.TRACKS:
+        if self.is_party:
+            App().lookup_action("party").change_state(GLib.Variant("b", False))
+        if App().settings.get_value("shuffle"):
             self.__play_shuffle_tracks(album, albums)
         else:
             self.__play_albums(album, albums)
-        self.emit("playlist-changed")
+        emit_signal(self, "playback-changed")
+
+    def play_albums(self, albums):
+        """
+            Play albums
+            @param album as [Album]
+        """
+        if not albums:
+            return
+        if App().settings.get_value("shuffle"):
+            album = choice(albums)
+        else:
+            album = albums[0]
+        self.play_album_for_albums(album, albums)
 
     def play_uris(self, uris):
         """
@@ -317,15 +204,20 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
                 self.play_album(album)
             else:
                 self.add_album(album)
+        emit_signal(self, "playback-changed")
+
+    def set_albums(self, albums):
+        """
+            Set player albums
+        """
+        self._albums = albums
 
     def clear_albums(self):
         """
             Clear all albums
         """
         self._albums = []
-        self.set_next()
-        self.set_prev()
-        self.emit("playlist-changed")
+        emit_signal(self, "playback-changed")
 
     def stop_after(self, track_id):
         """
@@ -385,16 +277,6 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
                             if track.id == self._current_playback_track.id:
                                 self._load_track(track)
                                 break
-                    else:
-                        tracks = []
-                        track = Track()
-                        for playlist_id in playlist_ids:
-                            tracks += App().playlists.get_tracks(playlist_id)
-                            for track in tracks:
-                                if track.id == self._current_playback_track.id:
-                                    break
-                        self.populate_playlist_by_tracks(
-                            tracks, playlist_ids, track)
                     if is_playing:
                         self.play()
                     else:
@@ -404,7 +286,6 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
                     self.seek(position / Gst.SECOND)
                 else:
                     Logger.info("Player::restore_state(): track missing")
-                self.emit("playlist-changed")
         except Exception as e:
             Logger.error("Player::restore_state(): %s" % e)
 
@@ -421,20 +302,15 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         """
             Set previous track
         """
-        if self.current_track.id == Type.RADIOS:
+        if isinstance(self.current_track, Radio):
             return
         try:
-            prev_track = ShufflePlayer.prev(self)
-
-            # Look at user playlist then
-            if prev_track.id is None:
-                prev_track = PlaylistPlayer.prev(self)
-
-            # Get a linear track then
-            if prev_track.id is None:
+            if App().settings.get_value("shuffle") or self.is_party:
+                prev_track = ShufflePlayer.prev(self)
+            else:
                 prev_track = LinearPlayer.prev(self)
             self._prev_track = prev_track
-            self.emit("prev-changed")
+            emit_signal(self, "prev-changed")
         except Exception as e:
             Logger.error("Player::set_prev(): %s" % e)
 
@@ -442,30 +318,22 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         """
             Play next track
         """
-        if self.current_track.id == Type.RADIOS or\
+        if isinstance(self.current_track, Radio) or\
                 self._current_track.id == self.__stop_after_track_id:
             self.__stop_after_track_id = None
             self._next_track = Track()
             return
         try:
             next_track = QueuePlayer.next(self)
-
-            # Look at shuffle
             if next_track.id is None:
-                next_track = ShufflePlayer.next(self)
-
-            # Look at user playlist then
-            if next_track.id is None:
-                next_track = PlaylistPlayer.next(self)
-
-            # Get a linear track then
-            if next_track.id is None:
-                next_track = LinearPlayer.next(self)
-
+                if App().settings.get_value("shuffle") or self.is_party:
+                    next_track = ShufflePlayer.next(self)
+                else:
+                    next_track = LinearPlayer.next(self)
             self._next_track = next_track
             if next_track.is_web:
                 App().task_helper.run(self._load_from_web, next_track, False)
-            self.emit("next-changed")
+            emit_signal(self, "next-changed")
         except Exception as e:
             Logger.error("Player::set_next(): %s" % e)
 
@@ -475,11 +343,10 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         """
         try:
             # In party or shuffle, just update next track
-            if self.is_party or\
-                    App().settings.get_enum("shuffle") == Shuffle.TRACKS:
+            if self.is_party or App().settings.get_value("shuffle"):
                 self.set_next()
                 # We send this signal to update next popover
-                self.emit("queue-changed")
+                emit_signal(self, "queue-changed")
             elif self._current_track.id is not None:
                 index = self.album_ids.index(
                     App().player._current_playback_track.album.id)
@@ -491,6 +358,16 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         except Exception as e:
             Logger.error("Player::skip_album(): %s" % e)
 
+    def update_next_prev(self, *ignore):
+        """
+            Update next/prev
+            @param player as Player
+        """
+        if self._current_track.id is not None:
+            if not self.is_party:
+                self.set_next()
+            self.set_prev()
+
     def update_crossfading(self):
         """
             Calculate if crossfading is needed
@@ -572,8 +449,8 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
             self._current_playback_track = self._current_track
         ShufflePlayer._on_stream_start(self, bus, message)
         BinPlayer._on_stream_start(self, bus, message)
-        self.set_prev()
         self.set_next()
+        self.set_prev()
 
 #######################
 # PRIVATE             #
@@ -628,11 +505,3 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         self._albums = albums
         if track is not None:
             self.load(track)
-
-    def __on_repeat_changed(self, settings, value):
-        """
-            reset next/prev
-            @param settings as Gio.Settings, value as str
-        """
-        self.set_next()
-        self.set_prev()
