@@ -12,8 +12,7 @@
 
 from gi.repository import GObject
 
-from lollypop.define import App
-from lollypop.objects import Track
+from lollypop.objects_track import Track
 
 
 class BasePlayer(GObject.GObject):
@@ -23,18 +22,16 @@ class BasePlayer(GObject.GObject):
     __gsignals__ = {
         "current-changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
         "duration-changed": (GObject.SignalFlags.RUN_FIRST, None, (int,)),
-        "loading-changed": (GObject.SignalFlags.RUN_FIRST, None, (bool,)),
-        "album-added": (GObject.SignalFlags.RUN_FIRST, None, (int,)),
-        "album-removed": (GObject.SignalFlags.RUN_FIRST, None, (int,)),
-        "playlist-changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
+        "loading-changed": (GObject.SignalFlags.RUN_FIRST, None,
+                            (bool, GObject.TYPE_PYOBJECT)),
         "next-changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
         "prev-changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
         "seeked": (GObject.SignalFlags.RUN_FIRST, None, (int,)),
         "status-changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
         "volume-changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
         "queue-changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
-        "rate-changed": (GObject.SignalFlags.RUN_FIRST, None, (int, int)),
-        "party-changed": (GObject.SignalFlags.RUN_FIRST, None, (bool,))
+        "playback-changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
+        "rate-changed": (GObject.SignalFlags.RUN_FIRST, None, (int, int))
     }
 
     def __init__(self):
@@ -48,19 +45,12 @@ class BasePlayer(GObject.GObject):
             self._base_init = True
             # Should player do crossfading
             self._crossfading = False
-            # A user playlist used as current playlist
-            self._playlist_ids = []
-            self._playlist_tracks = []
-            # Used by shuffle tracks to restore user playlist before shuffle
-            self._playlist_backup = None
             self._current_track = Track()
             self._current_playback_track = Track()
             self._next_track = Track()
             self._prev_track = Track()
             # Albums in current playlist
             self._albums = []
-            # Current shuffle mode
-            self._shuffle = App().settings.get_enum("shuffle")
             # For tracks from the cmd line
             self._external_tracks = []
 
