@@ -13,15 +13,17 @@
 from gi.repository import Gst, GstAudio, GstPbutils, GLib, Gio
 
 from time import time
+from gettext import gettext as _
 
 from lollypop.player_base import BasePlayer
-from lollypop.tagreader import TagReader
+from lollypop.tagreader import TagReader, Discoverer
 from lollypop.player_plugins import PluginsPlayer
 from lollypop.define import GstPlayFlags, App
 from lollypop.codecs import Codecs
-from lollypop.define import Type
 from lollypop.logger import Logger
-from lollypop.objects import Track
+from lollypop.objects_track import Track
+from lollypop.objects_radio import Radio
+from lollypop.utils import emit_signal, get_network_available
 
 
 class BinPlayer(BasePlayer):
@@ -79,7 +81,7 @@ class BinPlayer(BasePlayer):
         if self._crossfading and\
            self._current_track.id is not None and\
            self.is_playing and\
-           self._current_track.id != Type.RADIOS:
+           not isinstance(track, Radio):
             duration = App().settings.get_value(
                 "transition-duration").get_int32()
             self.__do_crossfade(duration, track)
@@ -96,17 +98,17 @@ class BinPlayer(BasePlayer):
                 self.load(self._next_track)
         else:
             self._playbin.set_state(Gst.State.PLAYING)
-            self.emit("status-changed")
+            emit_signal(self, "status-changed")
 
     def pause(self):
         """
             Change player state to PAUSED
         """
-        if self._current_track.id == Type.RADIOS:
+        if isinstance(App().player.current_track, Radio):
             self._playbin.set_state(Gst.State.NULL)
         else:
             self._playbin.set_state(Gst.State.PAUSED)
-        self.emit("status-changed")
+        emit_signal(self, "status-changed")
 
     def stop(self, force=False):
         """
@@ -115,15 +117,14 @@ class BinPlayer(BasePlayer):
         """
         self._current_track = Track()
         self._playbin.set_state(Gst.State.NULL)
-        self.emit("status-changed")
-        self.emit("current-changed")
+        emit_signal(self, "status-changed")
+        emit_signal(self, "current-changed")
         if force:
             self._prev_track = Track()
             self._next_track = Track()
-            App().player.emit("prev-changed")
-            App().player.emit("next-changed")
-            self._albums = []
-            self.reset_history()
+            emit_signal(self, "prev-changed")
+            emit_signal(self, "next-changed")
+            self.clear_albums()
 
     def stop_all(self):
         """
@@ -161,7 +162,7 @@ class BinPlayer(BasePlayer):
         if self._current_track.id is None:
             return
         # Seems gstreamer doesn't like seeking to end, sometimes
-        # doesn"t go to next track
+        # doesn't go to next track
         if position >= self._current_track.duration:
             self.next()
         else:
@@ -169,7 +170,7 @@ class BinPlayer(BasePlayer):
                                       Gst.SeekFlags.FLUSH |
                                       Gst.SeekFlags.KEY_UNIT,
                                       position * Gst.SECOND)
-            self.emit("seeked", position)
+            emit_signal(self, "seeked", position)
 
     @property
     def plugins(self):
@@ -258,12 +259,14 @@ class BinPlayer(BasePlayer):
         try:
             self.__cancellable.cancel()
             self.__cancellable = Gio.Cancellable()
+            if self._current_track.is_web:
+                emit_signal(self, "loading-changed", False, track)
             self._current_track = track
             # We check track is URI track, if yes, do a load from Web
             # Will not work if we add another music provider one day
             track_uri = App().tracks.get_uri(track.id)
             if track.is_web and track.uri == track_uri:
-                self.emit("loading-changed", True)
+                emit_signal(self, "loading-changed", True, track)
                 App().task_helper.run(self._load_from_web, track)
                 return False
             else:
@@ -286,11 +289,21 @@ class BinPlayer(BasePlayer):
                 App().task_helper.run(self.__update_current_duration,
                                       track, uri)
 
-        from lollypop.helper_web import WebHelper
-        helper = WebHelper()
-        helper.set_uri(track, self.__cancellable)
-        uri = helper.get_track_content(track)
-        GLib.idle_add(play_uri, uri)
+        if get_network_available():
+            from lollypop.helper_web import WebHelper
+            helper = WebHelper()
+            helper.set_uri(track, self.__cancellable)
+            uri = helper.get_track_content(track)
+            if uri is not None:
+                GLib.idle_add(play_uri, uri)
+            elif play:
+                GLib.idle_add(
+                    App().notify.send,
+                    "Lollypop",
+                    _("Can't find this track on YouTube"))
+                self.next()
+        elif play:
+            self.skip_album()
 
     def _scrobble(self, finished, finished_start_time):
         """
@@ -316,11 +329,13 @@ class BinPlayer(BasePlayer):
             @param bus as Gst.Bus
             @param message as Gst.Message
         """
-        self.emit("loading-changed", False)
+        if self._current_track.is_web:
+            emit_signal(self, "loading-changed", False,
+                        self._current_track.album)
         self._start_time = time()
         Logger.debug("Player::_on_stream_start(): %s" %
                      self._current_track.uri)
-        self.emit("current-changed")
+        emit_signal(self, "current-changed")
         for scrobbler in App().scrobblers:
             if scrobbler.available:
                 scrobbler.playing_now(self._current_track)
@@ -332,28 +347,16 @@ class BinPlayer(BasePlayer):
             @param bus as Gst.Bus
             @param message as Gst.Message
         """
-        # Some radio streams send message tag every seconds!
-        changed = False
-        if self._current_track.id >= 0 or self._current_track.duration > 0.0:
+        if isinstance(self._current_track, Track):
             return
         Logger.debug("Player::__on_bus_message_tag(): %s" %
                      self._current_track.uri)
         reader = TagReader()
         tags = message.parse_tag()
         title = reader.get_title(tags, "")
-        if len(title) > 1 and self._current_track.name != title:
-            self._current_track.name = title
-            changed = True
-            if self._current_track.name == "":
-                self._current_track.name = self._current_track.uri
-        artists = reader.get_artists(tags)
-        if len(artists) > 1 and self._current_track.artists != artists:
-            self._current_track.artists = artists.split(",")
-            changed = True
-            if not self._current_track.artists:
-                self._current_track.artists = self._current_track.album_artists
-        if changed:
-            self.emit("current-changed")
+        if len(title) > 1 and self._current_track.artists != [title]:
+            self._current_track.artists = [title]
+            emit_signal(self, "current-changed")
 
     def _on_bus_element(self, bus, message):
         """
@@ -370,16 +373,18 @@ class BinPlayer(BasePlayer):
             @param bus as Gst.Bus
             @param message as Gst.Message
         """
-        self.emit("loading-changed", False)
+        if self._current_track.is_web:
+            emit_signal(self, "loading-changed", False,
+                        self._current_track.album)
         Logger.info("Player::_on_bus_error(): %s" % message.parse_error()[1])
         if self.current_track.id is not None and self.current_track.id >= 0:
             if self.__codecs.is_missing_codec(message):
                 self.__codecs.install()
                 App().scanner.stop()
                 self.stop()
-            elif App().notify is not None:
+            else:
                 (error, parsed) = message.parse_error()
-                App().notify.send(parsed)
+                App().notify.send("Lollypop", parsed)
                 self.stop()
 
     def _on_bus_eos(self, bus, message):
@@ -390,7 +395,7 @@ class BinPlayer(BasePlayer):
         # Don't do anything if crossfade on, track already changed
         if self._crossfading:
             return
-        if self._current_track.id == Type.RADIOS:
+        if isinstance(App().player.current_track, Radio):
             return
         if self.is_playing and self._playbin.get_bus() == bus:
             if self._next_track.id is None:
@@ -411,7 +416,7 @@ class BinPlayer(BasePlayer):
         # Don't do anything if crossfade on, track already changed
         if self._crossfading:
             return
-        if self._current_track.id == Type.RADIOS:
+        if isinstance(App().player.current_track, Radio):
             return
         self._scrobble(self._current_track, self._start_time)
         # Increment popularity
@@ -422,15 +427,8 @@ class BinPlayer(BasePlayer):
                 pop_to_add = 1
             # In normal mode, based on tracks count
             else:
-                # Some users report an issue where get_tracks_count() return 0
-                # See issue #886
-                # Don"t understand how this can happen!
-                count = App().albums.get_tracks_count(
-                    self._current_track.album_id)
-                if count:
-                    pop_to_add = int(App().albums.max_count / count)
-                else:
-                    pop_to_add = 1
+                count = self._current_track.album.tracks_count
+                pop_to_add = int(App().albums.max_count / count)
             App().albums.set_more_popular(self._current_track.album_id,
                                           pop_to_add)
         if self._next_track.id is None:
@@ -524,15 +522,10 @@ class BinPlayer(BasePlayer):
                 pop_to_add = 1
             # In normal mode, based on tracks count
             else:
-                count = App().albums.get_tracks_count(
-                    self._current_track.album_id)
-                if count:
-                    pop_to_add = int(App().albums.max_count / count)
-                else:
-                    pop_to_add = 0
-            if pop_to_add > 0:
-                App().albums.set_more_popular(self._current_track.album_id,
-                                              pop_to_add)
+                count = self._current_track.album.tracks_count
+                pop_to_add = int(App().albums.max_count / count)
+            App().albums.set_more_popular(self._current_track.album_id,
+                                          pop_to_add)
 
         GLib.idle_add(self.__volume_down, self._playbin,
                       self._plugins, duration)
@@ -561,12 +554,12 @@ class BinPlayer(BasePlayer):
             @param uri as str
         """
         try:
-            reader = TagReader()
-            duration = reader.get_info(uri).get_duration() / 1000000000
+            discoverer = Discoverer()
+            duration = discoverer.get_info(uri).get_duration() / 1000000000
             if duration != track.duration and duration > 0:
                 App().tracks.set_duration(track.id, int(duration))
                 track.reset("duration")
-                GLib.idle_add(self.emit, "duration-changed", track.id)
+                emit_signal(self, "duration-changed", track.id)
         except Exception as e:
             Logger.error("BinPlayer::__update_current_duration(): %s" % e)
 
@@ -582,4 +575,4 @@ class BinPlayer(BasePlayer):
         else:
             vol = self.__playbin2.get_volume(GstAudio.StreamVolumeFormat.CUBIC)
             self.__playbin1.set_volume(GstAudio.StreamVolumeFormat.CUBIC, vol)
-        self.emit("volume-changed")
+        emit_signal(self, "volume-changed")
