@@ -21,11 +21,12 @@ import json
 
 from lollypop.database import Database
 from lollypop.define import App, Type
-from lollypop.objects import Track
-from lollypop.logger import Logger
+from lollypop.objects_track import Track
 from lollypop.sqlcursor import SqlCursor
 from lollypop.localized import LocalizedCollation
 from lollypop.shown import ShownPlaylists
+from lollypop.utils import get_mtime, emit_signal
+from lollypop.logger import Logger
 from lollypop.database_upgrade import DatabasePlaylistsUpgrade
 
 
@@ -40,9 +41,9 @@ class Playlists(GObject.GObject):
         "playlists-changed": (GObject.SignalFlags.RUN_FIRST, None, (int,)),
         # Objects added/removed to/from playlist
         "playlist-track-added": (
-            GObject.SignalFlags.RUN_FIRST, None, (int, str, int)),
+            GObject.SignalFlags.RUN_FIRST, None, (int, str)),
         "playlist-track-removed": (
-            GObject.SignalFlags.RUN_FIRST, None, (int, str, int))
+            GObject.SignalFlags.RUN_FIRST, None, (int, str))
     }
     __create_playlists = """CREATE TABLE playlists (
                             id INTEGER PRIMARY KEY,
@@ -50,6 +51,7 @@ class Playlists(GObject.GObject):
                             synced INT NOT NULL DEFAULT 0,
                             smart_enabled INT NOT NULL DEFAULT 0,
                             smart_sql TEXT,
+                            uri TEXT,
                             mtime BIGINT NOT NULL)"""
 
     __create_tracks = """CREATE TABLE tracks (
@@ -76,6 +78,23 @@ class Playlists(GObject.GObject):
         else:
             upgrade.upgrade(self)
 
+    def get_new_name(self):
+        """
+            Get a name for a new playlist
+            @return str
+        """
+        existing_playlists = []
+        for (playlist_id, name) in App().playlists.get():
+            existing_playlists.append(name)
+
+        # Search for an available name
+        count = 1
+        name = _("New playlist ") + str(count)
+        while name in existing_playlists:
+            count += 1
+            name = _("New playlist ") + str(count)
+        return name
+
     def add(self, name):
         """
             Add a playlist
@@ -86,10 +105,10 @@ class Playlists(GObject.GObject):
         if name == _("Loved tracks"):
             return Type.LOVED
         with SqlCursor(self, True) as sql:
-            result = sql.execute("INSERT INTO playlists (name, mtime)"
-                                 " VALUES (?, ?)",
-                                 (name, datetime.now().strftime("%s")))
-            GLib.idle_add(self.emit, "playlists-changed", result.lastrowid)
+            result = sql.execute("INSERT INTO playlists (name, mtime)\
+                                  VALUES (?, ?)",
+                                 (name, 0))
+            emit_signal(self, "playlists-changed", result.lastrowid)
             return result.lastrowid
 
     def exists(self, playlist_id):
@@ -119,7 +138,8 @@ class Playlists(GObject.GObject):
                         SET name=?\
                         WHERE rowid=?",
                         (name, playlist_id))
-            GLib.idle_add(self.emit, "playlists-changed", playlist_id)
+        emit_signal(self, "playlists-changed", playlist_id)
+        App().art.remove_artwork_from_cache(name, "ROUNDED")
 
     def remove(self, playlist_id):
         """
@@ -133,7 +153,7 @@ class Playlists(GObject.GObject):
             sql.execute("DELETE FROM tracks\
                         WHERE playlist_id=?",
                         (playlist_id,))
-            GLib.idle_add(self.emit, "playlists-changed", playlist_id)
+            emit_signal(self, "playlists-changed", playlist_id)
 
     def clear(self, playlist_id):
         """
@@ -143,6 +163,7 @@ class Playlists(GObject.GObject):
         with SqlCursor(self, True) as sql:
             sql.execute("DELETE FROM tracks\
                          WHERE playlist_id=?", (playlist_id,))
+        self.sync_to_disk(playlist_id)
 
     def add_uri(self, playlist_id, uri, signal=False):
         """
@@ -154,13 +175,9 @@ class Playlists(GObject.GObject):
         if self.exists_track(playlist_id, uri):
             return
         if signal:
-            position = len(self.get_track_uris(playlist_id))
-            self.emit("playlist-track-added", playlist_id, uri, position)
+            emit_signal(self, "playlist-track-added", playlist_id, uri)
         with SqlCursor(self, True) as sql:
             sql.execute("INSERT INTO tracks VALUES (?, ?)", (playlist_id, uri))
-            sql.execute("UPDATE playlists SET mtime=?\
-                         WHERE rowid=?", (datetime.now().strftime("%s"),
-                                          playlist_id))
 
     def add_uris(self, playlist_id, uris, signal=False):
         """
@@ -171,6 +188,7 @@ class Playlists(GObject.GObject):
         """
         for uri in uris:
             self.add_uri(playlist_id, uri, signal)
+        self.sync_to_disk(playlist_id)
 
     def add_tracks(self, playlist_id, tracks, signal=False):
         """
@@ -181,26 +199,7 @@ class Playlists(GObject.GObject):
         """
         for track in tracks:
             self.add_uri(playlist_id, track.uri, signal)
-
-    def insert_track(self, playlist_id, track, position):
-        """
-            Insert track at position, will remove track first if exists
-            @param playlist_id as int
-            @param track as Track
-            @param position as int
-        """
-        SqlCursor.add(self)
-        track_ids = self.get_track_ids(playlist_id)
-        if track.id in track_ids:
-            index = track_ids.index(track.id)
-            track_ids.remove(track.id)
-            if index < position:
-                position -= 1
-        track_ids.insert(position, track.id)
-        self.clear(playlist_id)
-        tracks = [Track(track_id) for track_id in track_ids]
-        self.add_tracks(playlist_id, tracks)
-        SqlCursor.remove(self)
+        self.sync_to_disk(playlist_id)
 
     def remove_uri(self, playlist_id, uri, signal=False):
         """
@@ -212,15 +211,10 @@ class Playlists(GObject.GObject):
         if not self.exists_track(playlist_id, uri):
             return
         if signal:
-            uris = self.get_track_uris(playlist_id)
-            position = uris.index(uri)
-            self.emit("playlist-track-removed", playlist_id, uri, position)
+            emit_signal(self, "playlist-track-removed", playlist_id, uri)
         with SqlCursor(self, True) as sql:
             sql.execute("DELETE FROM tracks WHERE uri=? AND playlist_id=?",
                         (uri, playlist_id))
-            sql.execute("UPDATE playlists SET mtime=?\
-                         WHERE rowid=?", (datetime.now().strftime("%s"),
-                                          playlist_id))
 
     def remove_uris(self, playlist_id, uris, signal=False):
         """
@@ -231,6 +225,7 @@ class Playlists(GObject.GObject):
         """
         for uri in uris:
             self.remove_uri(playlist_id, uri, signal)
+        self.sync_to_disk(playlist_id)
 
     def remove_tracks(self, playlist_id, tracks, signal=False):
         """
@@ -241,16 +236,7 @@ class Playlists(GObject.GObject):
         """
         for track in tracks:
             self.remove_uri(playlist_id, track.uri, signal)
-
-    def remove_uri_from_all(self, uri):
-        """
-            Remove track from all playlists
-            @param uri as str
-        """
-        with SqlCursor(self, True) as sql:
-            sql.execute("DELETE FROM tracks\
-                        WHERE uri=?",
-                        (uri,))
+        self.sync_to_disk(playlist_id)
 
     def get(self):
         """
@@ -258,7 +244,7 @@ class Playlists(GObject.GObject):
             @return [int, str, str]
         """
         with SqlCursor(self) as sql:
-            result = sql.execute("SELECT rowid, name, name\
+            result = sql.execute("SELECT rowid, name\
                                   FROM playlists\
                                   ORDER BY name\
                                   COLLATE NOCASE COLLATE LOCALIZED")
@@ -276,19 +262,6 @@ class Playlists(GObject.GObject):
                                   COLLATE NOCASE COLLATE LOCALIZED")
             return list(itertools.chain(*result))
 
-    def get_last(self):
-        """
-            Return 6 last modified playlist
-            @return [string]
-        """
-        with SqlCursor(self) as sql:
-            result = sql.execute("SELECT rowid, name\
-                                  FROM playlists\
-                                  WHERE smart_enabled=0\
-                                  ORDER BY mtime DESC\
-                                  LIMIT 4")
-            return list(result)
-
     def get_track_uris(self, playlist_id):
         """
             Return availables track uris for playlist
@@ -318,6 +291,8 @@ class Playlists(GObject.GObject):
             track_ids = App().tracks.get_never_listened_to()
         elif playlist_id == Type.RANDOMS:
             track_ids = App().tracks.get_randoms()
+        elif playlist_id == Type.ALL:
+            track_ids = App().tracks.get_ids()
         elif playlist_id == Type.LOVED:
             track_ids = App().playlists.get_track_ids_sorted(playlist_id)
         else:
@@ -396,7 +371,7 @@ class Playlists(GObject.GObject):
             v = result.fetchone()
             if v is not None:
                 return v[0]
-            return Type.NONE
+            return None
 
     def get_name(self, playlist_id):
         """
@@ -418,28 +393,6 @@ class Playlists(GObject.GObject):
                 return v[0]
             return ""
 
-    def get_names(self, playlist_ids):
-        """
-            Return playlist names
-            @param playlist_ids as [int]
-            @return names as [str]
-        """
-        names = []
-        for playlist_id in playlist_ids:
-            if playlist_id == Type.POPULARS:
-                names.append(_("Popular tracks"))
-            elif playlist_id == Type.RECENTS:
-                names.append(_("Recently played"))
-            elif playlist_id == Type.NEVER:
-                names.append(_("Never played"))
-            elif playlist_id == Type.RANDOMS:
-                names.append(_("Random tracks"))
-            elif playlist_id == Type.SEARCH:
-                names.append(_("Search"))
-            else:
-                names.append(self.get_name(playlist_id))
-        return names
-
     def get_synced(self, playlist_id, index):
         """
             True if playlist synced
@@ -558,6 +511,7 @@ class Playlists(GObject.GObject):
                         SET smart_enabled=?\
                         WHERE rowid=?",
                         (smart, playlist_id))
+            emit_signal(self, "playlists-changed", playlist_id)
 
     def set_smart_sql(self, playlist_id, request):
         """
@@ -570,69 +524,7 @@ class Playlists(GObject.GObject):
                         SET smart_sql=?\
                         WHERE rowid=?",
                         (request, playlist_id))
-
-    def import_uri(self, playlist_id, uri, start=None, down=True):
-        """
-            Import uri in playlist
-            @param playlist_id as int
-            @param uri as str
-            @param start track id as int
-            @param down as bool
-        """
-        try:
-            uri = uri.strip("\n\r")
-            f = Gio.File.new_for_uri(uri)
-            if f.query_exists():
-                if f.query_file_type(Gio.FileQueryInfoFlags.NONE,
-                                     None) == Gio.FileType.DIRECTORY:
-                    walk_uris = [uri]
-                    track_ids = []
-                    while walk_uris:
-                        uri = walk_uris.pop(0)
-                        try:
-                            d = Gio.File.new_for_uri(uri)
-                            infos = d.enumerate_children(
-                                "standard::name,standard::type",
-                                Gio.FileQueryInfoFlags.NONE,
-                                None)
-                        except Exception as e:
-                            Logger.info("Playlists::import_uri(): %s" % e)
-                            continue
-                        for info in infos:
-                            f = infos.get_child(info)
-                            if info.get_file_type() == Gio.FileType.DIRECTORY:
-                                walk_uris.append(f.get_uri())
-                            else:
-                                track_id = App().tracks.get_id_by_uri(
-                                    f.get_uri())
-                                if track_id is not None:
-                                    track_ids.append(track_id)
-                else:
-                    track_id = App().tracks.get_id_by_uri(uri)
-                    track_ids = [track_id]
-                tracks = []
-                if start is None:
-                    for track_id in track_ids:
-                        tracks.append(Track(track_id))
-                    self.add_tracks(playlist_id, tracks)
-                else:
-                    # Insert at wanted position
-                    playlist_track_ids = self.get_track_ids(playlist_id)
-                    start_idx = playlist_track_ids.index(start)
-                    if down:
-                        start_idx += 1
-                    for track_id in track_ids:
-                        playlist_track_ids.insert(start_idx, track_id)
-                        GLib.idle_add(self.emit, "playlist-add",
-                                      playlist_id, track_id, start_idx)
-                        start_idx += 1
-                    self.clear(playlist_id)
-                    tracks = []
-                    for track_id in playlist_track_ids:
-                        tracks.append(Track(track_id))
-                    self.add_tracks(playlist_id, tracks)
-        except:
-            pass
+            emit_signal(self, "playlists-changed", playlist_id)
 
     def get_position(self, playlist_id, track_id):
         """
@@ -659,13 +551,67 @@ class Playlists(GObject.GObject):
             result = sql.execute("SELECT uri\
                                   FROM tracks\
                                   WHERE playlist_id=?\
-                                  AND uri=?",
-                                 (playlist_id, uri))
+                                  AND uri=?", (playlist_id, uri))
             v = result.fetchone()
             if v is not None:
                 return True
             return False
 
+    def set_sync_uri(self, playlist_id, uri):
+        """
+            Set sync URI
+            @param playlist_id as int
+            @param uri as str
+        """
+        with SqlCursor(self, True) as sql:
+            sql.execute("UPDATE playlists\
+                        SET uri=?\
+                        WHERE rowid=?", (uri, playlist_id))
+
+    def get_sync_uri(self, playlist_id):
+        """
+            Get sync URI
+            @param playlist_id as int
+            @return str/None
+        """
+        with SqlCursor(self) as sql:
+            result = sql.execute("SELECT uri\
+                                  FROM playlists\
+                                  WHERE rowid=?", (playlist_id,))
+            v = result.fetchone()
+            if v is not None:
+                return v[0]
+            return None
+
+    def sync_to_disk(self, playlist_id, create=False):
+        """
+            Sync playlist_id to disk
+            @param playlist_id as int
+            @param create as bool => create file
+        """
+        try:
+            name = self.get_name(playlist_id)
+            # Clear cache
+            App().art.remove_artwork_from_cache(name, "ROUNDED")
+            uri = self.get_sync_uri(playlist_id)
+            if uri is None:
+                return
+            f = Gio.File.new_for_uri(uri)
+            if not f.query_exists() and not create:
+                return
+            uris = self.get_track_uris(playlist_id)
+            if not uris:
+                return
+            stream = f.replace(None, False,
+                               Gio.FileCreateFlags.REPLACE_DESTINATION, None)
+            stream.write("#EXTM3U\n".encode("utf-8"))
+            for uri in uris:
+                string = "%s\n" % uri
+                stream.write(string.encode("utf-8"))
+            stream.close()
+        except Exception as e:
+            Logger.error("Playlists::sync_to_disk(): %s", e)
+
     def exists_album(self, playlist_id, album):
         """
             Return True if object_id is already present in playlist
@@ -692,18 +638,36 @@ class Playlists(GObject.GObject):
             Import file as playlist
             @param f as Gio.File
         """
+        # Create playlist and get id
         basename = ".".join(f.get_basename().split(".")[:-1])
-        parser = TotemPlParser.Parser.new()
         playlist_id = self.get_id(basename)
-        if playlist_id in [Type.NONE, Type.LOVED]:
+        if playlist_id is None:
             playlist_id = self.add(basename)
-            uris = self.get_tracks(playlist_id)
-            if not uris:
-                parser.connect("entry-parsed", self.__on_entry_parsed,
-                               playlist_id, uris)
-                parser.parse_async(f.get_uri(), True,
-                                   None, self.__on_parse_finished,
-                                   playlist_id, uris)
+
+        # Check mtime has been updated
+        with SqlCursor(self) as sql:
+            result = sql.execute("SELECT mtime\
+                                 FROM playlists\
+                                 WHERE rowid=?", (playlist_id,))
+            v = result.fetchone()
+            if v is not None:
+                db_mtime = v[0]
+            else:
+                db_mtime = 0
+            info = f.query_info(Gio.FILE_ATTRIBUTE_TIME_MODIFIED,
+                                Gio.FileQueryInfoFlags.NONE, None)
+            mtime = get_mtime(info)
+            if db_mtime >= mtime:
+                return
+
+        # Load playlist
+        parser = TotemPlParser.Parser.new()
+        uris = []
+        parser.connect("entry-parsed", self.__on_entry_parsed,
+                       playlist_id, uris)
+        parser.parse_async(f.get_uri(), True,
+                           None, self.__on_parse_finished,
+                           playlist_id, uris)
 
     def get_cursor(self):
         """
@@ -728,7 +692,12 @@ class Playlists(GObject.GObject):
             @param playlist_id as int
             @param uris as [str]
         """
+        self.clear(playlist_id)
         self.add_uris(playlist_id, uris)
+        with SqlCursor(self, True) as sql:
+            sql.execute("UPDATE playlists SET mtime=?\
+                         WHERE rowid=?", (datetime.now().strftime("%s"),
+                                          playlist_id))
 
     def __on_entry_parsed(self, parser, uri, metadata, playlist_id, uris):
         """
