@@ -12,51 +12,17 @@
 
 from gi.repository import Gtk, Gdk, GLib, GObject, Pango
 
-from gettext import gettext as _
 from locale import strcoll
 
 from lollypop.view import LazyLoadingView
+from lollypop.helper_filtering import FilteringHelper
+from lollypop.helper_gestures import GesturesHelper
 from lollypop.fastscroll import FastScroll
 from lollypop.define import Type, App, ArtSize, SelectionListMask
-from lollypop.define import SidebarContent, ArtBehaviour, ViewType
+from lollypop.define import ArtBehaviour, ViewType
 from lollypop.logger import Logger
-from lollypop.utils import get_icon_name, on_query_tooltip
-from lollypop.shown import ShownLists, ShownPlaylists
-
-
-class TypeAheadPopover(Gtk.Popover):
-    """
-        Special popover for find as type
-    """
-
-    def __init__(self):
-        """
-            Init popover
-        """
-        Gtk.Popover.__init__(self)
-        self.__entry = Gtk.Entry()
-        self.__entry.show()
-        self.add(self.__entry)
-        self.get_style_context().add_class("padding")
-        self.connect("unmap", self.__on_unmap)
-
-    @property
-    def entry(self):
-        """
-            Get popover entry
-            @return Gtk.Entry
-        """
-        return self.__entry
-
-#######################
-# PRIVATE             #
-#######################
-    def __on_unmap(self, popover):
-        """
-            Clear entry
-            @param popover as Gtk.popover
-        """
-        self.__entry.set_text("")
+from lollypop.utils import get_icon_name, on_query_tooltip, popup_widget
+from lollypop.utils import emit_signal
 
 
 class SelectionListRow(Gtk.ListBoxRow):
@@ -89,30 +55,12 @@ class SelectionListRow(Gtk.ListBoxRow):
             @param height as str
         """
         Gtk.ListBoxRow.__init__(self)
+        self.__artwork = None
         self.__rowid = rowid
         self.__name = name
         self.__sortname = sortname
         self.__mask = mask
-
-        if rowid == Type.SEPARATOR:
-            height = -1
-            self.set_sensitive(False)
-        elif self.__mask & SelectionListMask.ARTISTS and\
-                self.__rowid >= 0 and\
-                App().settings.get_value("artist-artwork"):
-            self.get_style_context().add_class("row")
-            if height < ArtSize.ARTIST_SMALL:
-                height = ArtSize.ARTIST_SMALL
-            # Padding => application.css
-            height += 12
-        elif App().settings.get_enum("sidebar-content") ==\
-                SidebarContent.DEFAULT:
-            self.get_style_context().add_class("row-big")
-            # Padding => application.css
-            height += 30
-        else:
-            self.get_style_context().add_class("row")
-        self.set_size_request(-1, height)
+        self.set_style(height)
 
     def populate(self):
         """
@@ -123,25 +71,24 @@ class SelectionListRow(Gtk.ListBoxRow):
             separator.show()
             self.add(separator)
             self.set_sensitive(False)
-            self.emit("populated")
+            emit_signal(self, "populated")
         else:
             self.__grid = Gtk.Grid()
             self.__grid.set_column_spacing(7)
+            self.__grid.show()
             self.__artwork = Gtk.Image.new()
+            self.__grid.add(self.__artwork)
             self.__label = Gtk.Label.new()
             self.__label.set_markup(GLib.markup_escape_text(self.__name))
-            self.__label.set_ellipsize(Pango.EllipsizeMode.END)
             self.__label.set_property("has-tooltip", True)
             self.__label.connect("query-tooltip", on_query_tooltip)
-            self.__label.show()
-            self.__grid.show()
-            self.__grid.show()
-            self.__grid.add(self.__artwork)
+            self.__label.set_xalign(0)
             self.__grid.add(self.__label)
             if self.__mask & SelectionListMask.ARTISTS:
                 self.__grid.set_margin_end(20)
             self.add(self.__grid)
             self.set_artwork()
+            self.set_mask()
 
     def set_label(self, string):
         """
@@ -149,12 +96,15 @@ class SelectionListRow(Gtk.ListBoxRow):
             @param string as str
         """
         self.__name = string
-        self.__label.set_markup(GLib.markup_escape_text(string))
+        if not self.__mask & SelectionListMask.SIDEBAR:
+            self.__label.set_markup(GLib.markup_escape_text(string))
 
     def set_artwork(self):
         """
             set_artwork widget
         """
+        if self.__artwork is None:
+            return
         if self.__rowid == Type.SEPARATOR:
             pass
         elif self.__mask & SelectionListMask.ARTISTS and\
@@ -162,8 +112,8 @@ class SelectionListRow(Gtk.ListBoxRow):
                 App().settings.get_value("artist-artwork"):
             App().art_helper.set_artist_artwork(
                                     self.__name,
-                                    ArtSize.ARTIST_SMALL,
-                                    ArtSize.ARTIST_SMALL,
+                                    ArtSize.SMALL,
+                                    ArtSize.SMALL,
                                     self.get_scale_factor(),
                                     ArtBehaviour.ROUNDED |
                                     ArtBehaviour.CROP_SQUARE |
@@ -171,14 +121,75 @@ class SelectionListRow(Gtk.ListBoxRow):
                                     self.__on_artist_artwork)
             self.__artwork.show()
         elif self.__rowid < 0:
-            icon_name = get_icon_name(self.__rowid, self.__mask)
-            self.__artwork.set_from_icon_name(icon_name,
-                                              Gtk.IconSize.BUTTON)
+            icon_name = get_icon_name(self.__rowid)
+            self.__artwork.set_from_icon_name(icon_name, Gtk.IconSize.INVALID)
+            self.__artwork.set_pixel_size(20)
             self.__artwork.show()
-            self.emit("populated")
+            emit_signal(self, "populated")
         else:
             self.__artwork.hide()
-            self.emit("populated")
+            emit_signal(self, "populated")
+
+    def set_mask(self, mask=None):
+        """
+            Set row mask
+            @param mask as SelectionListMask
+        """
+        # Do nothing if widget not populated
+        if self.__artwork is None:
+            self.__mask = mask
+            return
+        # Do not update widget if mask does not changed
+        elif mask == self.__mask:
+            return
+        # If no mask, use current one
+        elif mask is None:
+            mask = self.__mask
+        # Else use new mask
+        else:
+            self.__mask = mask
+
+        if mask & (SelectionListMask.LABEL |
+                   SelectionListMask.ARTISTS |
+                   SelectionListMask.GENRES):
+            self.__artwork.set_property("halign", Gtk.Align.FILL)
+            self.__artwork.set_hexpand(False)
+            self.__label.show()
+            self.set_tooltip_text("")
+            self.set_has_tooltip(False)
+        else:
+            self.__artwork.set_property("halign", Gtk.Align.CENTER)
+            self.__artwork.set_hexpand(True)
+            self.__label.hide()
+            self.set_tooltip_text(self.__label.get_text())
+            self.set_has_tooltip(True)
+        if mask & SelectionListMask.ELLIPSIZE:
+            self.__label.set_ellipsize(Pango.EllipsizeMode.END)
+        else:
+            self.__label.set_ellipsize(Pango.EllipsizeMode.NONE)
+
+    def set_style(self, height):
+        """
+            Set internal sizing
+        """
+        if self.__rowid == Type.SEPARATOR:
+            height = -1
+            self.set_sensitive(False)
+        elif self.__mask & SelectionListMask.ARTISTS and\
+                self.__rowid >= 0 and\
+                App().settings.get_value("artist-artwork"):
+            self.get_style_context().add_class("row")
+            if height < ArtSize.SMALL:
+                height = ArtSize.SMALL
+            # Padding => application.css
+            height += 12
+        elif self.__mask & SelectionListMask.SIDEBAR:
+            self.get_style_context().add_class("row-big")
+            # Padding => application.css
+            height += 30
+        else:
+            self.get_style_context().add_class("row")
+        self.set_size_request(-1, height)
 
     @property
     def is_populated(self):
@@ -222,71 +233,95 @@ class SelectionListRow(Gtk.ListBoxRow):
         """
         if surface is None:
             self.__artwork.get_style_context().add_class("artwork-icon")
-            self.__artwork.set_size_request(ArtSize.ARTIST_SMALL,
-                                            ArtSize.ARTIST_SMALL)
+            self.__artwork.set_size_request(ArtSize.SMALL,
+                                            ArtSize.SMALL)
             self.__artwork.set_from_icon_name(
                                               "avatar-default-symbolic",
                                               Gtk.IconSize.DND)
         else:
             self.__artwork.get_style_context().remove_class("artwork-icon")
             self.__artwork.set_from_surface(surface)
-        self.emit("populated")
+        emit_signal(self, "populated")
 
 
-class SelectionList(LazyLoadingView):
+class SelectionList(FilteringHelper, LazyLoadingView, GesturesHelper):
     """
         A list for artists/genres
     """
     __gsignals__ = {
         "populated": (GObject.SignalFlags.RUN_FIRST, None, ()),
-        "pass-focus": (GObject.SignalFlags.RUN_FIRST, None, ())
+        "expanded": (GObject.SignalFlags.RUN_FIRST, None, (bool,))
     }
 
-    def __init__(self, base_type):
+    def __init__(self, base_mask):
         """
             Init Selection list ui
-            @param base_type as SelectionListMask
+            @param base_mask as SelectionListMask
         """
-        LazyLoadingView.__init__(self, ViewType.NOT_ADAPTIVE)
-        self.__base_type = base_type
+        LazyLoadingView.__init__(self, ViewType.DEFAULT)
+        FilteringHelper.__init__(self)
+        self.__base_mask = base_mask
+        self.__mask = SelectionListMask.NONE
         self.__sort = False
-        self.__mask = 0
+        self.__animation_timeout_id = None
         self.__height = SelectionListRow.get_best_height(self)
-        self._listbox = Gtk.ListBox()
-        self._listbox.connect("button-release-event",
-                              self.__on_button_release_event)
-        self._listbox.connect("key-press-event",
-                              self.__on_key_press_event)
-        self._listbox.set_sort_func(self.__sort_func)
-        self._listbox.set_selection_mode(Gtk.SelectionMode.MULTIPLE)
-        self._listbox.set_filter_func(self._filter_func)
-        self._listbox.show()
-        self._viewport.add(self._listbox)
-        overlay = Gtk.Overlay.new()
-        overlay.set_hexpand(True)
-        overlay.set_vexpand(True)
-        overlay.show()
-        overlay.add(self._scrolled)
-        self.__fastscroll = FastScroll(self._listbox,
-                                       self._scrolled)
-        overlay.add_overlay(self.__fastscroll)
-        self.add(overlay)
-        self.get_style_context().add_class("sidebar")
-        App().art.connect("artist-artwork-changed",
-                          self.__on_artist_artwork_changed)
-        self.__type_ahead_popover = TypeAheadPopover()
-        self.__type_ahead_popover.set_relative_to(self._scrolled)
-        self.__type_ahead_popover.entry.connect("activate",
-                                                self.__on_type_ahead_activate)
-        self.__type_ahead_popover.entry.connect("changed",
-                                                self.__on_type_ahead_changed)
+        self._box = Gtk.ListBox()
+        self._box.set_sort_func(self.__sort_func)
+        self._box.set_selection_mode(Gtk.SelectionMode.MULTIPLE)
+        self._box.show()
+        GesturesHelper.__init__(self, self._box)
+        self.__scrolled = Gtk.ScrolledWindow()
+        self.__scrolled.show()
+        self.__scrolled.set_property("expand", True)
+        self.__viewport = Gtk.Viewport()
+        self.__scrolled.add(self.__viewport)
+        self.__viewport.show()
+        self.__viewport.add(self._box)
+        if self.__base_mask & SelectionListMask.VIEW:
+            self.__overlay = Gtk.Overlay.new()
+            self.__overlay.set_hexpand(True)
+            self.__overlay.set_vexpand(True)
+            self.__overlay.show()
+            self.__overlay.add(self.__scrolled)
+            self.__fastscroll = FastScroll(self._box,
+                                           self.__scrolled)
+            self.__overlay.add_overlay(self.__fastscroll)
+            self.add(self.__overlay)
+            self.__base_mask |= SelectionListMask.LABEL
+            App().settings.connect("changed::artist-artwork",
+                                   self.__on_artist_artwork_changed)
+            App().art.connect("artist-artwork-changed",
+                              self.__on_artist_artwork_changed)
+        else:
+            self.__overlay = None
+            App().settings.connect("changed::show-sidebar-labels",
+                                   self.__on_show_sidebar_labels_changed)
+            self.__scrolled.set_policy(Gtk.PolicyType.NEVER,
+                                       Gtk.PolicyType.AUTOMATIC)
+            self.add(self.__scrolled)
+            self.get_style_context().add_class("sidebar")
+            self.__menu_button = Gtk.Button.new_from_icon_name(
+                "view-more-horizontal-symbolic", Gtk.IconSize.BUTTON)
+            self.__menu_button.set_property("halign", Gtk.Align.CENTER)
+            self.__menu_button.get_style_context().add_class("no-border")
+            self.__menu_button.connect("clicked",
+                                       lambda x: self.__popup_menu(None, x))
+            self.__menu_button.show()
+            self.add(self.__menu_button)
 
-    def mark_as(self, type):
+    def set_mask(self, mask):
         """
             Mark list as artists list
-            @param type as SelectionListMask
+            @param mask as SelectionListMask
         """
-        self.__mask = self.__base_type | type
+        self.__mask = mask
+
+    def add_mask(self, mask):
+        """
+            Mark list as artists list
+            @param mask as SelectionListMask
+        """
+        self.__mask |= mask
 
     def populate(self, values):
         """
@@ -294,7 +329,7 @@ class SelectionList(LazyLoadingView):
             @param [(int, str, optional str)], will be deleted
         """
         self.__sort = False
-        self._scrolled.get_vadjustment().set_value(0)
+        self.__scrolled.get_vadjustment().set_value(0)
         self.clear()
         self.__add_values(values)
 
@@ -303,7 +338,7 @@ class SelectionList(LazyLoadingView):
             Remove id from list
             @param object_id as int
         """
-        for child in self._listbox.get_children():
+        for child in self._box.get_children():
             if child.id == object_id:
                 child.destroy()
                 break
@@ -315,7 +350,7 @@ class SelectionList(LazyLoadingView):
         """
         self.__sort = True
         # Do not add value if already exists
-        for child in self._listbox.get_children():
+        for child in self._box.get_children():
             if child.id == value[0]:
                 return
         row = self.__add_value(value[0], value[1], value[2])
@@ -328,16 +363,17 @@ class SelectionList(LazyLoadingView):
             @param name as str
         """
         found = False
-        for child in self._listbox.get_children():
+        for child in self._box.get_children():
             if child.id == object_id:
                 child.set_label(name)
                 found = True
                 break
         if not found:
-            self.__fastscroll.clear()
+            if self.__base_mask & SelectionListMask.VIEW:
+                self.__fastscroll.clear()
             row = self.__add_value(object_id, name, name)
             row.populate()
-            if self.__mask & SelectionListMask.ARTISTS:
+            if self.mask & SelectionListMask.ARTISTS:
                 self.__fastscroll.populate()
 
     def update_values(self, values):
@@ -345,104 +381,95 @@ class SelectionList(LazyLoadingView):
             Update view with values
             @param [(int, str, optional str)]
         """
-        if self.__mask & SelectionListMask.ARTISTS:
+        if self.mask & SelectionListMask.VIEW:
             self.__fastscroll.clear()
         # Remove not found items
         value_ids = set([v[0] for v in values])
-        for child in self._listbox.get_children():
+        for child in self._box.get_children():
             if child.id not in value_ids:
                 self.remove_value(child.id)
         # Add items which are not already in the list
-        item_ids = set([child.id for child in self._listbox.get_children()])
+        item_ids = set([child.id for child in self._box.get_children()])
         for value in values:
             if not value[0] in item_ids:
                 row = self.__add_value(value[0], value[1], value[2])
                 row.populate()
-        if self.__mask & SelectionListMask.ARTISTS:
+        if self.mask & SelectionListMask.ARTISTS:
             self.__fastscroll.populate()
 
-    def select_ids(self, ids=[]):
+    def select_ids(self, ids=[], activate=True):
         """
             Select listbox items
             @param ids as [int]
+            @param activate as bool
         """
-        self._listbox.unselect_all()
-        for row in self._listbox.get_children():
-            if row.id in ids:
-                self._listbox.select_row(row)
-        for row in self._listbox.get_selected_rows():
-            row.activate()
-            break
-
-    def grab_focus(self):
-        """
-            Grab focus on treeview
-        """
-        self._listbox.grab_focus()
+        if ids:
+            rows = []
+            for row in self._box.get_children():
+                if row.id in ids:
+                    rows.append(row)
+            if rows:
+                self._box.unselect_all()
+                for row in rows:
+                    self._box.select_row(row)
+                if activate:
+                    rows[0].activate()
+        else:
+            self._box.unselect_all()
 
     def clear(self):
         """
             Clear treeview
         """
-        for child in self._listbox.get_children():
+        self.stop()
+        for child in self._box.get_children():
             child.destroy()
-        self.__fastscroll.clear()
-        self.__fastscroll.clear_chars()
-
-    def get_headers(self, mask):
-        """
-            Return headers
-            @param mask as SelectionListMask
-            @return items as [(int, str)]
-        """
-        lists = ShownLists.get(mask)
-        if mask & SelectionListMask.LIST_ONE and App().window.is_adaptive:
-            lists += [(Type.SEARCH, _("Search"), _("Search"))]
-            lists += [
-                (Type.CURRENT, _("Current playlist"), _("Current playlist"))]
-        if lists and\
-                App().settings.get_enum("sidebar-content") !=\
-                SidebarContent.DEFAULT:
-            lists.append((Type.SEPARATOR, "", ""))
-        return lists
-
-    def get_playlist_headers(self):
-        """
-            Return playlist headers
-            @return items as [(int, str)]
-        """
-        lists = ShownPlaylists.get()
-        if lists and\
-                App().settings.get_enum("sidebar-content") !=\
-                SidebarContent.DEFAULT:
-            lists.append((Type.SEPARATOR, "", ""))
-        return lists
+        if self.__base_mask & SelectionListMask.VIEW:
+            self.__fastscroll.clear()
+            self.__fastscroll.clear_chars()
 
     def select_first(self):
         """
             Select first available item
         """
         try:
-            self._listbox.unselect_all()
-            row = self._listbox.get_children()[0]
+            self._box.unselect_all()
+            row = self._box.get_children()[0]
+            self._box.select_row(row)
             row.activate()
         except Exception as e:
             Logger.warning("SelectionList::select_first(): %s", e)
 
-    def redraw(self):
+    def activate_child(self):
         """
-            Redraw list
+            Activated typeahead row
         """
-        for row in self._listbox.get_children():
-            row.set_artwork()
+        self._box.unselect_all()
+        for row in self._box.get_children():
+            style_context = row.get_style_context()
+            if style_context.has_class("typeahead"):
+                row.activate()
+            style_context.remove_class("typeahead")
 
     @property
-    def type_ahead_popover(self):
+    def filtered(self):
         """
-            Type ahead popover
-            @return TypeAheadPopover
+            Get filtered children
+            @return [Gtk.Widget]
         """
-        return self.__type_ahead_popover
+        filtered = []
+        for child in self._box.get_children():
+            if isinstance(child, SelectionListRow):
+                filtered.append(child)
+        return filtered
+
+    @property
+    def overlay(self):
+        """
+            Get list overlay
+            @return overlay as Gtk.Overlay
+        """
+        return self.__overlay
 
     @property
     def listbox(self):
@@ -450,15 +477,7 @@ class SelectionList(LazyLoadingView):
             Get listbox
             @return Gtk.ListBox
         """
-        return self._listbox
-
-    @property
-    def should_destroy(self):
-        """
-            True if view should be destroyed
-            @return bool
-        """
-        return False
+        return self._box
 
     @property
     def mask(self):
@@ -466,7 +485,11 @@ class SelectionList(LazyLoadingView):
             Get selection list type
             @return bit mask
         """
-        return self.__mask
+        return self.__mask | self.__base_mask
+
+    @property
+    def args(self):
+        return None
 
     @property
     def count(self):
@@ -474,27 +497,108 @@ class SelectionList(LazyLoadingView):
             Get items count in list
             @return int
         """
-        return len(self._listbox.get_children())
+        return len(self._box.get_children())
 
     @property
     def selected_ids(self):
         """
             Get selected ids
-            @return array of ids as [int]
+            @return [int]
         """
-        return [row.id for row in self._listbox.get_selected_rows()]
+        return [row.id for row in self._box.get_selected_rows()]
+
+    @property
+    def selected_id(self):
+        """
+            Get selected id
+            @return int
+        """
+        selected_row = self._box.get_selected_row()
+        return None if selected_row is None else selected_row.id
 
 #######################
-# PRIVATE             #
+# PROTECTED           #
 #######################
-    def __scroll_to_row(self, row):
+    def _scroll_to_child(self, row):
         """
             Scroll to row
             @param row as SelectionListRow
         """
-        coordinates = row.translate_coordinates(self._listbox, 0, 0)
+        coordinates = row.translate_coordinates(self._box, 0, 0)
         if coordinates:
-            self._scrolled.get_vadjustment().set_value(coordinates[1])
+            self.__scrolled.get_vadjustment().set_value(coordinates[1])
+
+    def _on_adaptive_changed(self, window, status):
+        """
+            Update internals
+            @param window as Window
+            @param status as bool
+        """
+        self.__base_mask &= ~(SelectionListMask.LABEL |
+                              SelectionListMask.ELLIPSIZE)
+        if status or self.mask & SelectionListMask.VIEW:
+            self.__base_mask |= (SelectionListMask.LABEL |
+                                 SelectionListMask.ELLIPSIZE)
+        elif App().settings.get_value("show-sidebar-labels"):
+            self.__base_mask |= SelectionListMask.LABEL
+        self.__set_rows_mask(self.__base_mask)
+
+    def _on_map(self, widget):
+        """
+            Unselect all if adaptive
+            @param widget as Gtk.Widget
+        """
+        LazyLoadingView._on_map(self, widget)
+        if App().window.is_adaptive:
+            self._box.unselect_all()
+
+    def _on_primary_long_press_gesture(self, x, y):
+        """
+            Show row menu
+            @param x as int
+            @param y as int
+        """
+        self.__popup_menu(y)
+
+    def _on_primary_press_gesture(self, x, y, event):
+        """
+            Activate current row
+            @param x as int
+            @param y as int
+            @param event as Gdk.Event
+        """
+        row = self._box.get_row_at_y(y)
+        if row is not None:
+            (exists, state) = event.get_state()
+            if state & Gdk.ModifierType.CONTROL_MASK or\
+                    state & Gdk.ModifierType.SHIFT_MASK:
+                pass
+            else:
+                self._box.unselect_all()
+
+    def _on_secondary_press_gesture(self, x, y, event):
+        """
+            Show row menu
+            @param x as int
+            @param y as int
+            @param event as Gdk.Event
+        """
+        self.__popup_menu(y)
+
+#######################
+# PRIVATE             #
+#######################
+    def __set_rows_mask(self, mask):
+        """
+            Show labels on child
+            @param status as bool
+        """
+        for row in self._box.get_children():
+            row.set_mask(mask)
+        if mask & SelectionListMask.ELLIPSIZE:
+            self.__scrolled.set_hexpand(True)
+        else:
+            self.__scrolled.set_hexpand(False)
 
     def __add_values(self, values):
         """
@@ -507,14 +611,14 @@ class SelectionList(LazyLoadingView):
             self._lazy_queue.append(row)
             GLib.idle_add(self.__add_values, values)
         else:
-            if self.__mask & SelectionListMask.ARTISTS:
+            if self.mask & SelectionListMask.ARTISTS:
                 self.__fastscroll.populate()
             self.__sort = True
-            self.emit("populated")
+            emit_signal(self, "populated")
             self.lazy_loading()
             # Scroll to first selected item
-            for row in self._listbox.get_selected_rows():
-                GLib.idle_add(self.__scroll_to_row, row)
+            for row in self._box.get_selected_rows():
+                GLib.idle_add(self._scroll_to_child, row)
                 break
 
     def __add_value(self, rowid, name, sortname):
@@ -525,13 +629,13 @@ class SelectionList(LazyLoadingView):
             @param sortname as str
             @return row as SelectionListRow
         """
-        if rowid > 0 and self.__mask & SelectionListMask.ARTISTS:
+        if rowid > 0 and self.mask & SelectionListMask.ARTISTS:
             used = sortname if sortname else name
             self.__fastscroll.add_char(used[0])
         row = SelectionListRow(rowid, name, sortname,
-                               self.__mask, self.__height)
+                               self.mask, self.__height)
         row.show()
-        self._listbox.add(row)
+        self._box.add(row)
         return row
 
     def __sort_func(self, row_a, row_b):
@@ -556,7 +660,7 @@ class SelectionList(LazyLoadingView):
             return False
         # String comparaison for non static
         else:
-            if self.__mask & SelectionListMask.ARTISTS:
+            if self.mask & SelectionListMask.ARTISTS:
                 a = row_a.sortname
                 b = row_b.sortname
             else:
@@ -564,93 +668,50 @@ class SelectionList(LazyLoadingView):
                 b = row_b.name
             return strcoll(a, b)
 
-    def __on_key_press_event(self, listbox, event):
+    def __popup_menu(self, y=None, row=None):
         """
-            Pass focus as signal
-            @param listbox as Gtk.ListBox
-            @param event as Gdk.Event
+            Show menu at y or row
+            @param y as int
+            @param relative as Gtk.Widget
         """
-        if event.keyval in [Gdk.KEY_Left, Gdk.KEY_Right]:
-            self.emit("pass-focus")
-
-    def __on_button_release_event(self, listbox, event):
-        """
-            Handle modifier
-            @param listbox as Gtk.ListBox
-            @param event as Gdk.Event
-        """
-        if event.button != 1 and\
-                self.__base_type in [SelectionListMask.LIST_ONE,
-                                     SelectionListMask.LIST_TWO]:
+        if self.__base_mask & SelectionListMask.SIDEBAR:
             from lollypop.menu_selectionlist import SelectionListMenu
-            from lollypop.widgets_utils import Popover
-            row = listbox.get_row_at_y(event.y)
-            if row is not None:
-                menu = SelectionListMenu(self, row.id, self.mask)
-                popover = Popover()
-                popover.bind_model(menu, None)
-                popover.set_relative_to(listbox)
-                rect = Gdk.Rectangle()
-                rect.x = event.x
-                rect.y = event.y
-                rect.width = rect.height = 1
-                popover.set_pointing_to(rect)
-                popover.popup()
-                return True
-        elif event.button == 1:
-            state = event.get_state()
-            static_selected = self.selected_ids and self.selected_ids[0] < 0
-            if (not state & Gdk.ModifierType.CONTROL_MASK and
-                    not state & Gdk.ModifierType.SHIFT_MASK) or\
-                    static_selected:
-                listbox.set_selection_mode(Gtk.SelectionMode.SINGLE)
-            row = listbox.get_row_at_y(event.y)
-            if row is not None and not (row.id < 0 and self.selected_ids):
-                # User clicked on random, clear cached one
-                if row.id == Type.RANDOMS:
-                    App().albums.clear_cached_randoms()
-                    App().tracks.clear_cached_randoms()
-            listbox.set_selection_mode(Gtk.SelectionMode.MULTIPLE)
+            from lollypop.widgets_menu import MenuBuilder
+            if row is None:
+                row = self._box.get_row_at_y(y)
+                if row is None:
+                    return
+                row_id = row.id
+            else:
+                row_id = None
+            menu = SelectionListMenu(self,
+                                     row_id,
+                                     self.mask,
+                                     App().window.is_adaptive)
+            menu_widget = MenuBuilder(menu)
+            menu_widget.show()
+            popup_widget(menu_widget, row)
 
-    def __on_artist_artwork_changed(self, art, artist):
+    def __on_artist_artwork_changed(self, object, value):
         """
-            Update row
-            @param art as Art
-            @param artist as str
+            Update row artwork
+            @param object as GObject.Object
+            @param value as str
         """
-        if self.__mask & SelectionListMask.ARTISTS:
-            for row in self._listbox.get_children():
-                if row.id >= 0 and row.name == artist:
+        artist = value if object == App().art else None
+        if self.mask & SelectionListMask.ARTISTS:
+            for row in self._box.get_children():
+                if artist is None:
+                    row.set_style(self.__height)
+                    row.set_artwork()
+                elif row.name == artist:
                     row.set_artwork()
                     break
 
-    def __on_type_ahead_activate(self, entry):
+    def __on_show_sidebar_labels_changed(self, settings, value):
         """
-            Close popover and activate row
-            @param entry as Gtk.Entry
+            Show/hide labels
+            @param settings as Gio.Settings
+            @param value as str
         """
-        self._listbox.unselect_all()
-        self.__type_ahead_popover.popdown()
-        for row in self._listbox.get_children():
-            style_context = row.get_style_context()
-            if style_context.has_class("typeahead"):
-                row.activate()
-            style_context.remove_class("typeahead")
-
-    def __on_type_ahead_changed(self, entry):
-        """
-            Search row and scroll down
-            @param entry as Gtk.Entry
-        """
-        search = entry.get_text().lower()
-        for row in self._listbox.get_children():
-            style_context = row.get_style_context()
-            style_context.remove_class("typeahead")
-        if not search:
-            return
-        for row in self._listbox.get_children():
-            if row.name.lower().find(search) != -1:
-                style_context = row.get_style_context()
-                style_context.add_class("typeahead")
-                GLib.idle_add(self.__scroll_to_row, row)
-                break
+        self._on_adaptive_changed(App().window, App().window.is_adaptive)
