@@ -49,7 +49,7 @@ class Discoverer:
         return info
 
 
-class TagReader(Discoverer):
+class TagReader:
     """
         Scanner tag reader
     """
@@ -58,7 +58,7 @@ class TagReader(Discoverer):
         """
             Init tag reader
         """
-        Discoverer.__init__(self)
+        pass
 
     def get_title(self, tags, filepath):
         """
@@ -592,6 +592,7 @@ class TagReader(Discoverer):
             @commit needed
         """
         artist_ids = []
+        added_artist_ids = []
         artistsplit = artists.split(";")
         sortsplit = sortnames.split(";")
         sortlen = len(sortsplit)
@@ -609,7 +610,7 @@ class TagReader(Discoverer):
                 else:
                     mbid = mbidsplit[i].strip()
                 # Get artist id, add it if missing
-                artist_id = App().artists.get_id(artist, mbid)
+                (artist_id, db_name) = App().artists.get_id(artist, mbid)
                 if i >= sortlen or sortsplit[i] == "":
                     sortname = None
                 else:
@@ -618,14 +619,19 @@ class TagReader(Discoverer):
                     if sortname is None:
                         sortname = format_artist_name(artist)
                     artist_id = App().artists.add(artist, sortname, mbid)
+                    added_artist_ids.append(artist_id)
                 else:
+                    # artists.get_id() is NOCASE, check if we need to update
+                    # artist name
+                    if db_name != artist:
+                        App().artists.set_name(artist_id, artist)
                     if sortname is not None:
                         App().artists.set_sortname(artist_id, sortname)
                     if mbid is not None:
                         App().artists.set_mb_artist_id(artist_id, mbid)
                 i += 1
                 artist_ids.append(artist_id)
-        return artist_ids
+        return (added_artist_ids, artist_ids)
 
     def add_genres(self, genres):
         """
@@ -647,7 +653,7 @@ class TagReader(Discoverer):
         return genre_ids
 
     def add_album(self, album_name, mb_album_id, artist_ids,
-                  uri, loved, popularity, rate, synced, mtime):
+                  uri, loved, popularity, rate, synced, mtime, storage_type):
         """
             Add album to db
             @param album_name as str
@@ -659,6 +665,7 @@ class TagReader(Discoverer):
             @param rate as int
             @param synced as int
             @param mtime as int
+            @param storage_type as StorageType
             @return (added as bool, album_id as int)
             @commit needed
         """
@@ -671,7 +678,7 @@ class TagReader(Discoverer):
             added = True
             album_id = App().albums.add(album_name, mb_album_id, artist_ids,
                                         parent_uri, loved, popularity,
-                                        rate, synced, mtime)
+                                        rate, synced, mtime, storage_type)
         # Now we have our album id, check if path doesn"t change
         if App().albums.get_uri(album_id) != parent_uri:
             App().albums.set_uri(album_id, parent_uri)
