@@ -10,7 +10,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gio, GLib, Gdk, GdkPixbuf, Pango
+from gi.repository import Gio, GLib, Gdk, GdkPixbuf, Pango, Gtk
 
 from math import pi
 from gettext import gettext as _
@@ -18,10 +18,21 @@ from urllib.parse import urlparse
 import unicodedata
 import cairo
 import time
+from threading import current_thread
 from functools import wraps
 
 from lollypop.logger import Logger
-from lollypop.define import App, Type, SelectionListMask, NetworkAccessACL
+from lollypop.define import App, Type, NetworkAccessACL
+from lollypop.define import StorageType
+
+
+def cancellable_sleep(seconds, cancellable):
+    if not App().debug:
+        while seconds > 0:
+            time.sleep(0.1)
+            seconds -= 0.1
+            if cancellable.is_cancelled():
+                raise Exception("cancelled")
 
 
 def seconds_to_string(duration):
@@ -94,17 +105,30 @@ def get_round_surface(image, scale_factor, radius):
     return surface
 
 
-def on_realize(widget):
+def set_cursor_type(widget, name="pointer"):
     """
         Set cursor on widget
         @param widget as Gtk.Widget
+        @param name as str
     """
     try:
         window = widget.get_window()
         if window is not None:
-            window.set_cursor(Gdk.Cursor(Gdk.CursorType.HAND2))
+            cursor = Gdk.Cursor.new_from_name(Gdk.Display.get_default(),
+                                              name)
+            window.set_cursor(cursor)
     except:
-        Logger.warning(_("You are using a broken cursor theme!"))
+        pass
+
+
+def get_default_storage_type():
+    """
+        Get default collection storage type check
+    """
+    if get_network_available("YOUTUBE"):
+        return StorageType.COLLECTION | StorageType.SAVED
+    else:
+        return StorageType.COLLECTION
 
 
 def on_query_tooltip(label, x, y, keyboard, tooltip):
@@ -122,18 +146,6 @@ def on_query_tooltip(label, x, y, keyboard, tooltip):
         return True
 
 
-def get_headerbar_buttons_width():
-    """
-        Get headerbar buttons width
-        @return int
-    """
-    adwaita_width = 36
-    settings = Gio.Settings.new("org.gnome.desktop.wm.preferences")
-    button_layout = settings.get_value("button-layout").get_string()
-    split = button_layout.split(",")
-    return len(split) * adwaita_width
-
-
 def set_proxy_from_gnome():
     """
         Set proxy settings from GNOME
@@ -195,9 +207,39 @@ def get_network_available(acl_name=""):
     return False
 
 
+def do_shift_selection(listbox, row):
+    """
+        Do a shift selection on a listbox
+    """
+    # Get last selected row
+    start_row = None
+    children = listbox.get_children()
+    selected_rows = listbox.get_selected_rows()
+    if selected_rows:
+        start_row = selected_rows[-1]
+    elif children:
+        start_row = children[0]
+    if start_row is not None:
+        start_index = children.index(start_row)
+        end_index = children.index(row)
+        for i in range(start_index, end_index):
+            listbox.select_row(children[i])
+
+
+def split_list(l, n=1):
+    """
+        Split list in n parts
+        @param l as []
+        @param n as int
+    """
+    length = len(l)
+    split = [l[i * length // n: (i + 1) * length // n] for i in range(n)]
+    return [l for l in split if l]
+
+
 def noaccents(string):
     """
-        Return string without accents
+        Return string without accents lowered
         @param string as str
         @return str
     """
@@ -206,6 +248,19 @@ def noaccents(string):
     return v.lower()
 
 
+def sql_escape(string):
+    """
+        Escape string for SQL request
+        @param string as str
+        @param ignore as [str]
+    """
+    nfkd_form = unicodedata.normalize("NFKD", string)
+    v = u"".join([c for c in nfkd_form if not unicodedata.combining(c)])
+    return "".join([c for c in v if
+                    c.isalpha() or
+                    c.isdigit()]).rstrip().lower()
+
+
 def escape(str, ignore=["_", "-", " ", "."]):
     """
         Escape string
@@ -313,6 +368,19 @@ def format_artist_name(name):
     return name
 
 
+def emit_signal(obj, signal, *args):
+    """
+        Emit signal
+        @param obj as GObject.Object
+        @param signal as str
+        @thread safe
+    """
+    if current_thread().getName() == "MainThread":
+        obj.emit(signal, *args)
+    else:
+        GLib.idle_add(obj.emit, signal, *args)
+
+
 def translate_artist_name(name):
     """
         Return translate formated artist name
@@ -324,18 +392,45 @@ def translate_artist_name(name):
     return name
 
 
-def get_position_list(items, position):
+def tracks_to_albums(tracks):
     """
-        Return a list with item and position
-        @param items as []
-        @param position as int
-        @return []
+        Convert tracks list to albums list
     """
-    _items = []
-    for item in items:
-        _items.append((item, position))
-        position += 1
-    return _items
+    albums = []
+    for track in tracks:
+        if albums and albums[-1].id == track.album.id:
+            albums[-1].append_track(track, False)
+        else:
+            album = track.album
+            album.set_tracks([track], False)
+            albums.append(album)
+    return albums
+
+
+def get_page_score(page_title, title, artist, album):
+    """
+        Calculate web page score
+        if page_title looks like (title, artist, album), score is lower
+        @return int/None
+    """
+    page_title = escape(page_title.lower(), [])
+    artist = escape(artist.lower(), [])
+    album = escape(album.lower(), [])
+    title = escape(title.lower(), [])
+    # YouTube page title should be at least as long as wanted title
+    if len(page_title) < len(title):
+        return -1
+    # Remove common word for a valid track
+    page_title = page_title.replace("official", "")
+    page_title = page_title.replace("video", "")
+    page_title = page_title.replace("audio", "")
+    # Remove artist name
+    page_title = page_title.replace(artist, "")
+    # Remove album name
+    page_title = page_title.replace(album, "")
+    # Remove title
+    page_title = page_title.replace(title, "")
+    return len(page_title)
 
 
 def is_readonly(uri):
@@ -386,14 +481,15 @@ def get_font_height():
     return int(layout.get_pixel_size()[1])
 
 
-def get_icon_name(object_id, type=SelectionListMask.ARTISTS):
+def get_icon_name(object_id):
     """
         Return icon name for id
         @param object_id as int
-        @param type as SelectionListMask
     """
     icon = ""
-    if object_id == Type.POPULARS:
+    if object_id == Type.SUGGESTIONS:
+        icon = "org.gnome.Lollypop-suggestions-symbolic"
+    elif object_id == Type.POPULARS:
         icon = "starred-symbolic"
     elif object_id == Type.PLAYLISTS:
         icon = "emblem-documents-symbolic"
@@ -401,31 +497,112 @@ def get_icon_name(object_id, type=SelectionListMask.ARTISTS):
         icon = "media-optical-cd-audio-symbolic"
     elif object_id == Type.ARTISTS:
         icon = "avatar-default-symbolic"
+    elif object_id == Type.ARTISTS_LIST:
+        icon = "org.gnome.Lollypop-artists-list-symbolic"
     elif object_id == Type.COMPILATIONS:
         icon = "system-users-symbolic"
     elif object_id == Type.RECENTS:
         icon = "document-open-recent-symbolic"
     elif object_id == Type.RADIOS:
-        icon = "audio-input-microphone-symbolic"
+        icon = "org.gnome.Lollypop-gradio-symbolic"
     elif object_id == Type.RANDOMS:
         icon = "media-playlist-shuffle-symbolic"
     elif object_id == Type.LOVED:
         icon = "emblem-favorite-symbolic"
     elif object_id == Type.NEVER:
-        icon = "audio-speakers-symbolic"
+        icon = "org.gnome.Lollypop-unplayed-albums-symbolic"
     elif object_id == Type.YEARS:
         icon = "x-office-calendar-symbolic"
     elif object_id == Type.CURRENT:
-        icon = "view-list-symbolic"
+        icon = "org.gnome.Lollypop-play-queue-symbolic"
+    elif object_id == Type.LYRICS:
+        icon = "audio-input-microphone-symbolic"
     elif object_id == Type.SEARCH:
         icon = "edit-find-symbolic"
     elif object_id == Type.GENRES:
-        icon = "folder-symbolic"
+        icon = "org.gnome.Lollypop-tag-symbolic"
+    elif object_id == Type.GENRES_LIST:
+        icon = "org.gnome.Lollypop-tag-list-symbolic"
     elif object_id == Type.WEB:
         icon = "goa-panel-symbolic"
     return icon
 
 
+def update_button(button, style, icon_size, icon_name):
+    """
+        Update button style
+        @param button as Gtk.Button
+        @param style as str
+        @param icon_size as Gtk.Icon_Size
+        @param icon_name as str
+    """
+    context = button.get_style_context()
+    context.remove_class("menu-button-48")
+    context.remove_class("menu-button")
+    context.add_class(style)
+    button.get_image().set_from_icon_name(icon_name, icon_size)
+
+
+def popup_widget(widget, parent, x=None, y=None):
+    """
+        Popup menu on widget as x, y
+        @param widget as Gtk.Widget
+        @param parent as Gtk.Widget
+        @param x as int
+        @param y as int
+        @return Gtk.Popover/None
+    """
+    def on_hidden(widget, hide, popover):
+        popover.popdown()
+
+    if App().window.is_adaptive:
+        App().window.container.show_menu(widget)
+        return None
+    else:
+        from lollypop.widgets_utils import Popover
+        popover = Popover()
+        popover.add(widget)
+        widget.connect("hidden", on_hidden, popover)
+        popover.set_relative_to(parent)
+        if x is not None and y is not None:
+            rect = Gdk.Rectangle()
+            rect.x = x
+            rect.y = y
+            rect.width = rect.height = 1
+            popover.set_pointing_to(rect)
+        popover.set_position(Gtk.PositionType.BOTTOM)
+        popover.popup()
+        return popover
+
+
+def update_track_indexes(listbox, start_index, end_index):
+    """
+        Update track number from start_index + 1 to end_index
+        start_index is a valid track number
+        @param listbox as Gtk.ListBox
+        @param start_index as int
+        @param end_index as int
+    """
+    if start_index == 0:
+        current_number = 0
+    else:
+        current_number = None
+    children = listbox.get_children()
+    if end_index == -1:
+        end_index = len(children)
+    for album_row in children[start_index:end_index]:
+        for track in album_row.album.tracks:
+            if current_number is None:
+                current_number = track.number
+                continue
+            current_number += 1
+            track.set_number(current_number)
+        # Update widget for revealed albums
+        if album_row.revealed:
+            for track_row in album_row.children:
+                track_row.update_number_label()
+
+
 def is_device(mount):
     """
         True if mount is a Lollypop device
@@ -465,6 +642,33 @@ def profile(f):
     return wrapper
 
 
+def install_youtube_dl():
+    try:
+        path = GLib.get_user_data_dir() + "/lollypop/python"
+        argv = ["pip3", "install", "-t", path, "-U", "youtube-dl"]
+        GLib.spawn_sync(None, argv, [], GLib.SpawnFlags.SEARCH_PATH, None)
+    except Exception as e:
+        Logger.error("install_youtube_dl: %s" % e)
+
+
+def get_youtube_dl():
+    """
+        Get youtube-dl path and env
+        @return (str, [])
+    """
+    if App().settings.get_value("recent-youtube-dl"):
+        python_path = GLib.get_user_data_dir() + "/lollypop/python"
+        path = "%s/bin/youtube-dl" % python_path
+        env = ["PYTHONPATH=%s" % python_path]
+        f = Gio.File.new_for_path(path)
+        if f.query_exists():
+            return (path, env)
+    if GLib.find_program_in_path("youtube-dl"):
+        return ("youtube-dl", [])
+    else:
+        return (None, [])
+
+
 # From eyeD3 start
 # eyeD3 is written and maintained by:
 # Travis Shirk <travis@pobox.com>
