@@ -11,7 +11,7 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 from lollypop.view import View
-from lollypop.define import App
+from lollypop.define import ViewType
 from lollypop.widgets_equalizer import EqualizerWidget
 
 
@@ -24,7 +24,7 @@ class EqualizerView(View):
         """
             Init view
         """
-        View.__init__(self)
+        View.__init__(self, ViewType.SCROLLED)
         widget = EqualizerWidget()
         widget.show()
         self.add(widget)
@@ -32,19 +32,6 @@ class EqualizerView(View):
 ##############
 # PROTECTED  #
 ##############
-    def _on_map(self, widget):
-        """
-            Show buttons
-        """
-        App().window.emit("show-can-go-back", True)
-        App().window.emit("can-go-back-changed", True)
-
-    def _on_unmap(self, widget):
-        """
-            Destroy self
-            @param widget as Gtk.Widget
-        """
-        self.destroy_later()
 
 ############
 # PRIVATE  #
