@@ -16,7 +16,7 @@ from gi.repository import Gtk
 from lollypop.widgets_playlist_smart import SmartPlaylistRow
 from lollypop.view import View
 from lollypop.logger import Logger
-from lollypop.define import App
+from lollypop.define import App, ViewType
 
 
 class SmartPlaylistView(View):
@@ -29,16 +29,14 @@ class SmartPlaylistView(View):
             Init PlaylistView
             @param playlist_id as int
         """
-        View.__init__(self)
+        View.__init__(self, ViewType.SCROLLED)
         self.__playlist_id = playlist_id
         self.__size_group = Gtk.SizeGroup()
         self.__size_group.set_mode(Gtk.SizeGroupMode.BOTH)
         builder = Gtk.Builder()
         builder.add_from_resource("/org/gnome/Lollypop/SmartPlaylistView.ui")
-        builder.connect_signals(self)
         widget = builder.get_object("widget")
         self.connect("size-allocate", self.__on_size_allocate, widget)
-        self._viewport.add(widget)
         self.__listbox = builder.get_object("listbox")
         self._scrolled.set_property("expand", True)
         self.__match_toggle = builder.get_object("match_toggle")
@@ -50,7 +48,9 @@ class SmartPlaylistView(View):
         self.__bottom_box = builder.get_object("bottom_box")
         if App().playlists.get_smart(playlist_id):
             self.__match_toggle.set_active(True)
-        self.add(self._scrolled)
+            self.__set_sensitive(True)
+        self.add_widget(widget)
+        builder.connect_signals(self)
 
     def populate(self):
         """
@@ -87,9 +87,6 @@ class SmartPlaylistView(View):
             split_order = sql.split("ORDER BY")
             split_spaces = split_order[1].split(" ")
             orderby = split_spaces[1]
-            # UNION does not support RANDOM() in OrderBy
-            if orderby == "rand":
-                orderby = "random()"
             if split_spaces[2] in ["ASC", "DESC"]:
                 orderby += " %s" % split_spaces[2]
             self.__select_combobox.set_active_id(orderby)
@@ -97,6 +94,14 @@ class SmartPlaylistView(View):
             self.__select_combobox.set_active(0)
             Logger.warning("SmartPlaylistView::populate: %s", e)
 
+    @property
+    def args(self):
+        """
+            Get default args for __class__
+            @return {}
+        """
+        return {"playlist_id": self.__playlist_id}
+
 #######################
 # PROTECTED           #
 #######################
@@ -111,9 +116,7 @@ class SmartPlaylistView(View):
             if child.sql is None:
                 continue
             request += "SELECT DISTINCT(tracks.rowid)"
-            if orderby == "random()":
-                request += ", random() as rand"
-            else:
+            if orderby != "random()":
                 request += ", %s" % orderby
             request += " FROM tracks"
             subrequest = " WHERE %s" % child.sql
@@ -128,16 +131,13 @@ class SmartPlaylistView(View):
                 request += ", %s" % "artists"
             if subrequest.find(" albums.") != -1:
                 request += ", %s" % "albums"
-            if subrequest.find(" track_genres.") != -1:
-                request += ", %s" % " track_genres"
+            if subrequest.find(" album_genres.") != -1:
+                request += ", %s" % " album_genres"
             if subrequest.find(" track_artists.") != -1:
                 request += ", %s" % " track_artists"
             request += subrequest + " UNION "
         request = request[:-7]  # " UNION "
-        if orderby == "random()":
-            request += " ORDER BY rand"
-        else:
-            request += " ORDER BY %s" % orderby
+        request += " ORDER BY %s" % orderby
         request += " LIMIT %s" % int(self.__limit_spin.get_value())
         return request
 
@@ -186,8 +186,7 @@ class SmartPlaylistView(View):
         else:
             request = self.__get_or_request()
         App().playlists.set_smart_sql(self.__playlist_id, request)
-        # FIXME
-        App().window.container.reload_view()
+        App().window.container.go_back()
 
     def _on_add_rule_button_clicked(self, button):
         """
@@ -201,17 +200,9 @@ class SmartPlaylistView(View):
             Enable/Disable smart playlist
             @param button as GtkButton
         """
-        App().playlists.set_smart(self.__playlist_id, button.get_active())
-        if button.get_active():
-            self.__up_box.set_sensitive(True)
-            self.__bottom_box.set_sensitive(True)
-            self.__add_rule_button.set_sensitive(True)
-            self.__listbox.set_sensitive(True)
-        else:
-            self.__up_box.set_sensitive(False)
-            self.__bottom_box.set_sensitive(False)
-            self.__add_rule_button.set_sensitive(False)
-            self.__listbox.set_sensitive(False)
+        active = button.get_active()
+        App().playlists.set_smart(self.__playlist_id, active)
+        self.__set_sensitive(active)
 
     def _on_map(self, widget):
         """
@@ -220,9 +211,6 @@ class SmartPlaylistView(View):
         """
         View._on_map(self, widget)
         App().enable_special_shortcuts(False)
-        if App().settings.get_value("show-sidebar"):
-            App().window.emit("can-go-back-changed", True)
-            App().window.emit("show-can-go-back", True)
 
     def _on_unmap(self, widget):
         """
@@ -243,6 +231,22 @@ class SmartPlaylistView(View):
         widget.show()
         self.__listbox.add(widget)
 
+    def __set_sensitive(self, sensitive):
+        """
+            Set view sensitive
+            @param sensitive as bool
+        """
+        if sensitive:
+            self.__up_box.set_sensitive(True)
+            self.__bottom_box.set_sensitive(True)
+            self.__add_rule_button.set_sensitive(True)
+            self.__listbox.set_sensitive(True)
+        else:
+            self.__up_box.set_sensitive(False)
+            self.__bottom_box.set_sensitive(False)
+            self.__add_rule_button.set_sensitive(False)
+            self.__listbox.set_sensitive(False)
+
     def __on_size_allocate(self, widget, allocation, child_widget):
         """
             Set child widget size
