@@ -10,59 +10,55 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, GLib
-
-from gettext import gettext as _
 from locale import strcoll
 
 from lollypop.view_flowbox import FlowBoxView
-from lollypop.define import App, Type, ViewType, SidebarContent
+from lollypop.define import App, Type, ViewType
+from lollypop.utils import popup_widget, tracks_to_albums
+from lollypop.objects_track import Track
 from lollypop.widgets_playlist_rounded import PlaylistRoundedWidget
+from lollypop.widgets_banner_playlists import PlaylistsBannerWidget
 from lollypop.shown import ShownPlaylists
+from lollypop.helper_signals import SignalsHelper, signals_map
 
 
-class PlaylistsManagerView(FlowBoxView):
+class PlaylistsManagerView(FlowBoxView, SignalsHelper):
     """
         Show playlists in a FlowBox
     """
 
-    def __init__(self, obj, view_type=ViewType.SCROLLED):
+    @signals_map
+    def __init__(self, view_type):
         """
             Init decade view
-            @param obj as Track/Album
             @param view_type as ViewType
         """
-        FlowBoxView.__init__(self, view_type)
-        self._empty_icon_name = "emblem-documents-symbolic"
-        self.__obj = obj
+        FlowBoxView.__init__(self, view_type |
+                             ViewType.SCROLLED | ViewType.OVERLAY)
         self.__signal_id = None
-        if not view_type & ViewType.DEVICES:
-            new_playlist_button = Gtk.Button(_("New playlist"))
-            new_playlist_button.connect("clicked",
-                                        self.__on_new_button_clicked)
-            new_playlist_button.set_property("halign", Gtk.Align.CENTER)
-            new_playlist_button.set_hexpand(True)
-            new_playlist_button.set_margin_top(5)
-            new_playlist_button.show()
-            self.insert_row(0)
-            self.attach(new_playlist_button, 0, 0, 1, 1)
+        self._empty_icon_name = "emblem-documents-symbolic"
+        self.__banner = PlaylistsBannerWidget(self)
+        self.__banner.show()
+        self.__banner.set_view_type(self._view_type)
+        self.add_widget(self._box, self.__banner)
         self._widget_class = PlaylistRoundedWidget
+        return [
+                (App().playlists, "playlists-changed", "_on_playlist_changed")
+        ]
 
-    def populate(self, items=None):
+    def populate(self):
         """
-            Populate items
-            @param items
+            Populate view
         """
-        if self.__obj is not None:
-            new_items = []
-            for item in App().playlists.get_ids():
-                if not App().playlists.get_smart(item):
-                    new_items.append(item)
-            items = new_items
-        elif items is None:
+        def on_load(items):
+            FlowBoxView.populate(self, items)
+
+        def load():
             items = [i[0] for i in ShownPlaylists.get()]
             items += App().playlists.get_ids()
-        FlowBoxView.populate(self, items)
+            return items
+
+        App().task_helper.run(load, callback=(on_load,))
 
     def add_value(self, item):
         """
@@ -74,7 +70,10 @@ class PlaylistsManagerView(FlowBoxView):
                 return
         # Setup sort on insert
         self._box.set_sort_func(self.__sort_func)
-        widget = PlaylistRoundedWidget(item[0], None, self._view_type)
+        from lollypop.utils import get_font_height
+        widget = PlaylistRoundedWidget(item[0],
+                                       self._view_type,
+                                       get_font_height())
         widget.populate()
         widget.show()
         self._box.insert(widget, -1)
@@ -89,6 +88,14 @@ class PlaylistsManagerView(FlowBoxView):
                 child.destroy()
                 break
 
+    @property
+    def args(self):
+        """
+            Get default args for __class__
+            @return {}
+        """
+        return {"view_type": self.view_type}
+
 #######################
 # PROTECTED           #
 #######################
@@ -98,83 +105,59 @@ class PlaylistsManagerView(FlowBoxView):
             Start lazy loading
             @param playlist_ids as [int]
         """
-        self._remove_placeholder()
-        widget = FlowBoxView._add_items(self, playlist_ids,
-                                        self.__obj,
-                                        self._view_type)
-        if widget is not None:
-            widget.connect("overlayed", self.on_overlayed)
+        FlowBoxView._add_items(self, playlist_ids, self._view_type)
 
-    def _on_map(self, widget):
+    def _on_child_activated(self, flowbox, child):
         """
-            Setup widget
+            Navigate into child
+            @param flowbox as Gtk.FlowBox
+            @param child as Gtk.FlowBoxChild
         """
-        FlowBoxView._on_map(self, widget)
-        self.__signal_id = App().playlists.connect("playlists-changed",
-                                                   self.__on_playlist_changed)
-        if self.__obj is None:
-            App().settings.set_value("state-one-ids",
-                                     GLib.Variant("ai", [Type.PLAYLISTS]))
-            App().settings.set_value("state-two-ids",
-                                     GLib.Variant("ai", []))
-            App().settings.set_value("state-three-ids",
-                                     GLib.Variant("ai", []))
+        App().window.container.show_view([Type.PLAYLISTS], child.data)
+
+    def _on_tertiary_press_gesture(self, x, y, event):
+        """
+            Play artist
+            @param x as int
+            @param y as int
+            @param event as Gdk.Event
+        """
+        child = self._box.get_child_at_pos(x, y)
+        if child is None or child.artwork is None:
+            return
+        track_ids = []
+        if child.data > 0 and App().playlists.get_smart(child.data):
+            request = App().playlists.get_smart_sql(child.data)
+            if request is not None:
+                track_ids = App().db.execute(request)
         else:
-            App().window.emit("can-go-back-changed", True)
-            App().window.emit("show-can-go-back", True)
+            track_ids = App().playlists.get_track_ids(child.data)
+        tracks = [Track(track_id) for track_id in track_ids]
+        albums = tracks_to_albums(tracks)
+        if albums:
+            App().player.play_album_for_albums(albums[0], albums)
 
-    def _on_unmap(self, widget):
-        """
-            Disconnect signal
-            @param widget as Gtk.Widget
-        """
-        if self.__signal_id is not None:
-            App().playlists.disconnect(self.__signal_id)
-            self.__signal_id = None
-
-    def _on_item_activated(self, flowbox, widget):
+    def _on_secondary_press_gesture(self, x, y, event):
         """
             Show Context view for activated album
-            @param flowbox as Gtk.Flowbox
-            @param widget as PlaylistRoundedWidget
+            @param x as int
+            @param y as int
+            @param event as Gdk.Event
         """
-        if not self._view_type & ViewType.SMALL and\
-                FlowBoxView._on_item_activated(self, flowbox, widget):
+        self._on_primary_long_press_gesture(x, y)
+
+    def _on_primary_long_press_gesture(self, x, y):
+        """
+            Show Context view for activated album
+            @param x as int
+            @param y as int
+        """
+        child = self._box.get_child_at_pos(x, y)
+        if child is None or child.artwork is None:
             return
-        show_sidebar = App().settings.get_value("show-sidebar")
-        sidebar_content = App().settings.get_enum("sidebar-content")
-        show_genres = sidebar_content == SidebarContent.GENRES
-        if not show_genres:
-            App().window.emit("show-can-go-back", True)
-            App().window.emit("can-go-back-changed", True)
-        if show_sidebar and show_genres and not App().window.is_adaptive:
-            App().window.container.list_two.select_ids([widget.data])
-        else:
-            App().window.container.show_view([Type.PLAYLISTS], [widget.data])
+        self.__popup_menu(child)
 
-#######################
-# PRIVATE             #
-#######################
-    def __sort_func(self, widget1, widget2):
-        """
-            Sort function
-            @param widget1 as PlaylistRoundedWidget
-            @param widget2 as PlaylistRoundedWidget
-        """
-        # Static vs static
-        if widget1.data < 0 and widget2.data < 0:
-            return widget1.data < widget2.data
-        # Static entries always on top
-        elif widget2.data < 0:
-            return True
-        # Static entries always on top
-        if widget1.data < 0:
-            return False
-        # String comparaison for non static
-        else:
-            return strcoll(widget1.name, widget2.name)
-
-    def __on_playlist_changed(self, playlists, playlist_id):
+    def _on_playlist_changed(self, playlists, playlist_id):
         """
             Update view based on playlist_id status
             @param playlists as Playlists
@@ -199,19 +182,79 @@ class PlaylistsManagerView(FlowBoxView):
                 if child.data == playlist_id:
                     child.destroy()
 
-    def __on_new_button_clicked(self, button):
+#######################
+# PRIVATE             #
+#######################
+    def __popup_menu(self, child):
         """
-            Add a new playlist
-            @param button as Gtk.Button
+            Popup menu for playlist
+            @param child as PlaylistRoundedWidget
         """
-        existing_playlists = []
-        for (playlist_id, name, sortname) in App().playlists.get():
-            existing_playlists.append(name)
+        from lollypop.widgets_menu import MenuBuilder
+        from lollypop.menu_playlist import PlaylistMenu, PlaylistMenuExt
+        menu = PlaylistMenu(child.data, App().window.is_adaptive)
+        menu_widget = MenuBuilder(menu)
+        if child.data >= 0:
+            menu_widget = MenuBuilder(menu)
+            main = menu_widget.get_child_by_name("main")
+            menu_ext = PlaylistMenuExt(child.data)
+            menu_ext.show()
+            main.add(menu_ext)
+        else:
+            menu_widget = MenuBuilder(menu)
+        menu_widget.show()
+        popup_widget(menu_widget, child)
 
-        # Search for an available name
-        count = 1
-        name = _("New playlist ") + str(count)
-        while name in existing_playlists:
-            count += 1
-            name = _("New playlist ") + str(count)
-        App().playlists.add(name)
+    def __sort_func(self, widget1, widget2):
+        """
+            Sort function
+            @param widget1 as PlaylistRoundedWidget
+            @param widget2 as PlaylistRoundedWidget
+        """
+        # Static vs static
+        if widget1.data < 0 and widget2.data < 0:
+            return widget1.data < widget2.data
+        # Static entries always on top
+        elif widget2.data < 0:
+            return True
+        # Static entries always on top
+        if widget1.data < 0:
+            return False
+        # String comparaison for non static
+        else:
+            return strcoll(widget1.name, widget2.name)
+
+
+class PlaylistsManagerDeviceView(PlaylistsManagerView):
+    """
+        Show playlists in a FlowBox
+    """
+
+    def __init__(self, index, view_type=ViewType.SCROLLED):
+        """
+            Init decade view
+            @param index as int
+            @param view_type as ViewType
+        """
+        PlaylistsManagerView.__init__(self, view_type)
+        self.__index = index
+        self._new_button.hide()
+
+    def populate(self):
+        """
+            Populate items
+            @param items
+        """
+        def on_load(items):
+            FlowBoxView.populate(self, items)
+
+        def load():
+            items = App().playlists.get_synced_ids(0)
+            items += App().playlists.get_synced_ids(self.__index)
+            return items
+
+        App().task_helper.run(load, callback=(on_load,))
+
+    @property
+    def args(self):
+        return None
