@@ -12,8 +12,8 @@
 
 from gi.repository import GLib, Gtk, Pango, GObject
 
-from lollypop.define import ArtSize, ViewType
-from lollypop.utils import on_query_tooltip, on_realize
+from lollypop.define import ArtSize, ViewType, MARGIN
+from lollypop.utils import on_query_tooltip, set_cursor_type
 
 
 class RoundedFlowBoxWidget(Gtk.FlowBoxChild):
@@ -22,7 +22,6 @@ class RoundedFlowBoxWidget(Gtk.FlowBoxChild):
     """
 
     __gsignals__ = {
-        "overlayed": (GObject.SignalFlags.RUN_FIRST, None, (bool,)),
         "populated": (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
@@ -40,39 +39,34 @@ class RoundedFlowBoxWidget(Gtk.FlowBoxChild):
         self._data = data
         self.__name = name
         self.__sortname = sortname
-        self.__filtered = False
         self.set_view_type(view_type)
         self._scale_factor = self.get_scale_factor()
-        self.set_property("halign", Gtk.Align.CENTER)
-        self.set_property("valign", Gtk.Align.CENTER)
-        self.connect("realize", on_realize)
 
     def populate(self):
         """
             Populate widget content
         """
-        grid = Gtk.Grid()
-        grid.set_orientation(Gtk.Orientation.VERTICAL)
-        self.__label = Gtk.Label.new()
-        self.__label.set_ellipsize(Pango.EllipsizeMode.END)
-        self.__label.set_property("halign", Gtk.Align.CENTER)
-        self.__label.set_property("has-tooltip", True)
-        self.__label.connect("query-tooltip", on_query_tooltip)
-        self.__label.set_markup(
+        self._grid = Gtk.Grid()
+        self._grid.set_row_spacing(MARGIN)
+        self._grid.set_orientation(Gtk.Orientation.VERTICAL)
+        self._grid.show()
+        self._label = Gtk.Label.new()
+        self._label.set_ellipsize(Pango.EllipsizeMode.END)
+        self._label.set_property("halign", Gtk.Align.CENTER)
+        self._label.set_property("has-tooltip", True)
+        self._label.connect("query-tooltip", on_query_tooltip)
+        self._label.set_markup(
             "<b>" + GLib.markup_escape_text(self.__name) + "</b>")
+        self._label.show()
         self._artwork = Gtk.Image.new()
+        self._artwork.connect("realize", set_cursor_type)
         self._artwork.set_size_request(self._art_size, self._art_size)
         self._artwork.show()
         self.set_artwork()
-        self._overlay = Gtk.Overlay()
-        self._overlay.add(self._artwork)
-        grid.add(self._overlay)
-        grid.add(self.__label)
-        self._widget = Gtk.EventBox()
-        self._widget.set_property("has-tooltip", True)
-        self._widget.add(grid)
-        self.add(self._widget)
-        self.show_all()
+        self._grid.add(self._artwork)
+        self._grid.add(self._label)
+        self._artwork.connect("realize", set_cursor_type)
+        self.add(self._grid)
 
     def set_view_type(self, view_type):
         """
@@ -81,8 +75,8 @@ class RoundedFlowBoxWidget(Gtk.FlowBoxChild):
         """
         self.__view_type = view_type
         if self.__view_type & ViewType.SMALL:
-            self._art_size = ArtSize.LARGE
-        elif self.__view_type & ViewType.MEDIUM:
+            self._art_size = ArtSize.MEDIUM
+        elif self.__view_type & ViewType.ADAPTIVE:
             self._art_size = ArtSize.BANNER
         else:
             self._art_size = ArtSize.BIG
@@ -100,16 +94,7 @@ class RoundedFlowBoxWidget(Gtk.FlowBoxChild):
             Rename widget
             @param name as str
         """
-        self.__label.set_markup("<b>" + GLib.markup_escape_text(name) + "</b>")
-
-    def set_filtered(self, b):
-        """
-            Set widget filtered
-            @param b as bool
-            @return bool (should be shown)
-        """
-        self.__filtered = b
-        return not b
+        self._label.set_markup("<b>" + GLib.markup_escape_text(name) + "</b>")
 
     def disable_artwork(self):
         """
@@ -150,21 +135,6 @@ class RoundedFlowBoxWidget(Gtk.FlowBoxChild):
         """
         return self._data
 
-    @property
-    def filter(self):
-        """
-            Current filter
-            @return str
-        """
-        return self.name.lower()
-
-    @property
-    def filtered(self):
-        """
-            True if filtered by parent
-        """
-        return self.__filtered
-
     @property
     def is_populated(self):
         """
@@ -172,6 +142,22 @@ class RoundedFlowBoxWidget(Gtk.FlowBoxChild):
         """
         return True
 
+    @property
+    def artwork(self):
+        """
+            Get album artwork
+            @return Gtk.Image
+        """
+        return self._artwork
+
+    @property
+    def artwork_name(self):
+        """
+            Get artwork name
+            return str
+        """
+        return self.name
+
 #######################
 # PROTECTED           #
 #######################
