@@ -12,69 +12,66 @@
 
 from gi.repository import Gtk, Pango, GObject
 
-from lollypop.define import App, ArtSize, Type, ArtBehaviour, ViewType
-from lollypop.utils import on_query_tooltip
-from lollypop.helper_overlay_radio import OverlayRadioHelper
+from lollypop.define import App, ArtSize, ArtBehaviour, ViewType, MARGIN_SMALL
+from lollypop.utils import on_query_tooltip, set_cursor_type, emit_signal
+from lollypop.objects_radio import Radio
 
 
-class RadioWidget(Gtk.FlowBoxChild, OverlayRadioHelper):
+class RadioWidget(Gtk.FlowBoxChild):
     """
         Widget with radio cover and title
     """
     __gsignals__ = {
         "populated": (GObject.SignalFlags.RUN_FIRST, None, ()),
-        "overlayed": (GObject.SignalFlags.RUN_FIRST, None, (bool,))
     }
 
-    def __init__(self, radio_id, radios, view_type, font_height):
+    def __init__(self, radio_id, view_type, font_height):
         """
             Init radio widget
             @param radio_id as int
-            @param radios as Radios
             @param view_type as ViewType
             @param font_height as int
         """
         Gtk.FlowBoxChild.__init__(self)
-        OverlayRadioHelper.__init__(self, radios)
-        self.__widget = None
-        self._artwork = None
+        self.__artwork = None
         self.__font_height = font_height
-        self._radio_id = radio_id
-        self.__radios = radios
-        self.__name = self.__radios.get_name(self._radio_id)
+        self._track = Radio(radio_id)
         self.__view_type = view_type
-        self._watch_loading = True
         self.set_view_type(view_type)
 
     def populate(self):
         """
             Init widget content
         """
-        if self._artwork is None:
-            self.__widget = Gtk.EventBox()
+        if self.__artwork is None:
             grid = Gtk.Grid()
+            grid.set_row_spacing(MARGIN_SMALL)
             grid.set_orientation(Gtk.Orientation.VERTICAL)
-            self._artwork = Gtk.Image.new()
+            grid.show()
+            self.__artwork = Gtk.Image.new()
+            self.__artwork.connect("realize", set_cursor_type)
+            self.__artwork.show()
             self.__label = Gtk.Label.new()
             self.__label.set_justify(Gtk.Justification.CENTER)
             self.__label.set_ellipsize(Pango.EllipsizeMode.END)
-            self.__label.set_text(self.__name)
+            self.__label.set_text(self._track.name)
+            self.__label.get_style_context().add_class("big-padding")
             self.__label.set_property("has-tooltip", True)
             self.__label.connect("query-tooltip", on_query_tooltip)
-            self.__widget.add(grid)
-            self._overlay = Gtk.Overlay.new()
-            self._overlay.add(self._artwork)
-            grid.add(self._overlay)
-            grid.add(self.__label)
-            self.add(self.__widget)
+            self.__label.show()
+            self.__spinner = Gtk.Spinner.new()
+            self.__spinner.get_style_context().add_class("big-padding")
+            self.__spinner.show()
+            self.__stack = Gtk.Stack.new()
+            self.__stack.show()
+            self.__stack.add(self.__label)
+            self.__stack.add(self.__spinner)
+            grid.add(self.__artwork)
+            grid.add(self.__stack)
+            self.add(grid)
             self.set_artwork()
             self.set_selection()
-            self.show_all()
-            self._lock_overlay = False
-            self.set_property("halign", Gtk.Align.CENTER)
-            self.set_property("valign", Gtk.Align.CENTER)
-            self.__widget.connect("enter-notify-event", self._on_enter_notify)
-            self.__widget.connect("leave-notify-event", self._on_leave_notify)
+            self.show()
         else:
             self.set_artwork()
 
@@ -82,28 +79,28 @@ class RadioWidget(Gtk.FlowBoxChild, OverlayRadioHelper):
         """
             Disable widget artwork
         """
-        if self._artwork is not None:
-            self._artwork.set_size_request(self.__art_size, self.__art_size)
-            self._artwork.set_from_surface(None)
+        if self.__artwork is not None:
+            self.__artwork.set_size_request(self.__art_size, self.__art_size)
+            self.__artwork.set_from_surface(None)
 
     def set_artwork(self):
         """
             Set artwork
         """
-        if self.__widget is None:
+        if self.__artwork is None:
             return
         if self.__art_size < ArtSize.BIG:
             frame = "small-cover-frame"
         else:
             frame = "cover-frame"
-        App().art_helper.set_frame(self._artwork,
+        App().art_helper.set_frame(self.__artwork,
                                    frame,
                                    self.__art_size,
                                    self.__art_size)
-        App().art_helper.set_radio_artwork(self.__name,
+        App().art_helper.set_radio_artwork(self._track.name,
                                            self.__art_size,
                                            self.__art_size,
-                                           self._artwork.get_scale_factor(),
+                                           self.__artwork.get_scale_factor(),
                                            ArtBehaviour.CACHE |
                                            ArtBehaviour.CROP,
                                            self.__on_radio_artwork)
@@ -115,20 +112,32 @@ class RadioWidget(Gtk.FlowBoxChild, OverlayRadioHelper):
         """
         self.__view_type = view_type
         if self.__view_type & ViewType.SMALL:
-            self.__art_size = ArtSize.LARGE
-        elif self.__view_type & ViewType.MEDIUM:
+            self.__art_size = ArtSize.MEDIUM
+        elif self.__view_type & ViewType.ADAPTIVE:
             self.__art_size = ArtSize.BANNER
         else:
             self.__art_size = ArtSize.BIG
         self.set_size_request(self.__art_size,
                               self.__art_size + self.__font_height)
 
+    def set_loading(self, loading):
+        """
+            Show spinner
+            @param loading as bool
+        """
+        if loading:
+            self.__spinner.start()
+            self.__stack.set_visible_child(self.__spinner)
+        else:
+            self.__spinner.stop()
+            self.__stack.set_visible_child(self.__label)
+
     def do_get_preferred_width(self):
         """
             Return preferred width
             @return (int, int)
         """
-        if self.__widget is None:
+        if self.__artwork is None:
             return (0, 0)
         width = Gtk.FlowBoxChild.do_get_preferred_width(self)[0]
         return (width, width)
@@ -138,21 +147,27 @@ class RadioWidget(Gtk.FlowBoxChild, OverlayRadioHelper):
             Set radio name
             @param name as str
         """
-        self.__name = name
         self.__label.set_label(name)
 
     def set_selection(self):
         """
             Mark widget as selected if currently playing
         """
-        if self._artwork is None:
+        if self.__artwork is None:
             return
-        selected = App().player.current_track.id == Type.RADIOS and\
-            self.__name == App().player.current_track.radio_name
+        selected = isinstance(App().player.current_track.id, Radio) and\
+            self._track.id == App().player.current_track.id
         if selected:
-            self._artwork.set_state_flags(Gtk.StateFlags.SELECTED, True)
+            self.__artwork.set_state_flags(Gtk.StateFlags.SELECTED, True)
         else:
-            self._artwork.set_state_flags(Gtk.StateFlags.NORMAL, True)
+            self.__artwork.set_state_flags(Gtk.StateFlags.NORMAL, True)
+
+    @property
+    def spinner(self):
+        """
+            Get radio spinner
+        """
+        return self.__spinner
 
     @property
     def is_populated(self):
@@ -163,29 +178,41 @@ class RadioWidget(Gtk.FlowBoxChild, OverlayRadioHelper):
         return True
 
     @property
-    def id(self):
+    def artwork(self):
         """
-            @return int
+            Get album artwork
+            @return Gtk.Image
         """
-        return self._radio_id
-
-    @property
-    def filter(self):
-        """
-            @return filter as str
-        """
-        return self.name.lower()
+        return self.__artwork
 
     @property
     def name(self):
         """
-            @return name as str
+            Get name
+            @return str
         """
-        return self.__name
+        return self.__label.get_text()
+
+    @property
+    def data(self):
+        """
+            Get track
+            @return int
+        """
+        return self._track
 
 #######################
 # PROTECTED           #
 #######################
+    def _on_loading_changed(self, player, status, track_id):
+        """
+            Show a spinner while loading
+            @param player as Player
+            @param status as bool
+            @param track_id as int
+        """
+        if track_id != self._track.id:
+            return
 
 #######################
 # PRIVATE             #
@@ -196,9 +223,9 @@ class RadioWidget(Gtk.FlowBoxChild, OverlayRadioHelper):
             @param surface as str
         """
         if surface is None:
-            self._artwork.set_from_icon_name("audio-input-microphone-symbolic",
+            self.__artwork.set_from_icon_name(
+                                             "audio-input-microphone-symbolic",
                                              Gtk.IconSize.DIALOG)
         else:
-            self._artwork.set_from_surface(surface)
-        self.show_all()
-        self.emit("populated")
+            self.__artwork.set_from_surface(surface)
+        emit_signal(self, "populated")
