@@ -225,6 +225,7 @@ class CollectionsSettingsWidget(Gtk.Bin):
             @param history as History
         """
         if uris:
+            App().stop_spotify()
             uri = uris.pop(0)
             App().scanner.del_from_db(uri, True, False)
             self.__progress.set_fraction((count - len(uris)) / count)
@@ -234,9 +235,7 @@ class CollectionsSettingsWidget(Gtk.Bin):
             App().player.stop()
             App().db.drop_db()
             App().db = Database()
-            App().window.container.list_two.hide()
-            App().window.container.stack.destroy_children()
-            App().window.container.update_list_one()
+            App().window.container.go_home()
             App().scanner.update(ScanType.FULL)
             self.__progress.get_toplevel().set_deletable(True)
 
